# FileName: atetool-update-v1.ps1
#=============================================================================
# Script Name: [ATE PLANNING TOOL update script]
# Created: [25/3/2013]
# Version: [1.0]
# Author: [Martin Kotlar]
# Email: [martin.kotlar@honeywell.com]
# Company: [Honeywell]  
# Arguments: None
# =============================================================================
# Purpose: This script is to be used to updated ATE Planning Tool to last
# version in the subversion repository
# =============================================================================

#=============================================================================
# FUNCTION LISTINGS
#=============================================================================
# Function: Decrypt-SecureString
# Arguments: String $sstr Secure string input texts
# =============================================================================
# Purpose: Decrypts secure string into plain text.
# =============================================================================
function Decrypt-SecureString {
  param(
      [Parameter(ValueFromPipeline=$true,Mandatory=$true,Position=0)]
      [System.Security.SecureString]
      $sstr
  )

  $marshal = [System.Runtime.InteropServices.Marshal]
  $ptr = $marshal::SecureStringToBSTR( $sstr )
  $str = $marshal::PtrToStringBSTR( $ptr )
  $marshal::ZeroFreeBSTR( $ptr )
  $str
} 

#=============================================================================
# Function: Replace-FileContent
# Arguments: 
#  String $inputFile Path to the input file to read/parse
#  String $outputFile Path to the output file to write replaced texts
#  String $pattern String for pattern matching (for String -match method)
#  String $replace String to replace pattern with
# =============================================================================
# Purpose: Reads $inputFile and replaces strings matching $pattern with $replace
#         string and writes it into $outputFile.
# =============================================================================
Function Replace-FileContent
{
  Param(
    [Parameter(Mandatory=$True,Position=1)]
     [string]$inputFile,
  	
     [Parameter(Mandatory=$True,Position=2)]
     [string]$outputFile,

     [Parameter(Mandatory=$True,Position=3)]
     [string]$pattern,

     [Parameter(Mandatory=$True,Position=4)]
     [string]$replace
  )
  Get-Content -Path $inputFile | ForEach-Object { 
      if ($_ -match $pattern)
      {
          $_ -replace $pattern, $replace
      }
      else
      {
          $_
      }
  } | Set-Content -Path $outputFile
}

#=============================================================================
# Function: AskView-FileContent
# Arguments: 
#  String $path Path to the file to view
#  String $question Text to show as question (without ending ?)
# =============================================================================
# Purpose: Ask whether user wants to see the file content and if confirmed
#           shows the content of the file using 'more' command. 
# =============================================================================
Function AskView-FileContent
{
  Param(
    [Parameter(Mandatory=$True,Position=1)]
     [string]$path,
  	
     [Parameter(Mandatory=$True,Position=2)]
     [string]$question
  )
  $my_confirm = "Undefined"
  do { $my_confirm = Read-Host ($question + " (use q to quit) [y/n]?") }
  until (($my_confirm -eq "y") -or ($my_confirm -eq "n"))
  if ($my_confirm -eq "y")
  { 
      echo "------  START OF FILE -----"
      cat $path | more 
      echo "------  END OF FILE -----"
  }
}

#=============================================================================
# Function: Set-PathWritable
# Arguments: 
#  String $path Path to the directory to set 
#  String $user Name of the user/group to set the path writable (Modify permission)
#  Boolean $cleanUp Whether to clean up inherited permissions
# =============================================================================
# Purpose: Adds Modify access rule to the list of ACL of the file/dir on the path. 
# =============================================================================
Function Set-PathWritable
{
  Param(
    [Parameter(Mandatory=$True)]
     [string]$path,
  	
     [Parameter(Mandatory=$True)]
     [string]$user,

     [Parameter(Mandatory=$False)]
     [bool]$cleanUp = $False
  )

  $acl = Get-Acl -Path $path
  if ($cleanUp)
  {
    $acl.SetAccessRuleProtection($True, $False)
  }    
  $rule = New-Object System.Security.AccessControl.FileSystemAccessRule($user, "Modify", "ContainerInherit, ObjectInherit", "None", "Allow")
  $acl.AddAccessRule($rule)
  Set-Acl -Path $path -AclObject $acl
}


$my_svn_base_url = 'https://aterepos.honeywell.com/svn/ATE_Tool/'
$my_iis_base_dir = "C:\inetpub\wwwroot\"
$my_svn_exec = 'c:\Program Files\TortoiseSVN\bin\svn.exe'
$my_backup_base = 'C:\Backup\ate-planning-tool\app\'
$my_iis_usr = "IUSR"   
#$my_iis_usr = "IIS_IUSRS"   

echo "======================================================"
echo "ATE Planning Tool automatic update from SVN repository"
echo "======================================================"
echo "To stop the script at any time: Ctrl+C"

do {
	echo '
Choose version of ATE Tool target:
1) production version (tags) http://wikiat.honeywell.com/projects/
2) production/test version (tags) http://wikiat.honeywell.com/atetool-testable/
3) development version (trunk) http://wikiat.honeywell.com/atetool-devel/
4) development/test version (trunk) http://wikiat.honeywell.com/atetool-testing/
5) B&P testing version (trunk) http://wikiat.honeywell.com/atetool-bap/'

	$my_target_selection = Read-Host -Prompt "Your choice"
	Switch ($my_target_selection)
    {
        1 
        { 
    		$my_target_dir="projects"
    		$my_db="tool"
    		$my_svn_dir="tags/1.3.0"
    	}
        2
        {
    		$my_target_dir="atetool-testable"
    		$my_db="tool-testable"
    		$my_svn_dir="tags/1.3.0"
        }
        3
        {
    		$my_target_dir="atetool-devel"
    		$my_db="tool-devel"
    		$my_svn_dir="trunk"
        }
        4
        {
          $my_target_dir="atetool-testing"
          $my_db="tool-testing"
          $my_svn_dir="trunk"
        }
        5
        {
    		$my_target_dir="atetool-bap"
    		$my_db="tool-bap"
    		$my_svn_dir="tags/1.3.0"
  		}
        default 
        {
          echo "Wrong input. Try again!"
          $my_target_selection = 0 
        }
    }
}
while (($my_target_selection -lt 1) -or ($my_target_selection -gt 5))

$my_target_path=$my_iis_base_dir + $my_target_dir
$my_temp_target=$my_target_path + "-temp-" + (Get-Random)
$my_svn_url=$my_svn_base_url + $my_svn_dir

echo "Target directory is: $my_target_path"
echo "Working directory is: $my_temp_target"

do {
  echo "
  Input Subversion repository credentials for $my_svn_url"
  $my_svn_user = Read-Host -Prompt "Username (email)"
  $my_svn_pass = Read-Host -Prompt "Password (not outputted)" -AsSecureString

  echo "Detailed information for svn repository:"
  & $my_svn_exec @("info","--non-interactive","--trust-server-cert","--no-auth-cache","--username","$my_svn_user","--password",(Decrypt-SecureString $my_svn_pass),"$my_svn_url")    
}
until ($LastExitCode -eq 0)

$my_start = "Undefined"
do {$my_start = Read-Host "Start operation? [y/n]"}
until (($my_start -eq "y") -or ($my_start -eq "n"))
if ($my_start -eq "n") { echo "Stopping script."; exit }
else { echo "Let's go!" }

echo "Exporting repository to $my_temp_target"
& $my_svn_exec @("export","--non-interactive","--trust-server-cert","--no-auth-cache","--username","$my_svn_user","--password",(Decrypt-SecureString $my_svn_pass),"$my_svn_url", "$my_temp_target")    

& $my_svn_exec @("info","--non-interactive","--trust-server-cert","--no-auth-cache","--username","$my_svn_user","--password",(Decrypt-SecureString $my_svn_pass),"$my_svn_url") > $my_temp_target\svn-export-info.txt     
$revision = cat $my_temp_target\svn-export-info.txt | Where { $_ -match "^Last Changed Rev:" }
$revision = $revision -replace "Last Changed Rev: ", ""
echo "Last applicable SVN Revision is $revision"
echo "Creating directories"
mkdir $my_temp_target\backup

echo "Updating user file permission on writable directories"
Set-PathWritable -Path $my_temp_target\log -User $my_iis_usr
Set-PathWritable -Path $my_temp_target\temp -User $my_iis_usr
Set-PathWritable -Path $my_temp_target\backup -User $my_iis_usr

#echo "Setting public directory permissions"
#Set-PathWritable -Path $my_temp_target\public -User $my_iis_usr
if (Test-Path $my_target_path\public)
{
	echo "Copying content of public directory:"
	Copy-Item $my_target_path\public -Destination $my_temp_target -Force -Recurse
}
if (Test-Path $my_target_path\data\actual)
{
	echo "Copying actual data to new instance"
	Copy-Item $my_target_path\data\actual -Destination $my_temp_target\data -Force -Recurse
}

echo "Changing database name in configuration file in app (Nette-based)"
Replace-FileContent -inputFile $my_temp_target\app\config.neon -outputFile $my_temp_target\app\config.neon.new -pattern "database: tool" -replace "database: $my_db"
Rename-Item $my_temp_target\app\config.neon $my_temp_target\app\config.neon.orig
Rename-Item $my_temp_target\app\config.neon.new $my_temp_target\app\config.neon
AskView-FileContent -Path $my_temp_target\app\config.neon -Question "View app config.neon" 

echo "Changing database name in configuration file in app2"
Replace-FileContent -inputFile $my_temp_target\app2\config\config-mysql.php -outputFile $my_temp_target\app2\config\config-mysql.php.new -pattern 'define\(''mysqldbname'',"tool"\);' -replace "define('mysqldbname',""$my_db"");"
Rename-Item $my_temp_target\app2\config\config-mysql.php $my_temp_target\app2\config\config-mysql.php.orig
Rename-Item $my_temp_target\app2\config\config-mysql.php.new $my_temp_target\app2\config\config-mysql.php
AskView-FileContent -Path $my_temp_target\app2\config\config-mysql.php -Question "View app2 config-mysql.php" 

echo "Updating revision to $revision in version.php file"
Replace-FileContent -inputFile $my_temp_target\version.php -outputFile $my_temp_target\version.php.new -pattern "define\('APPLICATION_VERSION_REV', ''\);" -replace "define('APPLICATION_VERSION_REV', '$revision');"
echo "Updated lines:"
cat $my_temp_target\version.php.new | Where-Object { $_ -match "APPLICATION_VERSION_REV" }
Rename-Item $my_temp_target\version.php $my_temp_target\version.php.orig
Rename-Item $my_temp_target\version.php.new $my_temp_target\version.php
AskView-FileContent -Path $my_temp_target\version.php -Question "View version.php" 

echo ""
echo "Going to switch to new version of $my_target_path"
$my_confirm = "Undefined"
do { $my_confirm = Read-Host "Continue [y/n]?" }
until (($my_confirm -eq "y") -or ($my_confirm -eq "n"))
if ($my_confirm -eq "n")
{
	Write-Host "Changes were not applied!" -foregroundcolor "Red"
	echo "Cleaning up..."
	Remove-Item -Path $my_temp_target -Force -Recurse
	echo "Stopping script."
	exit
}

$my_backup_path=$my_backup_base + $my_target_dir + "-backup-" + (Get-Date -UFormat "%Y-%m-%d--%H-%M-%S")
echo "Backing-up original directory to:"
echo $my_backup_path
echo "and switching to new version."
if (Test-Path $my_target_path)
{
    Move-Item $my_target_path $my_backup_path
}
Rename-Item $my_temp_target $my_target_path
Write-Host "Done." -foregroundcolor "Green"
echo "" 
echo "You might want to reload apache."
echo "" 

exit