<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009 - 2015 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package components/EAC
 */

/**
 * EACControl provides displaying of EAC overview
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class EACControl extends Control
{
    /** @const Determination of input data */
    /** TODO - Should be moved into the ProjectEACReporter. ProjectEACReporter should be then redesigned into more general "EACReporter" */
    const REPORT_LEVEL_PROJECT = 0,
          REPORT_LEVEL_WORKPACKAGE = 1;
    
    /** @const Keys of EAC data array */
    const KEY_CONTENT = 'content',
          KEY_YEARS = 'years';
    
    /** @const Format of numbers which are shown by templates */
    const NUMBER_FORMAT_PRECISION = 1,
          NUMBER_FORMAT_DECIMAL_DELIMITER = '.',
          NUMBER_FORMAT_ORDER_DELIMITER = ' ';
    
    /** @const EAC control states */
    const STATE_READY = 0,
          STATE_UNAVAILABLE = 1,
          STATE_NO_DATA = 2;
            
    /** @var Project */
    private $project;
    /** @var WorkPackage */
    private $workpackage;
    /** @var Task */
    private $task;
    /** @var int Unrolled year */
    private $year;
    /** @var int Current state */
    private $eac_state;
    /** @var int List of states processed by state manager */
    private $solved_states;
	/** @var boolean Decides if all months for current year should be expanded or not */
	private $expandMonths;
	/** @var array Contains all WorkPackages to obtain data from for EAC Control (recursion applied) */
	private $WPsToReport;
    
    /**
     * @param Project|WorkPackage $control_item
     * @param int $year
	 * @param boolean $expandMonths
     */
    public function __construct($control_item, $year = NULL, $expandMonths = FALSE)
    {
        parent::__construct();
        
        $this->project = new Project();
        $this->workpackage = new WorkPackage();
        $this->task = new Task();
        $this->year = ($year) ? $year : date('Y');
        $this->eac_state = self::STATE_READY;
        $this->solved_states = array();
		$this->expandMonths = $expandMonths;
		$this->WPsToReport = array();
        
        if ($control_item instanceof Project)
            $this->project = $control_item;
        elseif ($control_item instanceof WorkPackage)
            $this->workpackage = $control_item;
        elseif ($control_item instanceof Task)
            $this->task = $control_item;
        else
            $this->eac_state = self::STATE_UNAVAILABLE;
    }
    
    /**
     * Shows EAC table
     */
    public function render()
    {
		if ($this->presenter->isAjax()) // is ajax request
		{
			$template = $this->template;
			$template->setFile(__DIR__ . '/EACControl.latte');
			$template->expandMonths = $this->expandMonths;

			if ($this->workpackage->ID)
			{
				// Temporary loading of project id
				// TODO project id should be loaded automatically in find method
				$this->workpackage->ProjectID = $this->presenter->context->workPackageRepository->getProjectID($this->workpackage->ID);
				$this->project = $this->presenter->context->projectRepository->find($this->workpackage->ProjectID);
			}
			elseif ($this->task->ID)
				$this->project = $this->presenter->context->projectRepository->find($this->task->ProjectID);

			if (!$this->project)
				$this->project = new Project();

			if ($this->eac_state != self::STATE_UNAVAILABLE)
			{
				$template->eac_data = $this->getEACDataTable();

				if (!count($template->eac_data[self::KEY_YEARS]))
				{
					$this->eac_state = self::STATE_NO_DATA;
				}
				else
				{
					$template->unrolled_year = $this->year;
					$template->delta_keys = array(
						ProjectEACReporter::KEY_PLANNED_EAC_TOTAL_DELTA
					);

					$template->precision = self::NUMBER_FORMAT_PRECISION;
					$template->decimal_delimiter = self::NUMBER_FORMAT_DECIMAL_DELIMITER;
					$template->order_delimiter = self::NUMBER_FORMAT_ORDER_DELIMITER;
				}
			}

			$this->handleState();

			$template->eac_state = $this->eac_state;

			$template->render();
		}
    }
    
    /**
     * Shows total of deltas of cost EAC in single line
     */
    public function renderLine()
    {
        $template = $this->template;
        $template->setFile(__DIR__ . '/EACControlLine.latte');
        
        if ($this->workpackage->ID)
        {
            // Temporary loading of project id
            // TODO project id should be loaded automatically in find method
            $this->workpackage->ProjectID = $this->presenter->context->workPackageRepository->getProjectID($this->workpackage->ID);
            $this->project = $this->presenter->context->projectRepository->find($this->workpackage->ProjectID);
        }
        elseif ($this->task->ID)
            $this->project = $this->presenter->context->projectRepository->find($this->task->ProjectID);
        
        if (!$this->project)
            $this->project = new Project();
        
        if ($this->eac_state != self::STATE_UNAVAILABLE)
        {
            $template->eac_data = $this->getEACDataLine();
            
            if (!count($template->eac_data[self::KEY_YEARS]))
            {
                $this->eac_state = self::STATE_NO_DATA;
            }
            else
            {
                $template->unrolled_year = $this->year;
                $template->precision = self::NUMBER_FORMAT_PRECISION;
                $template->decimal_delimiter = self::NUMBER_FORMAT_DECIMAL_DELIMITER;
                $template->order_delimiter = self::NUMBER_FORMAT_ORDER_DELIMITER;
            }
        }
        
        $this->handleState();
        
        $template->eac_state = $this->eac_state;
            
        $template->render();
    }
    
    /**
     * Prepares EAC data for given control item per the entire duration of item
     * @return array
     *     $arr[EACControl::KEY_CONTENT] => mapping table between keys of values and human readable descriptions of the values 
     *                                          $arr[ProjectEACReporter::KEY_] => description string
     *     $arr[EACControl::KEY_YEARS] => list of years for control item is defined
     *     $arr[year from list EACControl::KEY_YEARS][ProjectEACReporter::KEY_] => value
     */
    protected function getEACDataTable()
    {
        $eac_data = array();
        $eac_data[self::KEY_CONTENT] = array(
            ProjectEACReporter::KEY_LABOR_PLANNED_HOURS_MONTH => 'Plan (h)',
            ProjectEACReporter::KEY_LABOR_EAC_HOURS_MONTH => 'EAC (h)',
            ProjectEACReporter::KEY_LABOR_PLAN_EAC_HOURS_MONTH_DELTA => 'Delta (h)',
			ProjectEACReporter::KEY_LABOR_PLANNED_COSTS_MONTH => 'Labor Planned Costs',
			ProjectEACReporter::KEY_NON_LABOR_PLANNED_COSTS_MONTH => 'Non-labor Planned Costs',
			ProjectEACReporter::KEY_LABOR_EAC_COSTS_MONTH => 'Labor EAC Costs',
			ProjectEACReporter::KEY_NON_LABOR_EAC_COSTS_MONTH => 'Non-labor EAC Costs',
            ProjectEACReporter::KEY_PLANNED_EAC_TOTAL_DELTA => 'Planned - EAC',
        );
        $eac_data[self::KEY_YEARS] = $this->presenter->context->projectRepository->findProjectYears($this->project->ID);
        
        $this->initEACData($eac_data);
        
        foreach($eac_data[self::KEY_YEARS] as $year)
        {
            try
            {
                if ($this->workpackage->ID)
                    $eac_reporter = new ProjectEACReporter($this->presenter->context, $this->workpackage, $year);
                elseif ($this->task->ID)
                    $eac_reporter = new ProjectEACReporter($this->presenter->context, $this->task, $year);
                else
                    $eac_reporter = new ProjectEACReporter($this->presenter->context, $this->project, $year);

                $eac_data[$year][ProjectEACReporter::KEY_LABOR_PLANNED_HOURS_MONTH] = $eac_reporter->getProjectLabourPlannedHoursMonthTotals();
                $eac_data[$year][ProjectEACReporter::KEY_LABOR_EAC_HOURS_MONTH] = $eac_reporter->getProjectLabourEACHoursMonthTotals();
				$eac_data[$year][ProjectEACReporter::KEY_LABOR_PLAN_EAC_HOURS_MONTH_DELTA] = $eac_reporter->getProjectLabourPlanEACMonthDelta();
                $eac_data[$year][ProjectEACReporter::KEY_LABOR_PLANNED_COSTS_MONTH] = $eac_reporter->getProjectLabourPlannedCostsMonthTotals();
                $eac_data[$year][ProjectEACReporter::KEY_LABOR_EAC_COSTS_MONTH] = $eac_reporter->getProjectLabourEACCostsMonthTotals();
                $eac_data[$year][ProjectEACReporter::KEY_NON_LABOR_PLANNED_COSTS_MONTH] = $eac_reporter->getProjectNonLabourPlannedCostsMonthTotals();
				// TODO: fix ProjectEACReporter instead
                $eac_data[$year][ProjectEACReporter::KEY_NON_LABOR_EAC_COSTS_MONTH] = $this->getProperProjectNonLabourEACCostsMonthTotals($eac_reporter, $year);
				$eac_data[$year][ProjectEACReporter::KEY_PLANNED_EAC_TOTAL_DELTA] = $this->getProperProjectPlannedCostsEACCostsMonthTotals($eac_data[$year]);
            }
            catch (InvalidStateException $e)
            {
                Debugger::log('EACControl: ' . $e->getMessage(), Debugger::ERROR);
            }
        }
        return $eac_data;
    }
    
    /**
     * Prepares EAC data for given control item per the entire duration of item
     * @return array
     *     $arr[EACControl::KEY_CONTENT] => mapping table between keys of values and human readable descriptions of the values 
     *                                          $arr[ProjectEACReporter::KEY_] => description string
     *     $arr[EACControl::KEY_YEARS] => list of years for control item is defined
     *     $arr[year from list EACControl::KEY_YEARS][ProjectEACReporter::KEY_] => value
     */
    protected function getEACDataLine()
    {       
        $eac_data = array();
        $eac_data[self::KEY_CONTENT] = array(
            ProjectEACReporter::KEY_LABOR_PLAN_EAC_COSTS_MONTH_DELTA => 'Delta (USD)',
            ProjectEACReporter::KEY_NON_LABOR_PLAN_EAC_COSTS_MONTH_DELTA => 'Delta (USD)'
        );        
        $eac_data[self::KEY_YEARS] = $this->presenter->context->projectRepository->findProjectYears($this->project->ID);
        
        $this->initEACData($eac_data);
        
        foreach($eac_data[self::KEY_YEARS] as $year)
        {
            try
            {
                if ($this->workpackage->ID)
                    $eac_reporter = new ProjectEACReporter($this->presenter->context, $this->workpackage, $year);
                elseif ($this->task->ID)
                    $eac_reporter = new ProjectEACReporter($this->presenter->context, $this->task, $year);
                else
                    $eac_reporter = new ProjectEACReporter($this->presenter->context, $this->project, $year);

                $eac_data[$year][ProjectEACReporter::KEY_LABOR_PLAN_EAC_COSTS_MONTH_DELTA] = $eac_reporter->getProjectLabourPlanEACCostsMonthTotalsDelta();
                $eac_data[$year][ProjectEACReporter::KEY_NON_LABOR_PLAN_EAC_COSTS_MONTH_DELTA] = $this->getProperProjectNonLabourCostsMonthTotalsDelta($eac_reporter, $year);
            }
            catch (InvalidStateException $e)
            {
                Debugger::log('EACControl: ' . $e->getMessage(), Debugger::ERROR);
            }
        }
        
        return $eac_data;
    }
    
    /**
     * Initializes structure of EAC data after its content and time range is set
     * @param array &$eac_data
     */
    protected function initEACData(&$eac_data)
    {
        foreach($eac_data[self::KEY_YEARS] as $year)
        {
            foreach(array_keys($eac_data[self::KEY_CONTENT]) as $key)
                $eac_data[$year][$key] = $eac_data[$this->year][$key] = new MonthArray(0.0);
        }
    }
    
    /**
     * Handles given state via $this->eac_state
     */
    private function handleState()
    {
        if (in_array($this->eac_state, $this->solved_states))
            return;
        
        array_push($this->solved_states, $this->eac_state);
        
        switch($this->eac_state)
        {
            case self::STATE_UNAVAILABLE:
                $this->flashMessage('EAC section is unavailable.', 'error');
                break;
            case self::STATE_NO_DATA:
                $this->flashMessage('No available data for EAC.', 'info');
                break;
            default:
                break;
        }
    }
	
	/**
	 * This is a hotfix function, beacause ProjectEACReporter can't calculate Non-labor EAC Costs
	 * right when a TaskPlan for the particular Task is not in DB. We should fix it there and 
	 * redirect all reports, GUIs and tracking table to it (and remove code which handles this broken
	 * ProjecEACReporter functionality in them).
	 */
	private function getProperProjectNonLabourEACCostsMonthTotals($eac_reporter, $year)
	{
		$costs = new MonthArray((float) 0, false);
		/* getTaskList() equivalent */
		$tasks = array();
		if ($this->workpackage->ID)
		{
			// Tasks just of this WP (not any from nested WPs)
			$tasks = $this->presenter->context->
					taskRepository->findAllWorkPackageTasks($this->workpackage->ID);

			// there could be nested Workpackages with more Tasks
			$this->findAllWPsToReport($this->workpackage->ID);

			// now get all of its children WPs and add theirs Tasks to the list
			foreach ($this->WPsToReport as $wp)
			{
				$WPTasks = $this->presenter->context->taskRepository
						->findAllWorkPackageTasks($wp->ID);

				$tasks += $WPTasks;
			}
		}
		elseif ($this->task->ID)
			$tasks = array($this->task->ID => $this->task);
		else
		{
			// we have a project, so get its tasks
			$allTasks = $this->presenter->context->taskRepository
				->findAllProjectTasks($this->project->ID);

			// check every task if he has some data to report.
			foreach($allTasks as $task)
			{
				// has some plan
				if($this->presenter->context->taskPlanRepository->findTaskYearPlan($task->ID, $year))
				{
					$tasks[$task->ID] = $task;
					continue;
				}
				// has actual costs
				if(!$task->isLabourTask())
				{
					if($this->presenter->context->costCenterRepository->findActualCostsYearSummary($task->ID, $year, 'USD')->sum() != 0)
					{
						$tasks[$task->ID] = $task;
					}
				}
			}
		}

		/* EAC calculation part equivalent */
		$currency = "USD";
		$actualMonth = FiscalCalendar::getFiscalMonthByTimestamp(time())->index + 1;

		foreach ($tasks as $task)
		{
			if (!$task->isLabourTask())
			{
				$costAssignments = $this->presenter->context->assignmentsCostRepository->findAllForTask($task->ID);
				$costActuals = $this->presenter->context->actualsCostRepository->findForTask($task->ID, $currency);
			}
			else
				continue;

			$taskPlans = $this->presenter->context->taskPlanRepository->findTaskPlans($task->ID);

			for ($i = 1; $i < 13; $i++)
			{
				if ($year == Date('Y'))
				{
					if ($i >= $actualMonth)
					{
						if(isset($taskPlans[$year]))
						{
							$costs[$i] += $this->getAssignedCost($costAssignments, $taskPlans[$year], $year, $i);
						}
						else
						{
							$costs[$i] += $this->getAssignedCost($costAssignments, null, $year, $i);
						}
					}
					else
					{
						$costs[$i] += $this->getActualsCost($costActuals, $year, $i);
					}
				}
				elseif ($year > Date('Y'))
				{
					if(isset($taskPlans[$year]))
					{
						$costs[$i] += $this->getAssignedCost($costAssignments, $taskPlans[$year], $year, $i);
					}
					else
					{
						$costs[$i] += $this->getAssignedCost($costAssignments, null, $year, $i);
					}
				}
				else
				{
					$costs[$i] += $this->getActualsCost($costActuals, $year, $i);
				}
			}
		}
		return $costs;
		
	}
	
	private function getActualsCost($costActuals, $year, $month)
	{
		if (isset($costActuals[$year]))
		{
			if (isset($costActuals[$year][$month]))
			{
				return $costActuals[$year][$month];
			}
			else
			{
				return 0;
			}
		}
		else
		{
			return 0;
		}
	}
	
	private function getAssignedCost($costAssignments, $taskPlan, $year, $month)
	{
		if (isset($costAssignments[$year]))
		{
			if (isset($costAssignments[$year][$month]))
			{
				return $costAssignments[$year][$month]->Cost;
			}
			else
			{
				if($taskPlan != null)
					return $taskPlan->getMonthPlan($month)->Budget;
				else
					return 0;
			}
		}
		else
		{
			if($taskPlan != null)
				return $taskPlan->getMonthPlan($month)->Budget;
			else
				return 0;
		}
	}
	
	/**
	 * A part of ProjectEACReporter hotfix.
	 */
	private function getProperProjectPlannedCostsEACCostsMonthTotals($cachedEAC)
	{
		$labourPlannedCosts = $cachedEAC[ProjectEACReporter::KEY_LABOR_PLANNED_COSTS_MONTH];
		$nonLabourPlannedCosts = $cachedEAC[ProjectEACReporter::KEY_NON_LABOR_PLANNED_COSTS_MONTH];
		$labourEACCosts = $cachedEAC[ProjectEACReporter::KEY_LABOR_EAC_COSTS_MONTH];
		$nonLabourEACCosts = $cachedEAC[ProjectEACReporter::KEY_NON_LABOR_EAC_COSTS_MONTH];
		
		$planned = $labourPlannedCosts->plus($nonLabourPlannedCosts);
		$eac = $labourEACCosts->plus($nonLabourEACCosts);
		return $planned->minus($eac);
	}
	
	/**
	 *  A part of ProjectEACReporter hotfix.
	 */
	private function getProperProjectNonLabourCostsMonthTotalsDelta($eac_reporter, $year)
	{
		$sap = $eac_reporter->getProjectNonLabourPlannedCostsMonthTotals();
		$actual = $this->getProperProjectNonLabourEACCostsMonthTotals($eac_reporter, $year);
		return $sap->minus($actual);
	}
	
	/**
	 * Recursive function gets all wp to report
	 * 
	 * @param int $workPackageID
	 */
	private function findAllWPsToReport($workPackageID)
	{
		$wpChildren = $this->presenter->context->workPackageRepository->findByParentID($workPackageID);
		$this->WPsToReport = array_merge($this->WPsToReport, $wpChildren);
		foreach ($wpChildren as $wp)
		{
			$this->findAllWPsToReport($wp->ID);
		}
	}
	
    /**
     * Load values to eac component
     */
    public function handleAjaxCall()
    {
		$this->presenter->invalidateControl('eacComponent');
    }
}