<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for administration of persons.
 *
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class PersonPresenter extends SecuredPresenter
{
	/** @var session */
	private $session;
    /** @persistent */
    public $year;
    /** @persistent */
    public $id;
	
    /**
	 * Initialization
	 */
	public function startup()
	{
        parent::startup();

        $this->session = $this->getSession('Person');
        $this->session->setExpiration(0);

        $this->setYear($this->year);
        
        // Forget year before opening another person detail
        if ($this->getAction() == 'default')
        	$this->year = NULL;
	}
    
	/**
	 * Compares two persons by surname alphabetically.
	 * 
	 * @param array of Person that can be assigned to task
	 * @param array of Person that can be assigned to task
	 * @return int -1, 0, 1
	 */
	private function comparePersonBySurname($person1, $person2)
	{
		return strcasecmp($person1->getSurname(), $person2->getSurname());
	}
    
    /**
     * Global setting of templates
     */
    public function beforeRender()
    {
        parent::beforeRender();
        
        $this->template->year = $this->year;
    }
	
    /**
     * Shows overview of all persons
     */
    public function renderDefault()
    {
		$this->template->form = $this['filterPersonForm'];
		$filtered_persons = $persons = $finalPersonList = array();
		$orgUnit = null;
		
		if ($this->isAjax())
		{
			$filter = new PersonFilter();
			if (isset($this->payload->filter)) // filter is defined
			{
				$entry = $this->payload->filter; // $entry contains content of filterPersonForm

				$filter->ID = $entry->EID;
				$filter->Firstname = $entry->Name;
				$filter->Surname = $entry->Surname;
				$filter->CostCenterID = $entry->CostCenter;
				$filter->Status = $entry->FilterStatus;
					$orgUnit_id = NULL;
					if ($entry->OrganizationUnitSelector->TechnicalGroup)
						$orgUnit_id = $entry->OrganizationUnitSelector->TechnicalGroup;
					elseif ($entry->OrganizationUnitSelector->Cluster)
						$orgUnit_id = $entry->OrganizationUnitSelector->Cluster;
					elseif ($entry->OrganizationUnitSelector->Department)
						$orgUnit_id = $entry->OrganizationUnitSelector->Department;
					elseif ($entry->OrganizationUnitSelector->SBE)
						$orgUnit_id = $entry->OrganizationUnitSelector->SBE;
				$orgUnit = $this->context->organizationUnitRepository->find($orgUnit_id);
			}
			
			$this->context->formStateRepository->save('filterPersonForm', $this->template->form); // save filter state
			$filtered_persons = $this->context->personRepository->getPersonsByFilter($filter);
		}
		else
		{
			if ($this->context->formStateRepository->exists('filterPersonForm')) // exists saved filter
				$this->context->formStateRepository->load('filterPersonForm', $this->template->form);
		}
		
		if($orgUnit != null)
        {
            foreach($filtered_persons as $person)
                if($person->OrganizationUnitID != null)
                {
                    if(preg_match('#^' . $orgUnit->Path . '#', $this->context->organizationUnitRepository->find($person->OrganizationUnitID)->Path))
                        $finalPersonList[$person->ID] = $person;
                }
                else
                    Debugger::log('Person with eid: ' . $person->ID . ' does not have set up organization unit ID.', Debugger::ERROR);
        }
        else
		{
            $finalPersonList = $filtered_persons;
		}
		
		// get all nationalities to display them in person overview
		$nationalities = $this->context->nationalityRepository->findAll();
		
        usort($finalPersonList, array($this, 'comparePersonBySurname'));
        foreach ($finalPersonList as $person)
        {
            $persons[]['EID'] = $person->ID;
            $persons[count($persons)-1]['Surname'] = mb_convert_encoding($person->Surname, 'UTF-8');
            $persons[count($persons)-1]['Firstname'] = mb_convert_encoding($person->Firstname, 'UTF-8');
            if($person->OrganizationUnitID == null)
                $persons[count($persons)-1]['OrganizationUnit'] = '-';
            else
                $persons[count($persons)-1]['OrganizationUnit'] = $this->context->organizationUnitRepository->find($person->OrganizationUnitID)->Name;
            $persons[count($persons)-1]['CostCenter'] = $this->context->costCenterRepository->findCostCenter($person->CostCenterID)->Name;
            $persons[count($persons)-1]['CostCenterID'] = $this->context->costCenterRepository->findCostCenter($person->CostCenterID)->ID;
            $persons[count($persons)-1]['Comment'] = mb_convert_encoding(($person->Comment) ? substr($person->Comment, 0, 30) : '-', 'UTF-8');
			$persons[count($persons)-1]['Comment'] .=  mb_convert_encoding((strlen($person->Comment) > 30) ? '..' : '', 'UTF-8');
            $persons[count($persons)-1]['Comment_Full'] = mb_convert_encoding(($person->Comment) ? $person->Comment : '-', 'UTF-8');
			$persons[count($persons)-1]['Status'] = $person->Status;
			$persons[count($persons)-1]['NationalityID'] = (isset($nationalities[$person->NationalityID])) ? $nationalities[$person->NationalityID]->Name : '';
        }
        $this->template->persons = $persons;
    }

    /**
     * Shows detail of Person.
     * 
     * @param string $id Person EID.
	 * @param int $year Year.
	 * 
     */
    public function renderShowDetail($id, $year = NULL)
    {
		$this->id = $id;

		if ($year && is_numeric($year))
			$this->year = $year;

        $available_years = $this->context->aopRepository->getYears();
		if(!in_array($this->year, $available_years))
        {
			$this->year = (int) date('Y');
            $this->flashMessage('Can not show selected year.', 'info');
            $this->redirect('this');
        }        
        
        //=====person detail=====
        $person = $this->context->personRepository->find($this->id);
		if($person == null)
		{
			$this->flashMessage('Person with EID: ' . $id . ' not found.', 'error');
			$this->redirect('default');
		}
		$this->template->person = $person;
        
        $form = $this['detailPersonForm'];
		// set only active work centers
        $tmp = $this->context->costCenterRepository->findAllCostCenters();
        $costCenters = array();
        foreach ($tmp as $costCenter)
        {
			if($costCenter->Status == 1 || $person->CostCenterID == $costCenter->ID)
			{
				$costCenters[$costCenter->ID] = $costCenter->Name . " (" . $costCenter->ID . ")";
			}
        }
		$form['CostCenter']->setItems($costCenters);
        $formDefaults = array();
        $formDefaults['EID'] = $person->ID;
        $formDefaults['Surname'] = $person->Surname;

        $org_unit = $this->context->organizationUnitRepository->find($person->OrganizationUnitID);
		if ($org_unit)
		{
			switch ($org_unit->Type)
			{
				case OrganizationUnit::ORG_UNIT_TYPE_SBE:
					$formDefaults['OrganizationUnitSelector']['SBE'] = $person->OrganizationUnitID;
					break;
				case OrganizationUnit::ORG_UNIT_TYPE_DEP:
					$formDefaults['OrganizationUnitSelector']['Department'] = $person->OrganizationUnitID;
					break;
				case OrganizationUnit::ORG_UNIT_TYPE_CLUS:
					$formDefaults['OrganizationUnitSelector']['Cluster'] = $person->OrganizationUnitID;
					break;
				case OrganizationUnit::ORG_UNIT_TYPE_TG:
					$formDefaults['OrganizationUnitSelector']['TechnicalGroup'] = $person->OrganizationUnitID;
					break;
				default:
					break;
			}
		}
		if($person->Locked)
		{
			$form['OrganizationUnitSelector']['SBE']->setDisabled(true);
			$form['OrganizationUnitSelector']['Department']->setDisabled(true);
			$form['OrganizationUnitSelector']['Cluster']->setDisabled(true);
			$form['OrganizationUnitSelector']['TechnicalGroup']->setDisabled(true);
			$form['delete']->setDisabled(true);
		}

		$supervisor = $this->context->personRepository->find($person->SupervisorID);
		if ($supervisor)
		{
            $temp_org_unit = $org_unit;

            // Look into higher level
			if ($org_unit && (mb_strtolower($supervisor->ID) !== mb_strtolower($org_unit->Leader)))
			{
				$org_unit = $this->context->organizationUnitRepository->find($supervisor->OrganizationUnitID);
			}

            $leadership_info = (($org_unit) ? $org_unit->LeadershipInfo : null);
			$formDefaults['Supervisor_info'] = $supervisor->ReversedFullName . $leadership_info;
			$formDefaults['Supervisor'] = $person->SupervisorID;

			$org_unit = $temp_org_unit;
			unset($temp_org_unit, $leadership_info);
		}
		else
		{
			$formDefaults['Supervisor_info'] = 'Undefined';
			$formDefaults['Supervisor'] = '-';
		}

        $formDefaults['Name'] = $person->Firstname;
        $formDefaults['CostCenter'] = $person->CostCenterID;
        $formDefaults['Email'] = $person->Email;
        $formDefaults['Comment'] = $person->Comment;
		$formDefaults['NationalityID'] = $person->NationalityID;
		// setting of displayed content
		$all_SBE = $this->context->organizationUnitRepository->findAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_SBE);
		$all_setting = $this->context->settingDisplayedContentRepository->findByEID($person->ID);
		foreach ($all_SBE as $sbe)
		{
			if (isset($all_setting[$sbe->ID]))
				$formDefaults['SBE' . $sbe->ID] = TRUE;
		}
		
		// setting of licences
		$all_licences = $this->context->ecLicenceRepository->findAll();
		$this->template->allLicences = $all_licences;
		$licensed = $this->context->personECLicenceRepository->findByEID($person->ID);
		foreach ($all_licences as $licence)
		{
			if (isset($licensed[$licence->ECLicenceID]))
				$formDefaults[$licence->ECLicenceID] = TRUE;
		}
		
		if($person->Status)
		{
			$form['From']->addConditionOn($form['Status'], Form::EQUAL, 'Inactive')
                ->addRule(Form::FILLED, 'Select From value!');
			$formDefaults['Status'] = 'Active';
		}
		else
		{
			$form['From']->addConditionOn($form['Status'], Form::EQUAL, 'Active')
                ->addRule(Form::FILLED, 'Select From value!');
			$formDefaults['Status'] = 'Inactive';
		}
        if (substr($person->ID, 0, 3) == "ENN")
        {
            $this->template->changeEID = true;
        }
        else
        {
            $this->template->changeEID = false;
        }
		
        $form->setDefaults($formDefaults);
        $this->template->form = $form;
		
		$this->template->all_SBE = $all_SBE;
		
		
		//=====workable hours=====        
        $isAOP = null;
        
		$hour = $this->context->aopRepository->findForPerson($this->year, $this->id);
		$aopHour = $this->context->aopRepository->find($this->year);
		$isAOP = $this->context->workableHourRepository->isAop($this->year, $this->id);
        
        $formDefaultHours = array(); 
        for($i=0; $i<12; $i++)
        {            
            $formDefaultHours[ FiscalCalendar::fiscalMonthIndexToName($i, false) ] = $hour->getMonthAOPHours($i+1);
        }
        for($i=0; $i<12; $i++)
        {
            $formDefaultHours[ 'H' . FiscalCalendar::fiscalMonthIndexToName($i, false) ] = $aopHour->getMonthAOPHours($i+1);
        }
        $formDefaultHours['Year'] = $hour->getYear();
        $formDefaultHours['EID'] = $this->id;
        $formDefaultHours['isAop'] = $isAOP;
		$formDefaultHours['Remark'] = $this->context->workableHourRepository->getRemark($this->id, $this->year);
        $formDefaultHours['Yield'] = $this->context->workableHourRepository->getYield($this->id, $this->year)*100;
		$formDefaultHours['Contract'] = $this->context->workableHourRepository->getContract($this->id, $this->year)*100;
		$formDefaultHours['YieldRecalculation'] = 100;
        $formHour = $this['workableHourForm'];
        $formHour->setDefaults($formDefaultHours);
        $this->template->formHour = $formHour;
		$this->template->aopHours = $aopHour;
		
		//=====worker resource plan=====
        $actualMonth = FiscalCalendar::getFiscalMonthByTimestamp(time())->index+1;
        $this->template->actualYear = $this->year;
        $this->template->currentMonth = $actualMonth;
        $this->template->currentYear = date('Y');
         
        
        /* --- all projects tasks hours of preson and total year sum--- */
		$formActualsAndAssignedHours = $this['assignedAndActualHoursForm'];
		$projects = array();
		$projects = $this->context->projectRepository->findPersonAssignedAtProjectsInYear($this->id, $this->year); 
		// lest find projects where persons are not assigned
		$notAssignedProjects = $this->context->projectRepository->findPersonNotAssignedAtProjectsInYear($this->id, $this->year);
		foreach($notAssignedProjects as $project)
		{
			if(!isset($projects[$project->ID]))
				$projects[$project->ID] = $project;  
		}
        $assignedAndActualHours = array();
        $assignedAndActualHoursYearTotal = array(); 
        for($i = 1; $i < 13; $i++)
        {
            $assignedAndActualHoursYearTotal[$i] = 0;
        }
        $projectNames = array();
        $taskNames = array();
        $taskSapID = array();
		$actualHoursProjectMonthTotal = $assignedPermissions = array();
        foreach ($projects as $project)
        {
            $projectNames[$project->ID] = $project->Name;
			$projectNames[$project->ID . 'Fake'] = $project->Fake;
            $tasks = $this->context->taskRepository->findPersonTasksInYear($this->id, $project->ID, $this->year);
			$activeTasks = $this->context->taskRepository->findPersonActiveTasksInYear($this->id, $project->ID, $this->year);
			foreach($activeTasks as $task)
			{
				if(!isset($tasks[$task->ID]))
				{
					$tasks[$task->ID] = $task;
				}
			}
            foreach($tasks as $task)
            {
				if ($this->context->taskPlanRepository->findTaskYearPlan($task->ID, $this->year) === NULL)
				{
					// check for any values associated to this task
					// current task doesn't have TaskPlan, so it can't have
					// any assignments either -> check for Actuals / ActualsCosts
					$hasAnyValue = FALSE;
					if ($task->isLabourTask())
					{
						$people = $this->context->workRepository->findPeopleWorkingOnTaskInYear($task->ID, $this->year);
						if (!empty($people))
							$hasAnyValue = TRUE;
					}
					else
					{
						if ($this->context->costCenterRepository->findActualCostsYearSummary($task->ID, $this->year, 'USD')->sum() != 0)
							$hasAnyValue = TRUE;
					}
					
					if ($hasAnyValue)
					{
						// create a new TaskPlan and save it
						$task_plan = new TaskYearPlan($this->year);
						$task_plan->TaskID = $task->ID;
						$this->context->taskPlanRepository->save($task_plan);
					}
				}
				$assignmentControl = new AssignmentControl($person, $task);
				$assignedPermissions[$project->ID][$task->ID] = $assignmentControl;
                $assigned = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($task->ID, $this->year, $this->id);
                $actual = $this->context->workRepository->findPersonTaskYearWork($this->id, $task->ID, $this->year);
                $taskNames[$task->ID] = $task->Name;
				$taskNames[$task->ID . 'Fake'] = $task->Fake;
                $taskSapID[$task->ID] = $task->SAPID;
                foreach($assigned as $month => $hour)
                {
					if(!$assignmentControl->assignable)
						$formActualsAndAssignedHours[$project->ID . '_' . $task->ID . '_' . $month]->setAttribute('readonly', 'readonly');
					if($this->year == Date('Y'))
					{
						if($month >= $actualMonth)
						{
							$assignedAndActualHours[$project->ID][$task->ID][$month] = $hour->Hours;
							$assignedAndActualHoursYearTotal[$month] += $hour->Hours;
						}
						else
						{
							$assignedAndActualHours[$project->ID][$task->ID][$month] = $actual[$month];
							$assignedAndActualHoursYearTotal[$month] += $actual[$month];
						}
					}
					elseif($this->year > Date('Y'))
					{
						$assignedAndActualHours[$project->ID][$task->ID][$month] = $hour->Hours;
						$assignedAndActualHoursYearTotal[$month] += $hour->Hours;
					}
					else
					{
						$assignedAndActualHours[$project->ID][$task->ID][$month] = $actual[$month];
						$assignedAndActualHoursYearTotal[$month] += $actual[$month];
					}
                }
            }
			$actualHoursProjectMonthTotal[$project->ID] = $this->context->workRepository->findPersonProjectMonthWork($this->id, $project->ID, $this->year);
        }

        /* --- Total task hours sum --- */
        $assignedAndActualHoursTaskTotal = array();
        
        foreach ($assignedAndActualHours as $projectName => $project)
        {
            $assignedAndActualHoursTaskTotal[$projectName] = array();
            for($i = 1; $i < 13; $i++)
            {
                $assignedAndActualHoursTaskTotal[$projectName][$i] = 0;
            }
            foreach($project as $task)
            {
                foreach($task as $month => $hour)
                {
                    $assignedAndActualHoursTaskTotal[$projectName][$month] += $hour;
                }
            }
        }
		
		$actualHours = array();
		foreach ($assignedAndActualHours as $project)
			foreach ($project as $task => $value)
			    $actualHours[intval($task)] = $this->context->workRepository->findPersonTaskYearWork($this->id, $task, $this->year);
        
        if($assignedAndActualHours == null)
        {
            $this->flashMessage("Resource is not planned for selected year.", 'info');
        }

		//=====Person assigment transfer=====
		$urlPersonAssignmentTransferSelect = str_replace('?', '?do=PersonAssignmentTransferSelect&', $this->link('Person:showDetail'));
		$this->template->linkToPersonAssignmentTrasfer = $urlPersonAssignmentTransferSelect;
		//Task assignment transfer form
		$personAssignmentTransferForm = $this['personAssignmentTransferForm'];
		$this->template->personAssignmentTransferAllowed = $this->isAllowed(self::RESOURCE_PERSON_ASSIGNMENT_TRANSFER, self::ACTION_EDIT);
		$this->template->personAssignmentTransferForm = $personAssignmentTransferForm;
		if($person->Locked && $this->isAllowed(self::RESOURCE_RESOURCE_LOCK, self::ACTION_RUN))
			$this->template->assignmentTrasferMessage = "The worker: " . $person->getReversedFullNameWithEID() . ' to be tranferred is locked by resource owner. Are you sure you will not violate resource owner´s intentions?'; // inform PMO users is any worker is locked
		else
			$this->template->assignmentTrasferMessage = null;
		
		$this->template->assignedPermissions = $assignedPermissions;
		$this->template->workableHours = $this->context->aopRepository->findForPerson($this->year, $this->id);
        $this->template->assignedAndActualHours = $assignedAndActualHours;
        $this->template->projectNames = $projectNames;
        $this->template->taskNames = $taskNames;
        $this->template->taskSapID = $taskSapID;
        $this->template->assignedAndActualHoursTaskTotal = $assignedAndActualHoursTaskTotal;
        $this->template->assignedAndActualHoursYearTotal = $assignedAndActualHoursYearTotal;
        $this->template->person = $this->context->personRepository->find($this->id);
		$this->template->nationality = $this->context->nationalityRepository->find($person->NationalityID);
        $this->template->assignedAndActualHoursForm = $formActualsAndAssignedHours;
        $this->template->actualHoursProjectMonthTotal = $actualHoursProjectMonthTotal;
        $this->template->actualHoursArray = $actualHours;
		
    }

    /**
     * Renders form for creating new Person.
     */
    public function renderCreate($personImportEID = null)
    {
        $form = $this['createPersonForm'];
        $formDefaults = array();
        foreach ($this->request->post as $dbKey => $value)
            $formDefaults[$dbKey] = $value;
		if($personImportEID != null)
		{
			$formDefaults = array();
			$person = $this->context->personFoundRepository->findByEID($personImportEID);
			if (!$person)
			{
				$this->flashMessage("Person with EID \"$personImportEID\" not found.", 'error');
				$this->redirect('PersonImport:newPersons');
			}
			$formDefaults['EID'] = $person->EID;
			$formDefaults['Name'] = $person->Name;
			$formDefaults['Surname'] = $person->Surname;
			$formDefaults['CostCenter'] = $person->WorkCenterID;
			$orgUnit = $this->context->organizationUnitRepository->find($person->OrganizationUnitID);
			if($orgUnit != null)
			{
				switch ($orgUnit->Type)
				{
					case OrganizationUnit::ORG_UNIT_TYPE_SBE:
						$formDefaults['OrganizationUnitSelector']['SBE'] = $person->OrganizationUnitID;
						break;
					case OrganizationUnit::ORG_UNIT_TYPE_DEP:
						$formDefaults['OrganizationUnitSelector']['Department'] = $person->OrganizationUnitID;
						break;
					case OrganizationUnit::ORG_UNIT_TYPE_CLUS:
						$formDefaults['OrganizationUnitSelector']['Cluster'] = $person->OrganizationUnitID;
						break;
					case OrganizationUnit::ORG_UNIT_TYPE_TG:
						$formDefaults['OrganizationUnitSelector']['TechnicalGroup'] = $person->OrganizationUnitID;
						break;
					default:
						break;
				}
			}
			$formDefaults['NationalityID'] = 3;
		}
        $form->setDefaults($formDefaults);
        $this->template->form = $form;
		$all_SBE = $this->context->organizationUnitRepository->findAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_SBE);
		$this->template->all_SBE = $all_SBE;
    }
    
    /**
     * Creates general Person form.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentPersonForm($name)
    {
        $form = new AppForm($this, $name);

        $form->addText('EID', 'EID', 30, 7);
        $form->addText('Surname', 'Surname', 30, 100);
        $form->addText('Name', 'Name', 30, 100);
        $form->addText('Email', HPTForm::createLabel('E-mail', 'E-mail address will be automatically updated from Honeywell LDAP server upon Person login into application.'), 30, 100)
				->addCondition(Form::FILLED, TRUE)
					->addRule(Form::EMAIL, '%value is not valid e-mail address!');
		$form->addText('Supervisor', 'Supervisor', 7)
			->setAttribute('readonly', 'readonly');
		$form->addText('Supervisor_info', 'Supervisor_info')
			->setAttribute('readonly', 'readonly');
		
        $tmp = $this->context->costCenterRepository->findAllCostCenters();
        $costCenters = array();
        foreach ($tmp as $costCenter)
        {
			if($costCenter->Status == 1)
			{
				$costCenters[$costCenter->ID] = $costCenter->Name . " (" . $costCenter->ID . ")";
			}
        }
        $form->addSelect2('CostCenter', 'Work Center', $costCenters)
				->setPrompt('*')
				->setAttribute('style', 'width: 380px;');

	    $form['OrganizationUnitSelector'] = new OrganizationUnitPersonSelectorContainer(380, FALSE);
	        $form['OrganizationUnitSelector']['ajaxHandle']->setValue($this->link('OrganizationUnitPersonSelector'));
		
		$form->addTextArea('Comment', 'Comment', 35, 5);
        
		$form->addRadioList('Status', 'Status', array('Active' => 'Active', 'Inactive' => 'Inactive'));
		$form['Status']->getSeparatorPrototype()->setName(NULL);
		
		$all_nationality = $this->context->nationalityRepository->findAll();
		$nationalities = array();
		foreach ($all_nationality as $nationality)
		    $nationalities[$nationality->NationalityID] = $nationality->Name;
        $form->addSelect('NationalityID', 'Licenced up to', $nationalities)
				->setPrompt('')
				->setAttribute('style', 'width: 200px;');
		
        $form->addText('From', 'From')
            ->addCondition(Form::FILLED, TRUE)
            ->addRule(Form::PATTERN, '%value is not valid %label!', '[0-9]{4}-[0-9]{2}');
        $form['From']->getControlPrototype()->addClass('monthpicker');
		
        return $form;
    }
    
    /**
     * Creates form for filtering Persons based on createComponentPersonForm.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentFilterPersonForm($name)
    {
        $form = $this->createComponentPersonForm($name);
        
		$form->addCheckboxList('FilterStatus', 'Status', array('Active' => 'Active', 'Inactive' => 'Inactive'));
        $form['FilterStatus']->setDefaultValue(array('Active', 'Inactive'));
		$form['FilterStatus']->getSeparatorPrototype()->setName(NULL);
        $form->addSubmit('reset', 'Reset Filter')
            ->onClick[] = callback($this, 'filterPersonFormReset');
        $form->addSubmit('create', 'Create new person')
            ->onClick[] = callback($this, 'filterPersonFormCreate');
        if (!$this->isAllowed(self::RESOURCE_PERSON, self::ACTION_CREATE))
            $form['create']->setDisabled(TRUE);
        $form->getElementPrototype()->class('ajax');
        $form->onSuccess[] = array($this, 'filterPersonFormSubmitted');
        
        return $form;
    }
    
    /**
     * Creates form for showing Person's details based on createComponentPersonForm.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentDetailPersonForm($name)
    {
        $form = $this->createComponentPersonForm($name);
		
		$all_SBE = $this->context->organizationUnitRepository->findAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_SBE);
		foreach ($all_SBE as $sbe)
        {
            if (!$sbe->Virtual)
		        $form->addCheckbox('SBE' . $sbe->ID, $sbe->Short);
        }
		
		$form->addText('DisplayedContent', HPTForm::createLabel('Displayed Content', 'Current person is able to see only content of selected SBEs on sites where content is filtered.'));
		// Checkboxes for project licences where is person assigned
		$all_licences = $this->context->ecLicenceRepository->findAll();
		foreach ($all_licences as $licence)
		    $form->addCheckbox($licence->ECLicenceID, $licence->Name)
				->setDisabled(true);

        $form['EID']
                ->setAttribute('readonly', 'readonly');
        $form['Surname']->setRequired('Surname field is mandatory.');
        $form->addText('changeEID', 'Change EID')
                ->setAttribute('maxlength', '7');
        $form['CostCenter']
                ->setRequired('Please select Work Center.');

        $form['OrganizationUnitSelector']['SBE']
                ->addConditionOn($form['OrganizationUnitSelector']['Department'], ~Form::FILLED)
                ->elseCondition($form['OrganizationUnitSelector']['Cluster'], ~Form::FILLED)
                ->elseCondition($form['OrganizationUnitSelector']['TechnicalGroup'], ~Form::FILLED)
                        ->addRule(Form::FILLED, 'Organization Unit must be filled.');
        $form['OrganizationUnitSelector']['Department']
                ->addConditionOn($form['OrganizationUnitSelector']['SBE'], ~Form::FILLED)
                ->elseCondition($form['OrganizationUnitSelector']['Cluster'], ~Form::FILLED)
                ->elseCondition($form['OrganizationUnitSelector']['TechnicalGroup'], ~Form::FILLED)
                        ->addRule(Form::FILLED, 'Organization Unit must be filled.');
        $form['OrganizationUnitSelector']['Cluster']
                ->addConditionOn($form['OrganizationUnitSelector']['SBE'], ~Form::FILLED)
                ->elseCondition($form['OrganizationUnitSelector']['Department'], ~Form::FILLED)
                ->elseCondition($form['OrganizationUnitSelector']['TechnicalGroup'], ~Form::FILLED)
                        ->addRule(Form::FILLED, 'Organization Unit must be filled.');
        $form['OrganizationUnitSelector']['TechnicalGroup']
                ->addConditionOn($form['OrganizationUnitSelector']['SBE'], ~Form::FILLED)
                ->elseCondition($form['OrganizationUnitSelector']['Department'], ~Form::FILLED)
                ->elseCondition($form['OrganizationUnitSelector']['Cluster'], ~Form::FILLED)
                        ->addRule(Form::FILLED, 'Organization Unit must be filled.');
		
        $form['NationalityID']->addRule(Form::FILLED, 'Licenced Access must be filled.');
		
        $form->addSubmit('save', 'Save Details')
                ->onClick[] = callback($this, 'detailPersonFormSubmittedUpdate');
        if (!$this->isAllowed(self::RESOURCE_PERSON, self::ACTION_EDIT))
            $form['save']->setDisabled(TRUE);
        $form->addSubmit('delete', 'Delete')
                ->onClick[] = callback($this, 'detailPersonFormSubmittedDelete');
		$form['delete']->setAttribute('onclick', '
			return confirm("Are you sure to delete the person?");
		');
        if (!$this->isAllowed(self::RESOURCE_PERSON, self::ACTION_DELETE))
            $form['delete']->setDisabled(TRUE);
        $form->addSubmit('userRole', 'User Role')
                ->setValidationScope(FALSE)
                ->onClick[] = callback($this, 'userRoleClicked');
        if (!$this->testUserRoles('ADMIN'))        
            $form['userRole']->setDisabled(TRUE);
		
        return $form;
    }
    
    /**
     * Creates form for creating Person based on createComponentPersonForm.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentCreatePersonForm($name)
    {
        $form = $this->createComponentPersonForm($name);
		// setting of displayed content
		$all_SBE = $this->context->organizationUnitRepository->findAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_SBE);
		foreach ($all_SBE as $sbe)
        {
            if (!$sbe->Virtual)
		        $form->addCheckbox('SBE' . $sbe->ID, $sbe->Short)
				->setDefaultValue(true);
        }
		$form->addText('DisplayedContent', HPTForm::createLabel('Displayed Content', 'Current person is able to see only content of selected SBEs on sites where content is filtered.'));
		
        unset($form['EID']);
        $form->addText('EID', HPTForm::createLabel('EID', 'Must start with <strong>E</strong> followed by 6 digits or <strong>ENN</strong> followed by 4 digits'));
        $form['EID']
                ->setRequired('EID field is mandatory.')
                ->addRule(Form::PATTERN, 'EID must start with \'E\' followed by 6 digits or \'ENN\' followed by 4 digits.', '('.Person::ID_PREG_EXP.')');
        $form['Surname']->setRequired('Surname field is mandatory.');
        $form['CostCenter']
                ->setRequired('Please select Work Center.');
		
		$form['OrganizationUnitSelector']['SBE']
                ->addConditionOn($form['OrganizationUnitSelector']['Department'], ~Form::FILLED)
                ->elseCondition($form['OrganizationUnitSelector']['Cluster'], ~Form::FILLED)
                ->elseCondition($form['OrganizationUnitSelector']['TechnicalGroup'], ~Form::FILLED)
                        ->addRule(Form::FILLED, 'Organization Unit must be filled.');
        $form['OrganizationUnitSelector']['Department']
                ->addConditionOn($form['OrganizationUnitSelector']['SBE'], ~Form::FILLED)
                ->elseCondition($form['OrganizationUnitSelector']['Cluster'], ~Form::FILLED)
                ->elseCondition($form['OrganizationUnitSelector']['TechnicalGroup'], ~Form::FILLED)
                        ->addRule(Form::FILLED, 'Organization Unit must be filled.');
        $form['OrganizationUnitSelector']['Cluster']
                ->addConditionOn($form['OrganizationUnitSelector']['SBE'], ~Form::FILLED)
                ->elseCondition($form['OrganizationUnitSelector']['Department'], ~Form::FILLED)
                ->elseCondition($form['OrganizationUnitSelector']['TechnicalGroup'], ~Form::FILLED)
                        ->addRule(Form::FILLED, 'Organization Unit must be filled.');
        $form['OrganizationUnitSelector']['TechnicalGroup']
                ->addConditionOn($form['OrganizationUnitSelector']['SBE'], ~Form::FILLED)
                ->elseCondition($form['OrganizationUnitSelector']['Department'], ~Form::FILLED)
                ->elseCondition($form['OrganizationUnitSelector']['Cluster'], ~Form::FILLED)
                        ->addRule(Form::FILLED, 'Organization Unit must be filled.');
		
        $form['NationalityID']->addRule(Form::FILLED, 'Licenced Access must be filled.');
		
        $form->addSubmit('save', 'Save')
                ->onClick[] = callback($this, 'detailPersonFormSubmittedSave');
        if (!$this->isAllowed(self::RESOURCE_PERSON, self::ACTION_CREATE))
            $form['save']->setDisabled(TRUE);
        $form->addSubmit('userRole', 'User Role')
                ->setDisabled(TRUE);
        $form->addSubmit('delete', 'Delete')
                ->setDisabled(TRUE);
        if (!$this->isAllowed(self::RESOURCE_PERSON, self::ACTION_DELETE))
            $form['delete']->setDisabled(TRUE);
		
        return $form;
    }
    
    /**
     * Handler triggered when Reset button is pressed. It will drop
     * remembered filter.
     */
    public function filterPersonFormReset()
    {
		$this->context->formStateRepository->drop('filterPersonForm');
		$this->redirect('this');
    }
    
    /**
     * Redirection to create person form.
     */
    public function filterPersonFormCreate()
    {
		$this->redirect('Person:create');
    }
    
    /**
     * Person form submitted.
     * 
     * @param AppForm $form 
     */
    public function filterPersonFormSubmitted(AppForm $form)
    {
        if ($this->isAjax())
        {
            $this->payload->filter = $form->getValues();
            $this->invalidateControl('persons');
        }
    }
    
    /**
     * Person Form Submitted - Save.
     * 
     * @param SubmitButton $button 
     */
    public function detailPersonFormSubmittedSave(SubmitButton $button)
    {
        $entry = $button->form->getValues();
		if ($this->isAllowed(self::RESOURCE_PERSON, self::ACTION_CREATE))
		{
			if ($button->form->isValid())
			{
                $person = new Person();
                $person->setID(strtoupper($entry['EID']));
                $person->setSurname($entry['Surname']);
                $person->setFirstname($entry['Name']);
				    $orgUnit_id = NULL;
					if ($entry->OrganizationUnitSelector->TechnicalGroup)
						$orgUnit_id = $entry->OrganizationUnitSelector->TechnicalGroup;
					elseif ($entry->OrganizationUnitSelector->Cluster)
						$orgUnit_id = $entry->OrganizationUnitSelector->Cluster;
					elseif ($entry->OrganizationUnitSelector->Department)
						$orgUnit_id = $entry->OrganizationUnitSelector->Department;
					elseif ($entry->OrganizationUnitSelector->SBE)
						$orgUnit_id = $entry->OrganizationUnitSelector->SBE;
                $person->setOrganizationUnitID($orgUnit_id);
                $person->setCostCenterID($entry['CostCenter']);
                $person->setEmail($entry['Email']);
                $person->setComment($entry['Comment']);
				if($entry['NationalityID'])
					$person->setNationalityID($entry['NationalityID']);
				$person->setStatus(1);
                if ($this->context->personRepository->save($person))
                {
					$this->flashMessage("Person " . $person->ID . " saved.", 'success');	
					// setting of displayed content
					$success = TRUE;
					$all_SBE = $this->context->organizationUnitRepository->findAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_SBE);
					$all_setting = $this->context->settingDisplayedContentRepository->findByEID($person->ID);
					$existing_setting = array();
					foreach ($all_setting as $setting)
						$existing_setting[] = $setting->OrganizationUnitID;
					foreach ($all_SBE as $sbe)
					{
						if (!$sbe->Virtual)
						{
								$setting = new SettingDisplayedContent();
								$setting->EID = $person->ID;
								$setting->OrganizationUnitID = $sbe->ID;
								$success = $success && $this->context->settingDisplayedContentRepository->save($setting);
						}					
					}
					if (!$success)
					{
						$this->flashMessage('Setting of displayed content cannot be saved.', 'error');
					}
                }
                else
                {
                    $this->flashMessage("Saving failed. Person ID '" . $person->ID . "' already exists.", 'error');
                }
            }
			else
			{
				$this->flashMessage("Invalid form values.");
			}
		}
		else
		{
			$this->flashMessage("You are not authorized to save person. ADMIN role required.", 'error');
		}
        $this->redirect('Person:showDetail', $entry['EID']);
    }
    
    /**
     * Person Form Submitted - Update.
     * 
     * @param SubmitButton $button 
     */
    public function detailPersonFormSubmittedUpdate(SubmitButton $button)
    {
        $entry = $button->form->getValues();
        $deleteOldUser = false;
		if ($this->isAllowed(self::RESOURCE_PERSON, self::ACTION_EDIT))
		{
			if ($button->form->isValid())
			{
                $person = $this->context->personRepository->find($entry['EID']);
                $person->setSurname($entry['Surname']);
                $person->setFirstname($entry['Name']);
				    $orgUnit_id = NULL;
					if ($entry->OrganizationUnitSelector->TechnicalGroup)
						$orgUnit_id = $entry->OrganizationUnitSelector->TechnicalGroup;
					elseif ($entry->OrganizationUnitSelector->Cluster)
						$orgUnit_id = $entry->OrganizationUnitSelector->Cluster;
					elseif ($entry->OrganizationUnitSelector->Department)
						$orgUnit_id = $entry->OrganizationUnitSelector->Department;
					elseif ($entry->OrganizationUnitSelector->SBE)
						$orgUnit_id = $entry->OrganizationUnitSelector->SBE;
				$orgUnit = $this->context->organizationUnitRepository->find($orgUnit_id);
				if(!$orgUnit->Status && ($person->Status && ($entry['Status'] == 'Active')))
					$this->flashMessage("Person's " . $person->ID . " organization unit cannot be chaged. Organizaiton unit " . $orgUnit->Name . " is inactive.", 'error');
				else
					$person->setOrganizationUnitID($orgUnit_id);
				$person->setCostCenterID($entry['CostCenter']);
                $person->setEmail($entry['Email']);
                $person->setComment($entry['Comment']);
				if($entry['NationalityID'])
				{
					if($entry['NationalityID'] <> $person->NationalityID) // if nationality is changed, EC check must follow
					{
						$projects = $this->context->projectRepository->findPersonAssignedAtProjectsInYearAndFuture($person->ID, Date('Y'));
						foreach($projects as $project)
						{
							if(!$this->context->ecUtility->hasAccess($entry['NationalityID'], $project->ECLicenceID))
							{
								$tasks = $this->context->taskRepository->findAllProjectTasksInYearAndFuture($project->ID, Date('Y'));
								foreach ($tasks as $task)
								{
									if($task->LabourTaskFlag == 1)
									{
										$assignment = 0;
										$assigns = $this->context->personTaskAssignmentRepository->findPersonTaskYearAndFutureAssignment($task->ID, Date('Y'), $entry['EID']);
										foreach($assigns as $assign)
										{
											if($assign->Month >= (FiscalCalendar::currentFiscalMonth() + 1))
												$assignment += $assign->Hours;
										}
										if($assignment > 0)
										{
											$this->flashMessage('Licenced Access can not be changed. Person is assigned to project: ' . $project->Name . ' that requires secure access based on licence.', 'error');
											$this->redirect('Person:showDetail', $entry['EID']);
										}
									}
								}
							}
						}
						$person->setNationalityID($entry['NationalityID']);
					}
				}
				
				// setting of displayed content
				$success = TRUE;
				$all_SBE = $this->context->organizationUnitRepository->findAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_SBE);
				$all_setting = $this->context->settingDisplayedContentRepository->findByEID($person->ID);
				$existing_setting = array();
				foreach ($all_setting as $setting)
					$existing_setting[] = $setting->OrganizationUnitID;
				foreach ($all_SBE as $sbe)
				{
                    if (!$sbe->Virtual)
                    {
                        if (!$entry->{'SBE' . $sbe->ID} && in_array($sbe->ID, $existing_setting))
                            $success = $success && $this->context->settingDisplayedContentRepository->delete($person->ID, $sbe->ID);
                        elseif ($entry->{'SBE' . $sbe->ID} && !in_array($sbe->ID, $existing_setting))
                        {
                            $setting = new SettingDisplayedContent();
                            $setting->EID = $person->ID;
                            $setting->OrganizationUnitID = $sbe->ID;
                            $success = $success && $this->context->settingDisplayedContentRepository->save($setting);
                        }
                    }					
				}
				if (!$success)
				{
					$this->flashMessage('Setting of displayed content cannot be saved.', 'error');
				}				
				
                if(substr($person->ID, 0, 3) == "ENN" AND Person::validEID($entry['changeEID']))
                {
                    if($this->context->personRepository->find($entry['changeEID']))
                    {
                        $this->flashMessage("Person " . $entry['changeEID'] . " already exists in ATE TOOL.", 'error');
                    }
                    else
                    {
                        if($this->context->eidChangeAuthenticator->authenticate($entry['changeEID']))
                        {
                            if($this->context->personRepository->updateEID($entry['EID'], $entry['changeEID']))
                            {
								$person->ID = $entry['changeEID'];
                                $deleteOldUser = true;
                                $this->flashMessage("Person " . $person->ID . " updated.", 'success');
                            }
                            else
                            {
                                $this->flashMessage("Person " . $person->ID . " updated unsuccessfully.", 'error');
                            }
                        }
                        else
                        {
                            $this->flashMessage("Wrong EID. " . $entry['changeEID'] . " not found via LDAP.", 'error');
                        }
                    }
                }
                elseif (substr($person->ID, 0, 3) == "ENN" AND $entry['changeEID'] != "")
                {
                    $this->flashMessage('"'.$entry['changeEID'].'" is not valide EID', 'error');
                }
                else
                {
					$changeStatus = false;
					$from = null;
					if($entry['From'] != null)
					{
						// from inacive to active
						if($entry['Status'] == 'Active')
						{
							if(!$orgUnit->Status)
							{
								$this->flashMessage("Person " . $person->ID . " cannot be activated. Person belongs to inactive organizaiton unit.", 'error');
								$changeStatus = false;
							}
							else
							{
								$changeStatus = true;
								$person->setStatus(1);
							}
							//Inactive work center check
							$costCenter = $this->context->costCenterRepository->findCostCenter($person->CostCenterID);
							if(!$costCenter->Status)
							{
								$this->flashMessage("Person " . $person->ID . " cannot be activated. Person belongs to inactive work center: " . $person->CostCenterID . ".", 'error');
								$changeStatus = false;
							}
							else
							{
								$changeStatus = false;
								$person->setStatus(1);
							}
						}
						// from active to inactive
						else
						{
							$leadingOrgUnits = $this->context->organizationUnitRepository->findByLeader($person->ID);
							$canBeDeactivatedLeader = $canBeDeactivatedFutureAssignment = true;
							$listOfOrgUnits = array();
							foreach ($leadingOrgUnits as $orgUnitID => $orgUnit)
							{
								if($orgUnit->Status == 1)
								{
									$canBeDeactivatedLeader = false;
									$listOfOrgUnits[] = $orgUnit->Name;
								}
							}
							// Future assignment check
							$listOfProjects = Array();
							$projects = $this->context->projectRepository->findPersonAssignedAtProjectsInYearAndFuture($person->ID, Date('Y'));
							foreach($projects as $project)
							{
								$tasks = $this->context->taskRepository->findAllProjectTasksInYearAndFuture($project->ID, Date('Y'));
								foreach ($tasks as $task)
								{
									if($task->LabourTaskFlag == 1)
									{
										$assignment = 0;
										$assigns = $this->context->personTaskAssignmentRepository->findPersonTaskYearAndFutureAssignment($task->ID, Date('Y'), $entry['EID']);
										foreach($assigns as $assign)
										{
											if($assign->Month >= (FiscalCalendar::currentFiscalMonth() + 1))
												$assignment += $assign->Hours;
										}
										if($assignment > 0)
										{
											$canBeDeactivatedFutureAssignment = false;
											$listOfProjects[] = $project->Name;
										}
									}
								}
							}

							if(!$canBeDeactivatedLeader)
							{
								$this->flashMessage("Person " . $person->ReversedFullNameWithEID . " cannot be deactivated. Person is leading active organizaiton unit: " . implode(', ', $listOfOrgUnits) . '.', 'error');
							}
							if(!$canBeDeactivatedFutureAssignment)
							{
								$this->flashMessage("Person " . $person->ReversedFullNameWithEID . " cannot be deactivated. Person is assigned to project: " . implode(', ', $listOfProjects) . '.', 'error');
							}
							if($canBeDeactivatedLeader && $canBeDeactivatedFutureAssignment)
							{
								$changeStatus = true;
								$person->setStatus(0);
							}

						}
						$from = $entry['From'];
					}
                    if ($this->context->personRepository->update($person))
                    {
                        $this->flashMessage("Person " . $person->ID . " updated.", 'success');
						//START - set workable hours according to status
						if(($entry['From'] != null) && $changeStatus)
						{
							$fromYear = (int)substr($from, 0, 4);
							$fromMonth = (int)substr($from, -2);
							$toYear = max($this->context->aopRepository->getYears());
							for($year = $fromYear; $year <= $toYear; $year++)
							{
								$currentWH = $this->context->aopRepository->findForPerson($year, $person->ID);
								$aop = $this->context->aopRepository->find($year);
								$hour = new WorkableHour();
								$hour->setID($person->ID);
								$hour->setYear($year);
								// first year
								if ($year == $fromYear)
								{
									if($person->Status == 1)
									{
										for($i = 1; $i<13 ; $i++)
										{
											if($i < $fromMonth)
											{
												$hour->setMonthHours($currentWH->AOP[$i], $i);
											}
											else
											{
												$hour->setMonthHours($aop->AOP[$i], $i);
											}
										}
									}
									else
									{
										for($i = 1; $i<13 ; $i++)
										{
											if($i < $fromMonth)
											{
												$hour->setMonthHours($currentWH->AOP[$i], $i);
											}
											else
											{
												$hour->setMonthHours(0, $i);
											}
										}
									}
									$hour->IsAOP = 0;
								}
								// other years
								else
								{
									if($person->Status == 1)
									{
										$hour->setYield(100);
										$hour->setContract(100);
									}
									else
									{
										$hour->setYield(0);
										$hour->setContract(0);
									}
									$hour->IsAOP = 1;
								}
								// saving hours
								$origin = $this->context->workableHourRepository->search($hour->Year, $hour->ID);
								if ($origin)
								{
									if(is_null($hour->Yield) && is_null($hour->Contract))
									{
										$hour->setContract($this->context->workableHourRepository->getContract($person->ID, $year)*100);
										$hour->setYield($this->context->workableHourRepository->getYield($person->ID, $year)*100);
									}
									$this->context->workableHourRepository->update($hour);
								}
								else
								{
									$this->context->workableHourRepository->create($hour);
								}
							}
							$this->flashMessage("Workable hours of person: " . $hour->ID . " updated.", 'success');
						}
						//END - set workable hours according to status
                    }
                    else
                    {
                        $this->flashMessage("Person " . $person->ID . " updated unsuccessfully.", 'error');
                    }
                }
            }
			else
			{
				$this->flashMessage("Invalid form values.");
			}
		}
		else
		{
			$this->flashMessage("You are not authorized to update person. PMO role required.", 'error');
		}
        if($deleteOldUser)
            $this->redirect('Person:showDetail', $entry['changeEID']);
        else
            $this->redirect('Person:showDetail', $entry['EID']);
    }
    
    /**
     * Person Form Submitted - Delete.
     * @param SubmitButton $button 
     */
    public function detailPersonFormSubmittedDelete(SubmitButton $button)
    {
        Debugger::log('DetailPersonFormSubmittedDelete', Debugger::DEBUG);
        $entry = $button->form->getValues();
        $id = $entry['EID'];
		if ($this->isAllowed(self::RESOURCE_PERSON, self::ACTION_DELETE))
		{
            if ($this->context->personTaskAssignmentRepository->isPersonAssignedToAnyTasks($id))
            {
                $this->flashMessage('This Person can not be deleted. Person is assigned to one or more Tasks.', 'error');
            }
            else
            {
                if ($this->context->personRepository->delete($id))
                {
                    $this->flashMessage("Person $id deleted.", 'success');
                    $this->redirect('Person:default');
                }
                else
                {
                    $this->flashMessage("Error: Person " . $id . " not deleted.", 'error');
                }
            }
        }
		else
		{
			$this->flashMessage("You are not authorized to delete person. PMO role required.", 'error');
		}
        $this->redirect('Person:showDetail', $entry['EID']);
    }
    
    /**
     * User role button redirects to UserRole:default.
     * 
     * @param SubmitButton $button User role button.
     */
    public function userRoleClicked(SubmitButton $button)
    {
        $id = $this->getHttpRequest()->getQuery('id');
        $this->redirect('UserRole:default', $id);
    }
	
    /**
     * Creates form for showing workable hours.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentWorkableHourForm($name)
    {      
        $form = new AppForm($this, $name);

        for($i=0; $i<12; $i++)
        {
        $form->addHidden('H' . FiscalCalendar::fiscalMonthIndexToName($i, false))
                ->setAttribute('class', 'workableHoursInputHidden');
        }        
        $form->addHidden('EID');
        $form->addHidden('Year');
        $form->addHidden('isAop')
                ->setAttribute('class', 'isAop');
        $form->addCheckbox('AOP', 'AOP Calendar')
                ->setAttribute('class', 'workableHoursCheckBox');
        for($i=0; $i<12; $i++)
        {
            $form->addText(FiscalCalendar::fiscalMonthIndexToName($i, false), FiscalCalendar::fiscalMonthIndexToName($i, true))
                ->addConditionOn($form['AOP'], Form::EQUAL, FALSE)
                    ->addRule(Form::PATTERN, 'Hours must be positive integer.', '[0-9]+');
                $form[FiscalCalendar::fiscalMonthIndexToName($i, false)]->setAttribute('size', '3')
                ->setAttribute('class', 'workableHoursInputText');
        }
		
		$per = array();
		for($i = 0; $i <= 100; $i++)
		{
			$per[$i] = $i . '%';
		}
        $form->addSelect('Yield', 'Yield', $per)
                ->setHtmlId("yield");
		
		$form->addSelect('YieldRecalculation', 'Yield', $per)
                ->setHtmlId("yieldRecalculation");
		
		$form->addSelect('Contract', 'Contract', $per)
                ->setHtmlId("contract");
		
		$form->addTextArea('Remark', 'Workable hours remark', 35, 5);
        
        $form->addSubmit('Save', 'Save Workable Hours');
		if($this->year && $this->year < date('Y'))
            $form['Save']->setDisabled(TRUE);		
		
        $form->onSuccess[] = array($this, 'workableHourFormSubmittedSave');        
        return $form;
    }
    
    /**
     * Workable hour form submitted.
     * 
     * @param AppForm $form 
     */
    public function workableHourFormSubmittedSave($form)
    {
        if($form->isValid())
        {
            $edit = $form->getValues();
            $hour = new WorkableHour();
            $hour->setID($edit['EID']);
            $hour->setYear($edit['Year']);
			$hour->setRemark($edit['Remark']);
            if ($edit['Jan'] != null)
            {
                for($i = 1; $i<13 ; $i++)
                {
                    $hour->setMonthHours(ceil($edit[ FiscalCalendar::fiscalMonthIndexToName($i-1, false) ]), $i);
                }
            }
            $hour->setYield($edit['Yield']);
			$hour->setContract($edit['Contract']);
            if($edit['AOP']==false)
            {
				$hour->IsAOP = 0;
                if ($this->context->workableHourRepository->search($hour->Year, $hour->ID))
                {
                    $this->context->workableHourRepository->update($hour);
                    $this->flashMessage("Workable hours of person: '" . $hour->ID . "' updated.", 'success');
                }
                else
                {
                    $this->context->workableHourRepository->create($hour);
                    $this->flashMessage("Workable hours of person: '" . $hour->ID . "' saved.", 'success');
                }
            }
            else
            {
				$hour->IsAOP = 1;
                if($hour->Contract == 100 && $hour->Yield == 100)
                {
                    if ($this->context->workableHourRepository->search($hour->Year, $hour->ID))
                    {
                        $this->context->workableHourRepository->delete($hour->Year, $hour->ID);
                        $this->flashMessage("Workable hours of person: '" . $hour->ID . "' deleted.", 'success');
                    }
                    else
                    {
                        $this->flashMessage("Workable hours of person: '" . $hour->ID . "' was already deleted.", 'success');
                    }
                }
                else
                {
                    if ($this->context->workableHourRepository->search($hour->Year, $hour->ID))
                    {
                        $this->context->workableHourRepository->update($hour);
                        $this->flashMessage("Workable hours of person: '" . $hour->ID . "' updated.", 'success');
                    }
                    else
                    {
                        $this->context->workableHourRepository->create($hour);
                        $this->flashMessage("Workable hours of person: '" . $hour->ID . "' saved.", 'success');
                    }
                }
            }
        }
        else
        {  
            $this->flashMessage("Invalid form values.");
        }
         $this->redirect('Person:showDetail', $edit['EID']);        
    }
	
    /**
     * Creates form for assigned and actuals hours.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentAssignedAndActualHoursForm($name)
    {
        $form = new AppForm($this, $name);

        $actualYear = Date('Y');
        $actualMonth = FiscalCalendar::getFiscalMonthByTimestamp(time())->index+1;
       	$projects = $allPersonTasks = array();
		$projects = $this->context->projectRepository->findPersonAssignedAtProjectsInYear($this->id, $this->year); 
		// lest find projects where persons are not assigned
		$notAssignedProjects = $this->context->projectRepository->findPersonNotAssignedAtProjectsInYear($this->id, $this->year);
		foreach($notAssignedProjects as $project)
		{
			if(!isset($projects[$project->ID]))
				$projects[$project->ID] = $project;  
		}
        $assignedAndActualHours = array();
        
        foreach ($projects as $project)
        {
            $tasks = $this->context->taskRepository->findPersonTasksInYear($this->id, $project->ID, $this->year);
			$activeTasks = $this->context->taskRepository->findPersonActiveTasksInYear($this->id, $project->ID, $this->year);
			foreach($activeTasks as $task)
			{
				if(!isset($tasks[$task->ID]))
				{
					$tasks[$task->ID] = $task;
				}
			}
			$allPersonTasks += $tasks; 
            foreach($tasks as $task)
            {
                $assigned = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($task->ID, $this->year, $this->id);
                $actual = $this->context->workRepository->findPersonTaskYearWork($this->id, $task->ID, $this->year);

                foreach($assigned as $month => $hour)
                {
					if($this->year == Date('Y'))
					{
						if($month >= $actualMonth)
						{
							$assignedAndActualHours[$project->ID][$task->ID][$month] = $hour->Hours;
						}
						else
						{
							$assignedAndActualHours[$project->ID][$task->ID][$month] = $actual[$month];
						}
					}
					elseif($this->year > Date('Y'))
					{
						$assignedAndActualHours[$project->ID][$task->ID][$month] = $hour->Hours;
					}
					else
					{
						$assignedAndActualHours[$project->ID][$task->ID][$month] = $actual[$month];
					}
                }
            }
        }
		$person = $this->context->personRepository->find($this->id);
        foreach($assignedAndActualHours as $projectName => $project)
        {
			$projectObj = $this->context->projectRepository->find($projectName);
			$projectState = $projectObj->State;
            foreach($project as $taskName => $task)
            {
                foreach($task as $month => $hour)
                {
                    if($this->year < $actualYear || $projectState != 1)
                    {
                        $form->addtext($projectName . '_' . $taskName . '_' . $month, NULL, 3)
                            ->setDefaultValue($assignedAndActualHours[$projectName][$taskName][$month])
                            ->setAttribute('readonly', 'readonly');
                    }
                    elseif($this->year == $actualYear)
                    {
                        if($month >= $actualMonth)
						{
                            $form->addText($projectName . '_' . $taskName . '_' . $month, NULL, 3)
                                    ->setDefaultValue($assignedAndActualHours[$projectName][$taskName][$month])
                                    ->addRule(Form::FLOAT, 'Assigned hours must be float.')
                                    ->addRule(Form::RANGE, 'Assigned hours must be positive number from 0 to 9999.', array(0, 9999));
						}
                        else
                            $form->addtext($projectName . '_' . $taskName . '_' . $month, NULL, 3)
                                ->setDefaultValue($assignedAndActualHours[$projectName][$taskName][$month])
                                ->setAttribute('readonly', 'readonly');
                    }
                    else
                    {
                        $form->addText($projectName . '_' . $taskName . '_' . $month, NULL, 3)
                                ->setDefaultValue($assignedAndActualHours[$projectName][$taskName][$month])
                                ->addRule(Form::FLOAT, 'Assigned hours must be float.')
                                ->addRule(Form::RANGE, 'Assigned hours must be positive number from 0 to 9999.', array(0, 9999));
                    }
                }
            }
        }

		$buttonDisabled = false;
		$messageDisabled = array();
		$form->addSubmit('save', 'Save Worker Resource Plan');
		if($person->Locked && $this->isAllowed(self::RESOURCE_RESOURCE_LOCK, self::ACTION_RUN))
			$form['save']->setAttribute('onclick', '
				return confirm("Worker is locked. Are you sure to update worker resource plan?");
			');
        if($this->year < date('Y'))
		{
			$buttonDisabled = true;
			$messageDisabled[] = TaskPlanEditing::TASK_PLAN_EDITING_MESSAGE_YEAR;
            $form['save']->setDisabled(TRUE);
		}
		if($person->Locked)
		{
			if(!($this->isAllowed(self::RESOURCE_RESOURCE_LOCK, self::ACTION_RUN) || ($person->SupervisorID == strtoupper(Environment::getUser()->getId()))))
			{
				$buttonDisabled = true;
				$messageDisabled[] = AssignmentControl::FORBIDEN_ASSIGNMENT_LOCKED_PERSON_MESSAGE;
				$form['save']->setDisabled(TRUE);
			}
		}
		if(!$person->Status)
		{
			$buttonDisabled = true;
			$messageDisabled[] = AssignmentControl::FORBIDEN_ASSIGNMENT_INACTIVE_PERSON_MESSAGE;
			$form['save']->setDisabled(TRUE);
		}
		if($buttonDisabled)
			$form['save']->setAttribute('data-form-button-tooltip', implode(', ', $messageDisabled). '. ' . AssignmentControl::FORBIDEN_ASSIGNMENT_FINAL_MESSAGE);
        $form->onSuccess[] = array($this, 'assignedAndActualHoursFormSubmitted');
        
        return $form;
    }
    
    /**
     * Assigned and actuals hours form submitted.
     * 
     * @param AppForm $form 
     */
    public function assignedAndActualHoursFormSubmitted(AppForm $form)
    {
        $actualMonth = FiscalCalendar::getFiscalMonthByTimestamp(time())->index+1;
        $values = $form->getValues();
        $tasks_id = array();
        $assigned = array();
        $i = 0;

        foreach($values as $task_id => $value)
        {
            $assigned[] = $value;
            if(!($i % 12))
            {
                $a = strpos($task_id, '_') + 1;
                $b = strrpos($task_id, '_');
                $tasks_id[] = (substr($task_id, $a, $b - $a));
            }
            $i++;
        }
		$i = 0;
		$k = 0;
        $success = false;
        foreach($tasks_id as $task_id)
        {
			if(!($i % 12))
				$k = 0;
            $person_task = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($task_id, $this->year, $this->id);
            foreach($person_task as $month)
            {
                if ($k+1 >= $actualMonth && $this->year == Date('Y'))
                    $month->setHours($assigned[$i]);
				elseif($this->year > Date('Y'))
					$month->setHours($assigned[$i]);
				$k++;
                $i++;
            }
            $j = 1;
            foreach($person_task as $month)
            {
                if ($j >= $actualMonth && $this->year == Date('Y'))
                {
                    if ($this->context->personTaskAssignmentRepository->save($month))
                    {
                        $success = true;
                    }
                    else
                    {
                        $success = false;
                    }
                    if(!$success)
                        break 2;
                }
				elseif($this->year > Date('Y'))
                {
                    if ($this->context->personTaskAssignmentRepository->save($month))
                    {
                        $success = true;
                    }
                    else
                    {
                        $success = false;
                    }
                    if(!$success)
                        break 2;
                }
                $j++;
            }
        }

        if($success)
        {
            $this->flashMessage("Worker resource plan updated.", 'success');
        }
        else
        {
            $this->flashMessage("Worker resource plan updating failed.", 'error');
        }
            
        $this->redirect('Person:showDetail', $this->id);
    }
	
	/**
	 * Prepares data for organization unit selector
	 * @param int $id ID of selected organization unit
	 */
	public function actionOrganizationUnitPersonSelector($id)
	{
		$org_unit_selector = new OrganizationUnitPersonSelectorContainer;
		$this->payload->organizationUnitSelectorResult = $org_unit_selector->ajaxHandle($id);
		$this->terminate();
	}
	
    /**
     * Creates component Task assignment trasfer form.
     * 
     * @param string $name Name of the form.
     * @return \AppForm Instance of the form.
     */
    public function createComponentPersonAssignmentTransferForm($name)
    {
        $form = new AppForm($this, $name);
        
		$eid = $this->getHttpRequest()->getQuery('id');
		
//		$form->addCheckbox('DeletePreviousAssignment', HPTForm::createLabel('Delete task previous assignment', 'If checked, tool deletes all previous assignement on targeted task.'));
		$form->addSelect2('AssignmentTransferTo', 'Transfer assignment to task')
				->setAttribute('style', 'width: 400px;')
				->setPrompt('*')
				->setRequired('Please select Transfer assignment to Task!');
		
		$tasksAvailableForAssignmentTransfer = $this->context->taskRepository->findAllTasksWhereIsPersonAssigned($eid);
		$taskItems = array();
		foreach($tasksAvailableForAssignmentTransfer as $task)
		{
			$taskItems[$task->ID] = $task->Name . ' (' . $task->SAPID . ')';
		}
		$form->addSelect2('AssignmentTransferFrom', 'Transfer assignment from task', $taskItems)
				->setAttribute('style', 'width: 400px;')
				->setPrompt('*')
				->setRequired('Please select Transfer assignment to Task!');
        
        return $form;
    }
	
	/**
	 * Signal filters tasks for assignment transfer.
	 * 
	 * @param type $q
	 * @param type $year
	 */
	public function handlePersonAssignmentTransferSelect($q, $year)
	{
		// Gets tasks with task plan for given year
		$taskItems = $items = array();
		$text = trim($q);
		$taskWithPlanForYear = $this->context->taskRepository->findLabourTaskWithTaskPlanForYear($year);
		foreach ($taskWithPlanForYear as $task)
		{
			if($text !== '')
			{
				if((stripos($task->Name, $text) !== false) || (stripos($task->SAPID, $text)!== false))
				{
					$taskItems = array('id' => $task->ID, 'text' => $task->Name . ' (' . $task->SAPID . ')', 'disabled' => false);
					$items[] = $taskItems;
				}
			}
			else
			{
				$taskItems = array('id' => $task->ID, 'text' => $task->Name . ' (' . $task->SAPID . ')', 'disabled' => false);
				$items[] = $taskItems;	
			}
		}
		
		$this->payload->items = $items;
		
		$this->terminate();
	}
    
	/**
	 * Signal transfer person assignment.
	 * 
	 * @param int $fromID From TaskID.
	 * @param int $toID To TaskID.
	 */
	public function handlePersonAssignmentTransferAction($fromID, $toID)
	{
		$fromTask = $this->context->taskRepository->find($fromID);
		$toTask = $this->context->taskRepository->find($toID);
		$eid = $this->getHttpRequest()->getQuery('id');
		if(is_null($fromTask))
		{
			$this->flashMessage("Task with ID " . $fromID . " was not found. ", 'error');
			$this->redirect('this');
		}
		if(is_null($toTask))
		{
			$this->flashMessage("Task with ID " . $toID . " was not found. ", 'error');
			$this->redirect('this');
		}
		$fromProject = $this->context->projectRepository->findProjectByTaskId($fromTask->ID);
		$toProject = $this->context->projectRepository->findProjectByTaskId($toTask->ID);
		if(is_null($fromProject))
		{
			$this->flashMessage("Task " . $fromTask->Name . " project was not found. ", 'error');
			$this->redirect('this');
		}
		if(is_null($toProject))
		{
			$this->flashMessage("Task " . $toTask->Name . " project was not found. ", 'error');
			$this->redirect('this');
		}
		
		// get task plan editing control object for from task
		$fromTaskPlanEditing = new TaskPlanEditing($fromTask, Date('Y'));
		// get task plan editing control object for to task
		$toTaskPlanEditing = new TaskPlanEditing($toTask, Date('Y'));
		
		if(!$fromTaskPlanEditing->editable)
		{
			$this->flashMessage('Origin task do not allow assignment transfer. ' . $fromTaskPlanEditing->message, 'error');
			$this->redirect('this');
		}
		
		if(!$toTaskPlanEditing->editable)
		{
			$this->flashMessage('Targeted task do not allow assignment transfer. ' . $toTaskPlanEditing->message, 'error');
			$this->redirect('this');
		}
		
		// chcek person
		$person = $this->context->personRepository->find($eid);
		if(is_null($person))
		{
			$this->flashMessage("Person with EID " . $eid . " was not found. ", 'error');
			$this->redirect('this');
		}
		$assignmentControl = new AssignmentControl($person, $toTask);
		if(!$assignmentControl->assignable)
		{
			$this->flashMessage($person->ReversedFullNameWithEID . " assignment was not transferred. " . $assignmentControl->Message, 'error');
			$this->redirect('this');
		}
		$atleastOneYearTransferred = false;
		$first_updateable_month = FiscalCalendar::currentFiscalMonth() + 1;
		$taskYears = $this->context->taskRepository->findTaskYears($fromID);
		foreach($taskYears as $trasferedYear)
		{
			if($trasferedYear >= Date('Y'))
			{
				if(is_null($this->context->taskPlanRepository->findTaskYearPlan($toTask->ID, $trasferedYear)))
				{
					$task_plan = new TaskYearPlan($trasferedYear);
					$task_plan->TaskID = $toTask->ID;
					$this->context->taskPlanRepository->save($task_plan);
				}
				// trasfer person task assignment
				$soruceAssignemnt = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($fromID, $trasferedYear, $eid);
				$targetAssignment = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($toID, $trasferedYear, $eid);
				$success = false;
				for($i = 1; $i < 13; $i++)
				{
					if($trasferedYear == Date('Y'))
					{
						if($first_updateable_month <= $i)
						{
							if($soruceAssignemnt[$i]->hours > 0)
							{
								$targetAssignment[$i]->hours += $soruceAssignemnt[$i]->hours;
								if($this->context->personTaskAssignmentRepository->save($targetAssignment[$i]))
								{
									$success = true;
								}
							}
						}
					}
					else
					{
						if($soruceAssignemnt[$i]->hours > 0)
						{
							$targetAssignment[$i]->hours += $soruceAssignemnt[$i]->hours;
							if($this->context->personTaskAssignmentRepository->save($targetAssignment[$i]))
							{
								$success = true;
							}
						}
					}
				}
				if($success)
				{
					$this->flashMessage($person->ReversedFullNameWithEID . ' assignment for year: ' . $trasferedYear . ' was transferred successfully.', 'success');
					for($i = 1; $i < 13; $i++)
					{
						if($trasferedYear == Date('Y'))
						{
							if($first_updateable_month <= $i)
							{
								$this->context->personTaskAssignmentRepository->deleteByPersonAndYearAndMonth($eid, $fromID, $trasferedYear, $i);
							}
						}
						else
						{
							$this->context->personTaskAssignmentRepository->deleteByPersonAndYearAndMonth($eid, $fromID, $trasferedYear, $i);
						}
					}
					$atleastOneYearTransferred = true;
				}
			}
		}
		if($atleastOneYearTransferred)
			$this->redirect('this');
		else
		{
			$this->flashMessage('There is no assignment to transfer.', 'warning');
			$this->redirect('this');
		}
	}
}
