<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing actuals cost from database using dibi.
 *
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 */
class ActualsCostDibiMapper extends DibiMapper implements IActualsCostMapper
{

    /**
     * Creates ActualsCost instance from DibiRow.
     * 
     * @param DibiRow $row
     * @return	ActualsCost 
     * @todo rewrite using annotations
     */
    private function load(DibiRow $row)
    {
        $actualsCost = new ActualsCost();
        $actualsCost->Project_sap_id    = $row->Project_SAP_ID;
        $actualsCost->Task_id	        = $row->Task_ID;
        $actualsCost->Activity_name     = $row->ActivityName;
        $actualsCost->Creation_date     = $row->CreationDate;
        $actualsCost->Posting_date      = $row->PostingDate;
        $actualsCost->Year              = $row->Year;
        $actualsCost->Fiscal_month      = $row->FiscalMonth;
        $actualsCost->Description       = $row->Description;
        $actualsCost->Category          = $row->Category;
		$actualsCost->Quantity          = $row->Quantity;
        $actualsCost->Value_usd         = $row->Value_USD;
        $actualsCost->Value_czk         = $row->Value_CZK;
        $actualsCost->Value             = $row->Value;
        $actualsCost->Currency          = $row->Currency;
		$actualsCost->Source            = $row->Source;
        return $actualsCost;
    }

    /**
     * Returns all actuals costs.
     * 
     * @return array Array of actuals costs.
     */
    public function findAll()
    {
        $actualsCost_db_result = $this->conn->query('SELECT * FROM [ActualsCost]')->fetchAll();
        $actualsCost = array();
        foreach ($actualsCost_db_result as $row)
        {
                $actualCost = $this->load($row);
                $actualsCost[] = $actualCost;
        }
        return $actualsCost;
    }

    /**
     * Returns all actuals costs for single task
     * 
     * @return array Array of arrays (years) of arrays (fiscal months)
     * of Actuals Cost records for single task.
     */
    public function findAllForTask($id)
    {
        $actualsCost_db_result = $this->conn->query('SELECT * FROM [ActualsCost] WHERE [Task_ID] = %s', $id)->fetchAll();
        $actualsCosts = array();
        foreach ($actualsCost_db_result as $row)
        {
                $actualsCost = $this->load($row);
                $actualsCosts[$actualsCost->Year][$actualsCost->Fiscal_month + 1][] = $actualsCost;
        }
        return $actualsCosts;
    }
	
    /**
     * Returns agregated actuals costs for single task
     * 
	 * @param int $id ID of a Task object
	 * @param string $currency Currency code (only USD or CZK accepted)
     * @return array Array of arrays (years) of arrays (fiscal months)
     * of Actuals Cost(just numbers)
     */
    public function findForTask($id, $currency)
    {
		if (!(($currency == 'USD') || ($currency == 'CZK')))
			throw new InvalidArgumentException('Unsupported currency code. Only USD and CZK supported. Invalid value:' . $currency);
        $actualsCost_db_result = $this->conn->query('SELECT * FROM [ActualsCost] WHERE [Task_ID] = %s', $id)->fetchAll();
        $actualsCosts = array();
        foreach ($actualsCost_db_result as $row)
        {
                $actualsCost = $this->load($row);
				if(!isset($actualsCosts[$actualsCost->Year][$actualsCost->Fiscal_month + 1]))
					$actualsCosts[$actualsCost->Year][$actualsCost->Fiscal_month + 1] = 0;
				if($currency == 'USD')
					$actualsCosts[$actualsCost->Year][$actualsCost->Fiscal_month + 1] += $actualsCost->Value_usd;
				elseif($currency == 'CZK')
					$actualsCosts[$actualsCost->Year][$actualsCost->Fiscal_month + 1] += $actualsCost->Value_czk;
		}
        return $actualsCosts;
    }

    /**
     * Returns single actuals cost record.
     * --- MOSTLY FOR DEBUGGING ---
     * 
     * @return ActualsCost Single ActualsCost record.
     */
    public function findSingle()
    {
        $actualsCost_db_row = $this->conn->query('SELECT * FROM [ActualsCost]')->fetch();
        $actualsCost = $this->load($actualsCost_db_row);
        return $actualsCost;
    }
    
    /**
	 * Returns all task years by task id.
	 * 
	 * @param int $id Task ID.
	 * @return	array Task duration according to actuals.
	 */
	public function findTaskYears($id)
	{
		$db_result = $this->conn->query('SELECT DISTINCT [Year] FROM [ActualsCost] WHERE [Task_ID] = %s ORDER BY [Year] ASC', $id)->fetchAll();
		
        $years = array();
		foreach ($db_result as $row)
		{
			$years[] = $row->Year;
		}
        
		return $years;
	}
	
	/**
	 * Function drops data from selected year.
	 * 
	 * @param int $year
	 * @return boolean Returns true if success, otherwise returns false.
	 */
	public function deleteDataFromYear($year, $source)
	{
		try
		{
		    $result = $this->conn->delete('ActualsCost')->where('[Year] = %i AND [Source] = %i', $year, $source)->execute();
			if (is_numeric($result))
				Debugger::log("Actuals costs for year = " . $year . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting Actuals costs for year = " . $year . " failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting Actuals costs for year = " . $year . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Function drops data from selected year and for given task.
	 * 
	 * @param int $taskID ID of a Task object
	 * @param int $year
	 * @param int $source Data source (e.g. SAP, IASAP, manual)
	 * @return boolean Returns true if success, otherwise returns false.
	 */
	public function deleteDataFromTaskYear($taskID, $year, $source)
	{
		try
		{
		    $result = $this->conn->delete('ActualsCost')->where('[Task_ID] = %s AND [Year] = %i AND [Source] = %i', $taskID, $year, $source)->execute();
			if (is_numeric($result))
				Debugger::log("Actuals costs for year = " . $year . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting Actuals costs for year = " . $year . " failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting Actuals costs for year = " . $year . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Function drops data for given task.
	 * 
	 * @param int $taskID
	 * @return boolean Returns true if success, otherwise returns false.
	 */
	public function deleteDataForTask($taskID)
	{
		try
		{
		    $result = $this->conn->delete('ActualsCost')->where('[Task_ID] = %s', $taskID)->execute();
			if (is_numeric($result))
				Debugger::log("Actuals costs for task = " . $taskID . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting Actuals costs for task = " . $taskID . " failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting Actuals costs for task = " . $taskID . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Function drops data for given task.
	 * 
	 * @param int $taskID
	 * @param int $source Data source (e.g. SAP, IASAP, manual)
	 * @return boolean Returns true if success, otherwise returns false.
	 */
	public function deleteDataForTaskAndSource($taskID, $source)
	{
		try
		{
		    $result = $this->conn->delete('ActualsCost')->where('[Task_ID] = %s AND [Source] = %i', $taskID, $source)->execute();
			if (is_numeric($result))
				Debugger::log("Actuals costs for task = " . $taskID . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting Actuals costs for task = " . $taskID . " failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting Actuals costs for task = " . $taskID . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Deletes records associated with given charge numbers in given range.
	 * 
	 * @param DateTime $startDate Delete from.
	 * @param DateTime $endDate Delete to.
	 * @param array $taskIDs List of Task IDs.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function deleteIASAPImport(DateTime $startDate, DateTime $endDate, array $taskIDs)
	{
		if (((int) $startDate->format('U')) > ((int) $endDate->format('U')))
			return TRUE;
		if (!count($taskIDs))
			return TRUE;
		
		$start_year = $startDate->format('Y');
		$end_year = $endDate->format('Y');
		$start_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $startDate->format('U'))->getIndex();
		$end_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $endDate->format('U'))->getIndex();
		$start_month_idx = ($start_year - 1) * 12 + $start_month + 1;
		$end_month_idx = ($end_year - 1) * 12 + $end_month + 1;
		
		try
		{
			$db_result = $this->conn->delete('ActualsCost')
					->where('(([Year] - 1) * 12 + [FiscalMonth] + 1) >= %i AND (([Year] - 1) * 12 + [FiscalMonth] + 1) <= %i', $start_month_idx, $end_month_idx)
					->where('[Task_ID] IN %in', $taskIDs)
					->where('[Source] = %i', ActualsCost::SOURCE_IASAP)
					->execute();
			
			if ($db_result)
			    Debugger::log('Deleting iASAP actuals cost import from ' . $start_year . '-' . $start_month . ' to ' . $end_year . '-' . $end_month, Debugger::INFO);
			elseif ($db_result !== 0)
				Debugger::log('Deleting iASAP actuals cost import from ' . $start_year . '-' . $start_month . ' to ' . $end_year . '-' . $end_month . ' failed', Debugger::ERROR);
			return ($db_result || ($db_result === 0)) ? TRUE : FALSE;
		}
		catch (DibiException $e)
		{
			Debugger::log('Deleting iASAP actuals cost import from ' . $start_year . '-' . $start_month . ' to ' . $end_year . '-' . $end_month . ' failed: ' . $e->getMessage(), Debugger::ERROR);
			return FALSE;
		}
	}
	
	/**
	 * Function imports data from array where items are ActualsCost objects.
	 * 
	 * @param array $actualCostsArray array of ActualsCost
	 * @param int $year
	 */
	public function importActualsCost($actualCostsArray)
	{
		$importArray = array();
        try
        {
			foreach($actualCostsArray as $actualsCost)
			{
				$array = array();
				$array['Project_SAP_ID'] = $actualsCost->Project_sap_id;
				$array['Task_ID'] = $actualsCost->Task_id;
				$array['ActivityName'] = $actualsCost->Activity_name;
				$array['CreationDate'] = $actualsCost->Creation_date;
				$array['PostingDate'] = $actualsCost->Posting_date;
				$array['Year'] = $actualsCost->Year;
				$array['FiscalMonth'] = $actualsCost->Fiscal_month;
				$array['Description'] = $actualsCost->Description;
				$array['Category'] = $actualsCost->Category;
				$array['Quantity'] = $actualsCost->Quantity;
				$array['Value_USD'] = $actualsCost->Value_usd;
				$array['Value_CZK'] = $actualsCost->Value_czk;
				$array['Value'] = $actualsCost->Value;
				$array['Currency'] = $actualsCost->Currency;
				switch ($actualsCost->Source)
				{
					case ActualsCost::SOURCE_IASAP:
						$array['Source'] = ActualsCost::SOURCE_IASAP;
						break;
					case ActualsCost::SOURCE_MANUAL:
						$array['Source'] = ActualsCost::SOURCE_MANUAL;
						break;
					default:
						$array['Source'] = ActualsCost::SOURCE_SAP;
						break;
				}
				$importArray[] = $array;
			}
            $import_db_result = $this->conn->query('INSERT INTO [ActualsCost] %ex', $importArray);
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Importing actuals cost failed. " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Saves actuals cost
	 * 
	 * @param ActualsCost $actualsCost
	 * @return boolean Returns true if cost center is successfully saved, otherwise false.
	 */
	public function save(ActualsCost $actualsCost)
	{
        try
        {
			$array = array();
			$array['Project_SAP_ID'] = $actualsCost->Project_sap_id;
			$array['Task_ID'] = $actualsCost->Task_id;
			$array['ActivityName'] = $actualsCost->Activity_name;
			$array['CreationDate'] = $actualsCost->Creation_date;
			$array['PostingDate'] = $actualsCost->Posting_date;
			$array['Year'] = $actualsCost->Year;
			$array['FiscalMonth'] = $actualsCost->Fiscal_month;
			$array['Description'] = $actualsCost->Description;
			$array['Category'] = $actualsCost->Category;
			$array['Quantity'] = $actualsCost->Quantity;
			$array['Value_USD'] = $actualsCost->Value_usd;
			$array['Value_CZK'] = $actualsCost->Value_czk;
			$array['Value'] = $actualsCost->Value;
			$array['Currency'] = $actualsCost->Currency;
			switch ($actualsCost->Source)
			{
				case ActualsCost::SOURCE_IASAP:
					$array['Source'] = ActualsCost::SOURCE_IASAP;
					break;
				case ActualsCost::SOURCE_MANUAL:
					$array['Source'] = ActualsCost::SOURCE_MANUAL;
					break;
				default:
					$array['Source'] = ActualsCost::SOURCE_SAP;
					break;
			}
            $import_db_result = $this->conn->insert('ActualsCost', $array)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Importing actuals cost failed. " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Deletes records associated with given task ID.
	 * 
	 * @param int $taskID Task ID.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function deleteIASAPActualsByTaskID($taskID)
	{
		try
		{
			$db_result = $this->conn->delete('ActualsCost')
					->where('[Task_ID] = %s', $taskID)
					->where('[Source] = %i', ActualsCost::SOURCE_IASAP)
					->execute();
			
			if ($db_result)
			    Debugger::log('Deleting iASAP actuals cost for task with ID: ' . $taskID, Debugger::INFO);
			elseif ($db_result !== 0)
				Debugger::log('Deleting iASAP actuals cost for task with ID: ' . $taskID . ' failed', Debugger::ERROR);
			return ($db_result || ($db_result === 0)) ? TRUE : FALSE;
		}
		catch (DibiException $e)
		{
			Debugger::log('Deleting iASAP actuals cost for task with ID: ' . $taskID . ' failed' . $e->getMessage(), Debugger::ERROR);
			return FALSE;
		}
	}
}
