<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2015 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing actuals for IASAP from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ActualsForIASAPDibiMapper extends DibiMapper implements IActualsForIASAPMapper
{

    /**
     * Creates ActualsForIASAP instance from DibiRow.
     * 
     * @param DibiRow $row
     * @return	ActualsForIASAP 
     * @todo rewrite using annotations
     */
    private function load(DibiRow $row)
    {
        $actuals = new ActualsForIASAP();
        $actuals->Eid             = $row->EID;
        $actuals->Charge_number   = $row->ChargeNumber;
        $actuals->Year            = $row->Year;
        $actuals->Fiscal_month    = $row->FiscalMonth;
        $actuals->Hours           = $row->Hours;
        $actuals->Date            = $row->Date;
		$actuals->ChargeDay       = $row->ChargeDay;
		$actuals->ModifiedAt      = $row->ModifiedAt;
        $actuals->Comment         = $row->Comment;
        return $actuals;
    }	

    /**
     * Deletes records from actuals by range (year & fiscal month) INCLUSIVELY.
	 * Dates are "rounded" to month indexes, so all data for all complete and partial
	 * months within date range are included. 
	 * 
	 * Methods translates $startDate and $endDate to indexes/numbers of months
	 * from year 0 using formula:
	 * (Year - 1) * 12 + Month 
	 * or (for fiscal months indexed in range <0,11> as in database
	 * (Year - 1) * 12 + FiscalMonth + 1
	 * 
	 * Examples of translation of date:
	 * "1st Jan 1" => 1
	 * "2nd Jan 1" => 1
	 * "5th May 10" => 113
	 * "30th Dec 2012" => 24144
	 * "1st Jan 2013" => 24145
	 * 
	 * Example of deleted data:
	 * $startDate is "2nd Feb 2012" and $endDate is "20th Feb 2012" => data for Feb 2012 are deleted
	 * $startDate is "1nd Feb 2012" and $endDate is "30th November 2012" => data from Feb 2012 to Nov 2012 are deleted
	 * $startDate is "3rd Feb 2012" and $endDate is "30th November 2013" => data from Feb 2012 to Dec 2012 and data from Jan 2013 to Nov 2013 are deleted
	 * $startDate is "3rd Feb 2012" and $endDate is "3rd Feb 2012" => data for Feb 2012 are deleted
     * 
     * @param DateTime $startDate
     * @param DateTime $endDate
     */    
    public function deleteByRange(DateTime $startDate, DateTime $endDate)
    {
		if (((int) $startDate->format('U')) > ((int) $endDate->format('U'))) // PHP 5.2.0 compliant test
			return;
		// start
        $start_year = $startDate->format('Y');
        $end_year = $endDate->format('Y');
		$start_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $startDate->format('U'))->getIndex();
		$end_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $endDate->format('U'))->getIndex();
		$start_month_idx = ($start_year - 1) * 12 + $start_month + 1;
		$end_month_idx = ($end_year - 1) * 12 + $end_month + 1;
		try
		{
		    $result = $this->conn->query('DELETE FROM [ActualsForIASAP] WHERE
					(([Year] - 1) * 12 + [FiscalMonth] + 1) >= %i AND 
					(([Year] - 1) * 12 + [FiscalMonth] + 1) <= %i', $start_month_idx, $end_month_idx);
			Debugger::log("Deleting actuals for IASAP by month indexes from $start_month_idx ($start_month, $start_year) to $end_month_idx ($end_month, $end_year).", Debugger::DEBUG);
			if ($result >= 0)
			{
				Debugger::log("Actuals for ISAP was deleted.", Debugger::INFO);
				$result = true;
			}
			else
				Debugger::log("Deleting of Actuals for ISAP failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of actuals for ISAP failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
    }
	
    /**
     * Saves new Actuals record.
     * 
     * @param ActualsForIASAP $actual
     * @return boolean Returns true if save successfull, otherwiste returns false.
     */
    public function save(ActualsForIASAP $actual)
    {
        try
        {
            $array = array();
            $array['EID'] = $actual->Eid;
            $array['ChargeNumber'] = $actual->Charge_number;
            $array['Year'] = $actual->Year;
            $array['FiscalMonth'] = $actual->Fiscal_month;
            $array['Hours'] = $actual->Hours;
			$array['ChargeDay'] = $actual->ChargeDay;
			$array['ModifiedAt'] = $actual->ModifiedAt;
            $array['Comment'] = $actual->Comment;
            $db_result = $this->conn->insert('ActualsForIASAP', $array)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving actuals for IASAP record failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
    }
	
    /**
     * Returns number all actuals for iASAP.
     * 
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @return int Number of stored actuals.
     */
    public function countActualsByRangeAndType(DateTime $startDate, DateTime $endDate)
    {
		if (((int) $startDate->format('U')) > ((int) $endDate->format('U'))) // PHP 5.2.0 compliant test
			return;
        $start_year = $startDate->format('Y');
        $end_year = $endDate->format('Y');
		$start_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $startDate->format('U'))->getIndex();
		$end_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $endDate->format('U'))->getIndex();
		$start_month_idx = ($start_year - 1) * 12 + $start_month + 1;
		$end_month_idx = ($end_year - 1) * 12 + $end_month + 1;
		
        $actuals_db_result = $this->conn->query('SELECT * FROM [ActualsTempForIASAP] WHERE
												(([Year] - 1) * 12 + [FiscalMonth] + 1) >= %i 
												AND (([Year] - 1) * 12 + [FiscalMonth] + 1) <= %i', 
												$start_month_idx,
												$end_month_idx)->fetchAll();
		$count = 0;
        foreach ($actuals_db_result as $row)
        {
            $count++;
        }
        return $count;
    }
}
