<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Actuals repository access class.
 *
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 */
class ActualsRepository extends Object
{
    /** @var IActualsMapper */
    private $mapper;

    public function __construct(IActualsMapper $mapper)
    {
        $this->mapper = $mapper;
    }	

    /**
     * Gets all actuals.
     * 
     * @return array array of Actuals
     */
    public function findAll()
    {
        return $this->mapper->findAll();
    }
	
    /**
     * Returns number all actuals.
     * 
     * @param DateTime $startDate
     * @param DateTime $endDate
	 * @param int $source Should be in range of ActualsTemp::SOURCE_*
     * @return int Number of stored actuals.
     */
    public function countActualsByRangeAndType(DateTime $startDate, DateTime $endDate, $source)
    {
        return $this->mapper->countActualsByRangeAndType($startDate, $endDate, $source);
    }
	
    /**
     * Gets single actuals record.
     * --- MOSTLY FOR DEBUGGING ---
     * 
     * @return Actuals Actuals single record
     */
    public function findSingle()
    {
        return $this->mapper->findSingle();
    }
    
    /**
	 * Returns all task years by task id.
	 * 
	 * @param int $id Task ID.
	 * @return	array Task duration according to actuals.
	 */
	public function findTaskYears($id)
    {
        return $this->mapper->findTaskYears($id);
    }
    
    /**
     * Deletes records from actuals by range (year & fiscal month) INCLUSIVELY.
	 * Dates are "rounded" to months, so all data for all complete and partial
	 * months within date range are included. 
	 * 
	 * Example of deleted data:
	 * $startDate is "2nd Feb 2012" and $endDate is "20th Feb 2012" => data for Feb 2012 are deleted
	 * $startDate is "1nd Feb 2012" and $endDate is "30th November 2012" => data from Feb 2012 to Nov 2012 are deleted
	 * $startDate is "3rd Feb 2012" and $endDate is "30th November 2013" => data from Feb 2012 to Dec 2012 and data from Jan 2013 to Nov 2013 are deleted
	 * $startDate is "3rd Feb 2012" and $endDate is "3rd Feb 2012" => data for Feb 2012 are deleted
	 * 
	 * @param DateTime $startDate
     * @param DateTime $endDate
	 * @param int $source Should be in range of Actuals::SOURCE_*
     */
    public function deleteByRange(DateTime $startDate, DateTime $endDate, $source = Actuals::SOURCE_ECHARGE)
    {
        return $this->mapper->deleteByRange($startDate, $endDate, $source);
    }
	
/**
     * Deletes records from actuals by range (year & fiscal month) INCLUSIVELY only for given task.
	 * Dates are "rounded" to months, so all data for all complete and partial
	 * months within date range are included. 
	 * 
	 * Example of deleted data:
	 * $startDate is "2nd Feb 2012" and $endDate is "20th Feb 2012" => data for Feb 2012 are deleted
	 * $startDate is "1nd Feb 2012" and $endDate is "30th November 2012" => data from Feb 2012 to Nov 2012 are deleted
	 * $startDate is "3rd Feb 2012" and $endDate is "30th November 2013" => data from Feb 2012 to Dec 2012 and data from Jan 2013 to Nov 2013 are deleted
	 * $startDate is "3rd Feb 2012" and $endDate is "3rd Feb 2012" => data for Feb 2012 are deleted
	 * 
	 * @param DateTime $startDate
     * @param DateTime $endDate
	 * @param Task $task
	 * @param int $source Should be in range of Actuals::SOURCE_*
     */
    public function deleteByRangeForTask(DateTime $startDate, DateTime $endDate, Task $task, $source = Actuals::SOURCE_ECHARGE)
    {
        return $this->mapper->deleteByRangeForTask($startDate, $endDate, $task, $source);
    }
	
	/**
	 * Deletes records associated with given charge numbers in given range.
	 * 
	 * @param DateTime $startDate Delete from.
	 * @param DateTime $endDate Delete to.
	 * @param array $charge_numbers List of charge numbers.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function deleteIASAPImport(DateTime $startDate, DateTime $endDate, array $charge_numbers)
	{
		return $this->mapper->deleteIASAPImport($startDate, $endDate, $charge_numbers);
	}
    
    /**
     * Saves new Actuals record.
     * 
     * @param Actuals $actual
     * @return boolean Returns true if save successfull, otherwiste returns false.
     */
    public function save(Actuals $actual)
    {
        return $this->mapper->save($actual);
    }
	
    /**
     * Adds new Actuals record.
     * If Actuals record does not exist, new one is inserted.
     * If Actuals record exists, it is updated (hours are added).
     * 
     * @param Actuals $actual
     * @return boolean Returns true if adding successfull, otherwiste returns false.
     */
    public function add(Actuals $actual)
    {
        return $this->mapper->add($actual);
    }
    
    /**
     * Gets date of the oldest record from given source.
     * 
	 * @param int $source Should be in range of Actuals::SOURCE_*
     * @return DibiDateTime
	 * @throws InvalidArgumentException
     */
    public function getLastImportDate($source)
    {
        return $this->mapper->getLastImportDate($source);
    }
	
	/**
	 * Deletes records associated with given task SAP ID.
	 * 
	 * @param String $sapID Task SAP ID.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function deleteActualsByTaskSAPID($sapID)
	{
		return $this->mapper->deleteActualsByTaskSAPID($sapID);
	}
	
	/**
	 * Deletes records associated with given task SAP ID and year.
	 * 
	 * @param String $sapID Task SAP ID.
	 * @param int $year Year.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function deleteActualsByTaskSAPIDAndYear($sapID, $year)
	{
		return $this->mapper->deleteActualsByTaskSAPIDAndYear($sapID, $year);
	}
	
	/**
	 * Deletes records associated with given task SAP ID.
	 * 
	 * @param String $sapID Task SAP ID.
	 * @param int $source Should be in range of Actuals::SOURCE_*
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function deleteActualsByTaskSAPIDAndSource($sapID, $source)
	{
		return $this->mapper->deleteActualsByTaskSAPIDAndSource($sapID, $source);
	}
}
