<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository of cost centers.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class CostCenterRepository extends Object
{
	/** @var ICostCenterMapper */
    private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(ICostCenterMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'CostCenter-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Cost Center"));
    }
	
    /**
     * Saves cost center.
     * 
     * @param CostCenter $costCenter cost center.
     * @return boolean Returns true if cost center is successfully saved, otherwise false.
     */
    public function save(CostCenter $costCenter)
    {
		if($this->mapper->save($costCenter))
		{
			$this->multilog->log('Work Center with ID: ' . $costCenter->ID . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
    }
	
    /**
     * Saves cost center labour rates.
     * 
     * @param CostCenterLabourRates $costCenterLR
     * @return boolean Returns true if cost center labour rates is successfully saved, otherwise false.
     */
    public function saveRates(CostCenterLabourRates $costCenterLR)
    {
		if($this->mapper->saveRates($costCenterLR))
		{
			$this->multilog->log('Work Center Labour Rates with ID: ' . $costCenterLR->ID . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
    }
	
    /**
     * Updates Cost Center and Cost Center Labour Rates.
     * 
     * @param CostCenter $costCenter
     * @return boolean Returns true if update successfull, otherwiste returns false.
     */
    public function update(CostCenter $costCenter)
    {
		$oldObject = $this->findCostCenter($costCenter->ID);
		if ($this->mapper->update($costCenter) && $this->mapper->updateRates($costCenter))
		{
			$this->multilog->log('Cost Center with ID: ' . $costCenter->ID . ' updated. ' . $this->multilog->differences($oldObject, $costCenter), ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
    }
	
	/**
	 * Returns cost center labour rate by cost center id and year.
	 * 
	 * @param string $id Cost center ID.
	 * @param int $year Year of whitch rates should be found.
	 * @return CostCenterLabourRate or NULL if not found.
	 */
	public function findCostCenterLabourRates($id, $year)
	{
		return $this->mapper->findCostCenterLabourRates($id, $year);
	}
        
	/**
	 * Returns cost center labour rates years array by cost center id.
	 * 
	 * @param string $id Cost center ID.
	 * @return	array of CostCenterLabourRate.
	 */
        public function findAllCostCenterLabourRates($id)
        {
                return $this->mapper->findAllCostCenterLabourRates($id);
        }

	/**
	 * Returns cost center with defined labour rate for every year.
	 * 
	 * @param string $id Cost center ID.
	 * @param int $year Year of whitch rates should be found.
	 * @return	CostCenterLabourRate or NULL if not found.
	 */
	public function findCostCenter($id)
	{
		return $this->mapper->findCostCenter($id);
	}

	/**
	 * Returns all cost centers as an array sorted by name.
	 * 
	 * @return	array associative array of CostCenter as value and its ID as a key
	 */
	public function findAllCostCenters()
	{
		return $this->mapper->findAllCostCenters();
	}

	/**
	 * Returns sum of costs on current task in month in a year.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $month Month (1-12).
	 * @param int $year Year.
	 * @param string $currency Currency code (only USD or CZK accepted)
	 * @return	float 
	 */
	public function findActualTaskCostsMonthSummary($task_id, $month, $year, $currency)
	{
		if (($month < 1) || ($month > 12))
			throw new InvalidArgumentException('Month must be integer in range <1,12>. Invalid value:'.$month);
		return $this->mapper->findActualTaskCostsMonthSummary($task_id, $month - 1, $year, $currency);
	}
    
    /**
     * Returns sum of costs on current task in a year.
     * 
     * @param int $task_id Task ID.
	 * @param int $year Year.
	 * @param string $currency Currency code (only USD or CZK accepted)
     * @return MonthArray with float values with hours for every month in a year.
     */
    public function findActualCostsYearSummary($task_id, $year, $currency)
    {
        return $this->mapper->findActualCostsYearSummary($task_id, $year, $currency);
    }
    
    /**
     * Returns all cost centers prefixes as an array.
     * 
     * @return array of strings
     */
    public function findAllCostCentersPrefix()
	{
		return $this->mapper->findAllCostCentersPrefix();
	}
	
	/**
     * Gets CostCenters by CostCenterFilter. Returns every cost center that matches all set conditions.
     * @param CostCenterFilter $filter
     * @return array Of CostCenters or empty array
    */
	public function findByFilter(CostCenterFilter $filter)
	{
		return $this->mapper->findByFilter($filter);
	}
	
	/**
	 * Returns cost center.
	 * 
	 * @param string $taskNamePrefix Task name prefix.
	 * @return	CostCenterLabourRate or NULL if not found.
	 */
	public function findCostCenterByTaskNamePrefix($taskNamePrefix)
	{
		return $this->mapper->findCostCenterByTaskNamePrefix($taskNamePrefix);
	}
}