<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and licence information, please view
 * the file licence.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing ECLicence from database using dibi.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ECLicenceDibiMapper extends DibiMapper implements IECLicenceMapper
{
	/**
	 * Creates ECLicence instance from DibiRow.
	 * @param DibiRow $row
	 * @return ECLicence
	 */
    private function load(DibiRow $row)
    {
		$licence = new ECLicence();
		$licence->ECLicenceID = $row->ECLicence_ID;
		$licence->Name = $row->Name;
		$licence->Value = $row->Value;
		$licence->ECLicenceNamePrefix = $row->ECLicenceNamePrefix;
		return $licence;
    }
	
	/**
	 * Saves ECLicence.
	 * @param ECLicence $licence
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(ECLicence $licence)
	{
		try
        {
            $array = array();
            $array['Name'] = $licence->Name;
            $array['Value'] = $licence->Value;
            $array['ECLicenceNamePrefix'] = $licence->ECLicenceNamePrefix;
            $db_result = $this->conn->insert('ECLicence', $array)->execute();
                        $licence->ECLicenceID = $this->conn->insertId();
            return TRUE;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving ECLicence with Name = " . $licence->Name . "failed. " . $e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
	
	/**
	 * Updates ECLicence.
	 * @param ECLicence $licence
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(ECLicence $licence)
	{
		try
        {
            $array = array();
            $array['ECLicence_ID'] = $licence->ECLicenceID;
            $array['Name'] = $licence->Name;
            $array['Value'] = $licence->Value;
            $array['ECLicenceNamePrefix'] = $licence->ECLicenceNamePrefix;
            $db_result = $this->conn->update('ECLicence', $array)->where('[ECLicence_ID] = %i', $licence->ECLicenceID)->execute();
            return TRUE;
        }
        catch (DibiException $e)
        {
            Debugger::log("ECLicence update with ID = " . $licence->ECLicenceID . " failed. " . $e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
	
	/**
	 * Deletes ECLicence.
	 * @param int $id ECLicence id.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($id)
	{
		try
		{
		    $db_result = $this->conn->delete('[ECLicence]')->where('[ECLicence_ID] = %i', $id)->execute();
			if ($db_result)
				Debugger::log("ECLicence with ID = " . $id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of ECLicence with ID = " . $id . " failed.", Debugger::ERROR);
			return ($db_result) ? TRUE : FALSE;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of ECLicence with ID = " . $id . " failed: " . $e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
	
	/**
	 * Finds ECLicence by id.
	 * @param int $id ECLicence id.
	 * @return ECLicence or NULL if not found.
	 */
	public function find($id)
	{
		$db_result = $this->conn->select('*')->from('[ECLicence]')->where('[ECLicence_ID] = %i', $id)->fetch();
		if ($db_result !== FALSE)
			return $this->load($db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds all ECLicences.
	 * @return array Array of ECLicence or empty array.
	 */
	public function findAll()
	{
		$db_result = $this->conn->select('*')->from('[ECLicence]')->orderBy('[Name]')->fetchAll();     
		$licence_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $row)
                $licence_array[$row->ECLicence_ID] = $this->load($row);
        }		
		return $licence_array;
	}
	
	/**
	 * Finds ECLicence by name.
	 * @param string $name
	 * @return ECLicence or NULL if not found.
	 */
	public function findByName($name)
	{
		$db_result = $this->conn->select('*')->from('[ECLicence]')->where('[Name] = %s', $name)->fetch();
		if ($db_result !== FALSE)
			return $this->load($db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds ECLicence by ECLicenceNamePrefix.
	 * @param string $prefix
	 * @return ECLicence or NULL if not found.
	 */
	public function findByECLicenceNamePrefix($prefix)
	{
		$db_result = $this->conn->select('*')->from('[ECLicence]')->where('[ECLicenceNamePrefix] = %s', $prefix)->fetch();
		if ($db_result !== FALSE)
			return $this->load($db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds max value of attribute Value.
	 * @return int Or NULL if error occured.
	 */
	public function findMaxValue()
	{
		$db_result = $this->conn->select('MAX([Value]) AS [Max]')->from('[ECLicence]')->fetch();
		if ($db_result !== FALSE)
			return $db_result->Max;
		else
			return NULL;
	}
}
