<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models/Fiscal
 */

/**
 * Business fiscal calendar representation for working with fiscal
 * months and planning AOP hours (Annual Operation Plan)
 * which are stored in the FiscalCalendar in database.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 * @static
 */
class FiscalCalendar extends Object 
{
    /** PHP date() function's format of date. */
    const DATE_FORMAT = 'Y-m-d';
    
    /**
     * Hidden constructor. Class is static.
     */
    private function __construct() 
    {
    }

    /**
     * Parses string containing date in fiscal format into Unix timestamp.
     * 
     * Input must be valid fiscal date in range of UNIX timestamp definition
     * which means older than 1st Jan 1970.
     * 
     * @param string $date Fiscal date as string.
     * @param boolean $is_utc Whether date parameter is in UTC/GMT timezone. Default FALSE (means localtime).
     * @return int Returns Unix timestamp.
     * @throws InvalidArgumentException on invalid input
     */
    public static function dateToTimestamp($date, $is_utc = FALSE) 
    {
		if (FiscalCalendar::isValidDate($date))
		{
			preg_match('/^([12]\d{3})-([012]\d)-([0123]\d)$/', $date, $matches);
			$year = $matches[1];
			$month = $matches[2];
			$day = $matches[3];
			if ($is_utc)
				return gmmktime (0, 0, 0, $month, $day, $year);
			else
				return mktime(0, 0, 0, $month, $day, $year);
		}
		else
			throw new InvalidArgumentException('Supplied date parameter is not valid fiscal date.');
    }

    /**
     * Parses string containing date in fiscal format into Unix timestamp.
     * 
     * Input must be valid fiscal date in range of UNIX timestamp definition
     * which means older than 1st Jan 1970.
     * 
     * @param string $datetime Fiscal date and time as string must be valid by function FiscalCalendar::isValidDateTime().
     * @param boolean $is_utc Whether date parameter is in UTC/GMT timezone. Default FALSE (means localtime).
     * @return int Returns Unix timestamp.
     * @throws InvalidArgumentException on invalid input
     */
    public static function datetimeToTimestamp($datetime, $is_utc = FALSE) 
    {
		if (FiscalCalendar::isValidDateTime($datetime))
		{
			preg_match('/^([12]\d{3})-([012]\d)-([0123]\d) ([012]\d):([012345]\d)(:([012345]\d)){0,1}$/', $datetime, $matches);
			$year = $matches[1];
			$month = $matches[2];
			$day = $matches[3];
			$hour = $matches[4];
			$minute = $matches[5];
			if (count($matches) > 6)
				$second = $matches[7];
			else
				$second = 0;
			
			if ($is_utc)
				return gmmktime ($hour, $minute, $second, $month, $day, $year);
			else
				return mktime($hour, $minute, $second, $month, $day, $year);
		}
		else
			throw new InvalidArgumentException('Supplied datetime parameter is not valid fiscal date.');
    }
    /**
     * Checks string containing fiscal date in format YYYY-mm-dd
     * or Y-m-d in PHP date() function format's syntax.
     * 
     * Examples of valid dates:
     * 2011-02-01
     * 2011-12-13
     * 
     * Examples of invalid dates:
     * 12-01-05
     * 2011-13-11
     * 2012-01-05 15:15
     * 2012-01-05 15:15
     * 
     * @param string $date Date as string.
     * @return boolean Returns TRUE on valid fiscal date format othewise FALSE.
     */
    public static function isValidDate($date) 
    {
		if (preg_match('/^([12]\d{3})-([012]\d)-([0123]\d)$/', $date, $matches))
		{
			$year = $matches[1];
			$month = $matches[2];
			$day = $matches[3];
			return checkdate($month, $day, $year);
		}
		else
			return false;
    }
    
    /**
     * Checks string containing fiscal datetime in format YYYY-mm-dd HH:MM:SS
     * or Y-m-d H:i:s in PHP date() function format's syntax.
     * 
     * Examples of valid datetimes:
     * 2011-02-01 23:40:50
     * 2011-12-13 12:50:00
     * 2011-02-01 12:50
     * 
     * Examples of invalid dates:
     * 12-01-05
     * 2011-13-11
     * 2012-01-05 15:15
     * 2012-01-05 15:15
     * 2012-01-07 25:15
     * 2012-01-07 20:62
     * 2012-01-07 20:2:5
     * 
     * @param string $date Date as string.
     * @return boolean Returns TRUE on valid fiscal date format othewise FALSE.
     */
    public static function isValidDateTime($datetime) 
    {
		if (preg_match('/^([12]\d{3})-([012]\d)-([0123]\d) ([012]\d):([012345]\d)(:[012345]\d){0,1}$/', $datetime, $matches))
		{
			$year = $matches[1];
			$month = $matches[2];
			$day = $matches[3];
			if (checkdate($month, $day, $year))
			{
				// test time
				$hour = 0 + $matches[4];
				if (($hour >= 0) && ($hour <= 23))
					return true;
				else
					return false;
			}
			else
				return false;
		}
		else
			return false;
    }    
	
    /**
     * Returns fiscal month matching given Unix timestamp.
     * 
     * @param int $ts Unix timestamp representing date/time within fiscal month.
     * @return FiscalMonth Returns fiscal month or NULL if timestamp doesn't match any.
     */
    public static function getFiscalMonthByTimestamp($ts) 
    {
		if (isset($ts) && is_int($ts) && ($ts > 0))
		{
			$date = date(FiscalCalendar::DATE_FORMAT, $ts);
			$month_data = dibi::select('*')->from('FiscalCalendar')->where('[From] <= %s AND %s <= [To]', $date, $date)->fetch();
			if ($month_data !== false)
				return new FiscalMonth($month_data->FiscalMonth, $month_data->Year, $month_data->From, $month_data->To, $month_data->AOPHours);
			else
				return NULL;
		}
		else
			return NULL;
    }
    
    /**
     * Returns fiscal month by month index and given year.
     * 
     * @param int $month_index Month index in a year.
     * @param int $year Fiscal year.
     * @return FiscalMonth  Returns fiscal month or NULL if not found.
     */
    public static function getFiscalMonth($month_index, $year) 
    {
		$month_data = dibi::select('*')->from('FiscalCalendar')->where('[FiscalMonth] = %u AND [Year] = %u', $month_index, $year)->fetch();
		if ($month_data !== false)
			return new FiscalMonth($month_data->FiscalMonth, $month_data->Year, $month_data->From, $month_data->To, $month_data->AOPHours);
		else
			return NULL;
    }
    
    /**
     * Returns fiscal month following after given fiscal month.
     * 
     * @param FiscalMonth $month
     * @return FiscalMonth  Returns fiscal month or NULL if not found.
     */
    public static function getNextFiscalMonth($month)
    {
		$month_index = $month->Index + 1;
		$year = $month->Year;
		if ($month_index > 11)
		{
			$month_index = 0;
			$year++;
		}
		
		return self::getFiscalMonth($month_index, $year);
    }

   /**
     * Returns array of english fiscal month names.
     *  
     * Indexed from 0.
     *  
     * @return array
     * @throws InvalidArgumentException if $index is out of bounds.
     */
	public static function getEnglishFiscalMonthNames()
	{
		$month_names = array('January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December');
		return $month_names;
	}
	
	
   /**
     * Returns english fiscal month name (January, ..., December) from index 0..11.
     *  
     * If $long is FALSE it returns short month names (Jan, Feb, ..., Dec)
     *  
     * @param int $month_index Month index in a year.
     * @param boolean $long_name Whether month name should be short.
     * @throws InvalidArgumentException if $index is out of bounds.
     */
    public static function fiscalMonthIndexToName($month_index, $long = true) 
    {
		$month_names = FiscalCalendar::getEnglishFiscalMonthNames();
		if (is_numeric($month_index) && ((0 + $month_index) == $month_index)
			&& ($month_index >= 0) && ($month_index <= 11))
			return (($long) ? $month_names[$month_index] : substr($month_names[$month_index], 0, 3));
		else
			throw new InvalidArgumentException("Month index must be integer in range 0 to 11. Invalid value: " . $month_index);
    }

   /**
     * Returns english fiscal month name (January, ..., December) as number (0..11).
     *  
     * @param string $month_name Month name.
     * @param boolean $long_name Whether given month name is short.
     * @return int Index of the month in a year (starting from 0). 
     * @throws InvalidArgumentException if $month_name is unknown of bounds.
     */
    public static function fiscalMonthNameToIndex($month_name, $long = true) 
    {
		$month_names = FiscalCalendar::getEnglishFiscalMonthNames();
		foreach ($month_names as $month_index => $valid_month_name)
		{
			if ($long)
			{
				if ($valid_month_name == $month_name)
					return $month_index;
			}
			else
			{
				if (substr($valid_month_name, 0, 3) == $month_name)
					return $month_index;
			}	
		}
		throw new InvalidArgumentException("Month name is incorrect. ".(($long) ? "Long name expected." : "Short name expected.")." Invalid value: " . $month_name);
    }
	
	
   /**
     * Returns current year.
     *  
     * @return int Current year. 
     */
    public static function currentYear() 
    {
		return (0 + date('Y'));
	}
    
    /**
     * Returns index of current fiscal month from index 0..11.
     * 
     * @return int Fiscal month.
     */
    public static function currentFiscalMonth()
    {
        $date = date("Y-m-d H:i:s");                
        $month_data = dibi::select('[From], [To]')->from('FiscalCalendar')->where('[Year] = %u', date('Y'))->fetchAll();
        $index = 0;
        foreach($month_data as $month)
        {
            if($month['From'] <= $date AND $month['To'] >= $date)
            {
                return $index; exit;
            }
            $index++;
        }
    }
	
	/**
     * Returns date(Y-m-d) of second wednesday of current fiscal month.
     * 
     * @return string date(Y-m-d) or NULL if an error occurred.
     */
	public static function currentSecondWednesday()
	{
		if(FiscalCalendar::currentFiscalMonth() == 0)
		{
			$secondWednesday = date('Y-m-d', strtotime('second wed of january ' . Date('Y')));
			return $secondWednesday;
		}
		else
		{
			$firstDayOfFiscalMonth = dibi::select('[From]')->from('FiscalCalendar')->where('[Year] = %u AND [FiscalMonth] = %i', date('Y'), FiscalCalendar::currentFiscalMonth())->fetch();

            if (isset($firstDayOfFiscalMonth['From']) && ($firstDayOfFiscalMonth['From'] instanceof DateTime))
            {
			    $secondWednesday = strtotime("+" . 9 . " days", strtotime($firstDayOfFiscalMonth['From']->format('Y-m-d')));
			    return date('Y-m-d', $secondWednesday);
            }
		}

        return NULL;
	}
}
