<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * Service for importing data from eCharge.
 *
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 */
class EChargeIASAPImportService extends Object
{
    
    /** @const php.ini maximum execution time */
    const MAXIMUM_EXECUTION_TIME = 10800;
   
    /** @var DIContainer DI context with application repositories access */
    private $context;

	/** @var ILogger Logging facility */
    private $logger;
    
    /** @var resource EChargeWrapper */
    private $echarge;

    /** @var String eCharge URL **/
    private $echarge_url;
    
    /** @var String eCharge EID **/
    private $echarge_username;
    
    /** @var String eCharge password **/
    private $echarge_password;

    /** @var Date eCharge start date **/
    private $echarge_startDate;
	
    /** @var Date eCharge start date **/
    private $origin_echarge_startDate;

    /** @var Date eCharge end date **/
    private $echarge_endDate;

	/** @var boolean Whether to report progress of running **/
    private $report_progress = false;

	/**
	 * Creates service for importing data from eCharge.
	 *  
	 * @param $context Application context.
	 * @param ILogger Object for logging the import progress.
	 */
    public function  __construct(DIContainer $context, ILogger $logger)
    {
        
        $this->context  = $context;
		$this->logger   = $logger;

        $this->echarge_url          = GlobalSettingReaderHelper::get('eCharge.urlBase');
        $this->echarge_username     = GlobalSettingReaderHelper::get('eCharge.credentials.user');
        $this->echarge_password     = GlobalSettingReaderHelper::get('eCharge.credentials.password');
        $this->echarge_startDate    = new DateTime(GlobalSettingReaderHelper::get('eCharge.actualsUpdate.startDate'));
		$this->origin_echarge_startDate    = new DateTime(GlobalSettingReaderHelper::get('eCharge.actualsUpdate.startDate'));
        $this->echarge_endDate      = new DateTime(); // today
        
        $this->echarge = new EChargeWrapper($this->echarge_url, $this->logger);
        
    }
    
    /**
     * Runs import actuals from eCharge
     * 
     * @return boolean
     * @throws EChargeIASAPImportServiceException 
     */
    public function runImport()
    {
        
        // adjust maximum execution time (php.ini)
        $maximum_execution_time = EChargeIASAPImportService::MAXIMUM_EXECUTION_TIME;
		ini_set('max_execution_time', $maximum_execution_time);
        if (ini_get('max_execution_time') != $maximum_execution_time)
            throw new EChargeIASAPImportServiceException("Setting maximum execution time to '$maximum_execution_time' failed, check server (php.ini) settings");
		
        $memory_limit = "2048M";
        if (!ini_set("memory_limit", $memory_limit)) 
			throw new EChargeIASAPImportServiceException("Setting maximum memory allocation to '$memory_limit' failed, check server (php.ini) settings");	
        
		$lock = $this->context->createLock('ECHARGE_IMPORT');
		if (!$lock->lock())
            throw new EChargeIASAPImportServiceException("ECharge import is already running.");
		
        // open logfile
        $this->logger->log('Echarge import start: ' . date('m-d-Y H:i:s'), ILogger::INFO);
        
        try
        {
            // log into eCharge
            $this->echarge->login($this->echarge_username, $this->echarge_password);
			
            //Deletes actuals from ActualsForTempIASAP table
			if($this->context->actualsTempForIASAPRepository->deleteAll())
			{
				$this->logger->log('Actuals for temp IASAP was deleted successfully.', ILogger::INFO);
			}
			else
			{
				$lock->unlock();
				throw new EChargeIASAPImportServiceException('Temp actuals for iASAP was not deleted successfully.');
			}
			
			$this->reportProgress("Progress:");
            while ($this->echarge_startDate <= $this->echarge_endDate)
            {
                $startDate = clone $this->echarge_startDate;
                $this->echarge_startDate->modify('+1 month -1 day');
                $endDate = ($this->echarge_startDate > $this->echarge_endDate) ? $this->echarge_endDate : $this->echarge_startDate;

                $iASAPsucc = 0; // number of successfully imported records
                $iASAPfail = 0; // number of records that couldn't be stored in DB

                foreach ($this->echarge->readActuals($startDate, $endDate) as $record)
                {
					$this->reportProgress("*");
					try
					{
						//$date = new DateTime($record['CHARGE DAY']); // incompatible with SAP decision date but compataible to eCharge dates
						$date = new DateTime($record['MODIFIED_AT']); // compatible with SAP decision date
					}
					catch (Exception $ex)
					{
						$this->logger->log("EChargeIASAPImportService: date validation failed. " . var_export($record, true), ILogger::ERROR);
						continue;
					}

					$actualsForIASAP = new ActualsTempForIASAP();
                    $actualsForIASAP->Eid = $record['USERID'];
                    $actualsForIASAP->Charge_number = str_replace('-', ' ', $record['ORDER/CHRG#']);
					$fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $date->format('U'));
					if ($fiscal_month === NULL)
					{
						$lock->unlock();
			            throw new EChargeIASAPImportServiceException("ECharge data are invalid, cannot determine fiscal month for: ". var_export($record, true));
					}
                    $actualsForIASAP->Fiscal_month = $fiscal_month->getIndex();
                    $actualsForIASAP->Year = $fiscal_month->getYear();
                    $actualsForIASAP->Hours = (float) $record['HOURS'];
					$actualsForIASAP->ChargeDay = new DateTime($record['CHARGE DAY']);
					$actualsForIASAP->ModifiedAt = new DateTime($record['MODIFIED_AT']);
					
					if(isset($record['COMMENT']))
						$actualsForIASAP->Comment = $record['COMMENT'];

					if($this->context->actualsTempForIASAPRepository->save($actualsForIASAP))
					{
						$iASAPsucc++;
					}
					else
					{
						$iASAPfail++;
						$this->logger->log("EChargeIASAPImportService: actuals for iASAP db insert failed. " . var_export($actualsForIASAP, true), ILogger::ERROR);
					}
                }
                
				if ($iASAPfail == 0)
                {
                    $msg = "EChargeIASAPImportService: All $iASAPsucc actuals for iASAP from " . $startDate->format('m-d-Y') . " till " . $endDate->format('m-d-Y') . " imported successfully";
                    $this->logger->log($msg, ILogger::NOTICE);
                    Debugger::log($msg, Debugger::INFO);
                }
                else
                {
                    $msg = "EChargeIASAPImportService: $iASAPsucc actuals for iASAP imported, $iASAPfail actuals for iASAP thrown away (" . $startDate->format('m-d-Y') . " - " . $endDate->format('m-d-Y') . ")";
                    $this->logger->log($msg, ILogger::WARNING);
                    Debugger::log($msg, Debugger::ERROR);
                }

                $this->echarge_startDate->modify('+1 day');
            }
			$actualsTempForIASAP = $this->context->actualsTempForIASAPRepository->countActualsByRangeAndType($this->origin_echarge_startDate, $this->echarge_endDate);
			$actualsCurrentForIASAP = $this->context->actualsForIASAPRepository->countActualsByRangeAndType($this->origin_echarge_startDate, $this->echarge_endDate);
			$this->logger->log('Current actuals for iASAP records: ' . $actualsCurrentForIASAP, ILogger::INFO);
			$this->logger->log('Temp actuals for iASAP records: ' . $actualsTempForIASAP, ILogger::INFO);
			// Compares temp actuals for iASAP to current actuals
			if($actualsCurrentForIASAP <= $actualsTempForIASAP)
			{
				//Deletes actuals from current table by range
				if($this->context->actualsForIASAPRepository->deleteByRange($this->origin_echarge_startDate, $this->echarge_endDate))
				{
					$this->logger->log('Current actuals for iASAP was deleted successfully.', ILogger::INFO);
				}
				else
				{
					$lock->unlock();
					$this->context->actualsTempForIASAPRepository->deleteAll();
					throw new EChargeIASAPImportServiceException('Current actuals for iASAP was not deleted successfully.');
				}
				//Moves actuals
				if($this->context->actualsTempForIASAPRepository->moveAll())
				{
					$this->logger->log('Temp actuals for iASAP were moved successfully. E-charge import ends.', ILogger::NOTICE);
				}
				else
				{
					$lock->unlock();
					$this->context->actualsTempForIASAPRepository->deleteAll();
					throw new EChargeIASAPImportServiceException('Temp actuals for iASAP were not moved successfully.');
				}
			}
			else
			{
				$lock->unlock();
				$this->context->actualsTempForIASAPRepository->deleteAll(ActualsTemp::SOURCE_ECHARGE);
				throw new EChargeIASAPImportServiceException('Count of current actuals is greater than count of temp actuals.');
			}
			$this->context->actualsTempForIASAPRepository->deleteAll();
			$this->reportProgress("\n");
			$lock->unlock();
        }
        catch (EChargeWrapperException $e)
        {
            $this->logger->log($e->getMessage(), ILogger::ERROR);
            Debugger::log($e->getMessage(), Debugger::ERROR);
			$lock->unlock();
			$this->context->actualsTempRepository->deleteAll(ActualsTemp::SOURCE_ECHARGE);
            return FALSE;
        }
        
        return TRUE;
        
    }

    /**
     * If set to TRUE, import echos character "*" for every
	 * downloaded block of data so to keep the connection alive.
     */
    public function setReportProgress($report)
    {
        $this->report_progress = ($report) ? TRUE : FALSE;
    }

    /**
	 * Reports progress to output and flushes buffers.
	 * Used as a workaround to keep clients alive because whole process can take minutes
     */
    private function reportProgress($report)
    {
		if ($this->report_progress)
		{
			echo $report;					
			ob_flush();					
			flush();
		}
    }
	
    /**
     * Gets maximum execution time.
     * 
     * @return int
     */
    public static function getMaximumExecutionTime()
    {
        return (int) constant('EChargeIASAPImportService::MAXIMUM_EXECUTION_TIME');
    }
    
}	

class EChargeIASAPImportServiceException extends Exception {}
