<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * Service for importing names of tasks, cost centers, cost shares
 * for input file generated from SAP with transaction CN47N
 * and WBS Elements names from file generated from SAP
 * with transaction CN43N.
 *
 * Messages from checking and parsing files are recorded using ILogger implementation
 * provided upon object construction.
 * 
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class MultipleProjectSAPDataImportService extends Object
{
	/** @const php.ini maximum execution time */
	const MAXIMUM_EXECUTION_TIME = 720;

	/** Types of source SAP transactions generating input files */
	const 
		SAP_CN47N_OUTPUT = 'tasks', // file with tasks with long names
		SAP_CN43N_OUTPUT = 'wbs', // file with WBS (work breakdown structure) with long names
		SAP_YPSDSNPLNYR_OC_OUTPUT = 'projectplans'; // file with projects, wbs and task with plans with shortened names

	const
		SAP_FILES_ENCODING  = 'CP1252'; // SAP files are coded in Microsoft Windows-1252 (Western Latin alphabet)
	
	/** Statistics names */
	const STATS_PROJECT_UPDATED = 'projects_updated';

	/** @var DIContainer DI context with application repositories access */
    private $context;

	/** @var ILogger logging facility */
    private $logger;

    /** @var string Directory with files to process */
    private $workDir;

    /** @var string Directory with templates of processed sap files */
    private $templateDir;

	private $cachedCostCenterPrefixes = NULL;

	private $stats = NULL;
	
	/** 
	 * Preloaded CostAsLabour task SAP_IDs and their respective CostCenters
	 * 
	 * @var array SAP_ID => CostCenterID from Tasks_*.txt file
	 */
	private $CostAsLabourCandidateTasks;
	
	/**
	 * Creates service for importing data from SAP input files.
	 *  
	 * @param DIContainer context Application context.
	 * @param string $working_dir Path to working directory with input files.
	 * @param ILogger $logger Object for logging the import progress.
	 */
    public function  __construct(DIContainer $context, $working_dir, ILogger $logger) 
    {
		$this->context = $context;
		$this->workDir = $working_dir;
		$this->templateDir = WWW_DIR . DIRECTORY_SEPARATOR . $this->context->parameters['sap_template_dir'] ;
		$this->logger = ($logger === NULL) ? (new NullLogger('')) : $logger;
		// TODO: stats can be unprecise when multiple files with same projects are found
		$this->stats = array();
		$this->stats['new_projects_found'] = 0;
		$this->stats['projects_processed'] = 0;
		$this->stats[self::STATS_PROJECT_UPDATED] = 0;
		$this->stats['projects_file_processed'] = 0;
		$this->CostAsLabourCandidateTasks = array();
    }
	
	/**
	 * Reads all files from working directory into array.
	 * @return array 
	 * @throws IOException if working directory cannot be read.
	 */
	private function getImportFilesFromWorkDir()
	{
		$sap_files = array();
		$dh = opendir($this->workDir);
		if ($dh !== FALSE) 
		{
			$file = readdir($dh);
			while ($file !== false)
			{
				if ($file != "." && $file != "..")
					$sap_files[] = $file;
				$file = readdir($dh);
			}
        }
		else
		{
			throw new IOException('Opening of directory '.$this->workDir.' failed. Cannot continue.');
		}
		closedir($dh);
		sort($sap_files, SORT_STRING);
		return $sap_files;
	}
	
	/**
	 * Runs import of all files in working directory.
	 * 
	 * @return boolean 
	 * @throws IOException if files cannot be read.
	 */
	public function runFilesImport()
	{
        // adjust maximum execution time (php.ini)
        $maximum_execution_time = self::MAXIMUM_EXECUTION_TIME;
		ini_set('max_execution_time', $maximum_execution_time);
        if (ini_get('max_execution_time') != $maximum_execution_time)
            throw new InvalidStateExeption("Setting maximum execution time to '$maximum_execution_time' failed, check server (php.ini) settings");

		$files = $this->getImportFilesFromWorkDir();
		// files has to be imported in this order (to assure that new tasks are created first and then renamed eventually):
		// 1. SAP_CN47N_OUTPUT - we need to fill the list of tasks treated as CostAsLabour
		// 2. SAP_YPSDSNPLNYR_OC_OUTPUT type files
		// 3. SAP_CN43N_OUTPUT
		// 4. SAP_CN47N_OUTPUT
		// (SAP_CN47N_OUTPUT and SAP_CN43N_OUTPUT order can be switched, but only for 3rd and 4th file)
		$files_to_process = array();
		$files_to_process[0] = array();
		$files_to_process[1] = array();
		$files_to_process[2] = array();
		foreach ($files as $file)
		{
			$file_info = $this->guessTransactionFileType(basename($file));
			if ($file_info === FALSE)
			{
				$this->logger->log('File "' . basename($file) . '" was ignored. Unknown file type.', ILogger::INFO);
				continue;
			}
			switch ($file_info['type'])
			{
				case self::SAP_YPSDSNPLNYR_OC_OUTPUT:
					$files_to_process[0][] = array('type' => $file_info['type'], 'path' => $file, 'year' => $file_info['year']);
					break;
				case self::SAP_CN43N_OUTPUT:
					$files_to_process[1][] = array('type' => $file_info['type'], 'path' => $file);
					break;
				case self::SAP_CN47N_OUTPUT:
					// fill the list of tasks treated with the CostAsLabour flag
					// ($this->CostAsLabourCandidateTasks)
					$this->prepareTaskCostAsLabourFlags(array('type' => $file_info['type'], 'path' => $file));
					// process CN47N as always
					$files_to_process[2][] = array('type' => $file_info['type'], 'path' => $file);
					break;
				default:
					$this->logger->log('File "' . basename($file) . '" was ignored. Unsupported type.', ILogger::WARNING);
			}
		}
        
        // Run the import
		$this->processFiles($files_to_process);        
        
		if ($this->stats['projects_processed'] > 0)
			$this->logger->log('Found ' . $this->stats['new_projects_found'] . ' new of ' . $this->stats['projects_processed'] . ' project(s) total in ' . $this->stats['projects_file_processed'] . ' transaction file(s).', (($this->stats['new_projects_found'] > 0) ? ILogger::NOTICE : ILogger::INFO));
	}
    
    /**
     * Handles files
     * @param array $files Array of files to process
     * @throws IOException
     */
    private function processFiles($files)
    {
        // run the import
		$clean_up_done = FALSE;
		for ($i = 0; $i < count($files); $i++)
		{
			foreach ($files[$i] as $file)
			{
				$fhandle = fopen($this->workDir . '/'. $file['path'], 'r');
				if ($fhandle === FALSE)
					throw new IOException('Cannot open file for importing data: '. $file['path']);
				$this->logger->log('Processing file "' . basename($file['path']) . '"', ILogger::INFO);
				switch ($file['type'])
				{
					case self::SAP_CN43N_OUTPUT:
						if ($this->checkCN43NTransactionFile($fhandle) !== FALSE)
							$this->processCN43NTransactionFile($fhandle);
						break;
					case self::SAP_CN47N_OUTPUT:
						if ($this->checkCN47NTransactionFile($fhandle) !== FALSE)
							$this->processCN47NTransactionFile($fhandle);
						break;
					case self::SAP_YPSDSNPLNYR_OC_OUTPUT:
						// clean up new projects
						if (!$clean_up_done)
						{
							$clean_up_done = $this->context->projectFoundRepository->deleteAllButBlacklisted();
//							$this->logger->log('Clean up of new (except blacklisted) projects.', ILogger::DEBUG);
						}
						if ($this->checkYPSDSNPLNYR_OCTransactionFile($fhandle) !== FALSE)
							$this->processYPSDSNPLNYR_OCTransactionFile($fhandle, $file['year']);
						break;
				}
				fclose($fhandle);
			}
		}
    }

	/**
	 * Guesses given filetype of input file by name.
	 * If guess is successful, returns associative array with variable
	 * or two elements (depending on file type):
	 * <ul>
	 * <li>type : (mandatory) type of file MultipleProjectSAPDataImportService::SAP_*_OUTPUT</li>
	 * <li>year : (optional) integer with the year in the file name</li>
	 * </ul>
	 * Returns FALSE in case that file type cannot be recognized by name.
	 * 
	 * @param string $filename If whole path is given only file base name is tested.
	 * @return	mixed Returns array with file type and other info if successful guess or false if not.
	 */
	public static function guessTransactionFileType($filename)
	{
		$name = basename($filename);
		$matches = array();
		if (preg_match('/^WBS_.*.txt$/i', $name, $matches))
			return array('type' => MultipleProjectSAPDataImportService::SAP_CN43N_OUTPUT);
		elseif (preg_match('/^Tasks_.*.txt$/i', $name, $matches))
			return array('type' => MultipleProjectSAPDataImportService::SAP_CN47N_OUTPUT);
		elseif (preg_match('/^PMB_.*_([0-9]{4}).dat$/i', $name, $matches))
			return array('type' => MultipleProjectSAPDataImportService::SAP_YPSDSNPLNYR_OC_OUTPUT, 'year' => $matches[1]);
		else
			return FALSE;
	}

	/**
	 * Guesses cost center of task by task name and returns cost center ID.
	 * 
	 * Cost center ID is decided by prefix of task name.
	 * If task prefix of cost center matches prefix of task name
	 * then cost center ID is returned.
	 * Testing is case insensitive (so prefix "BRN" and "brn" matches the same)
	 * 
	 * @param string $taskname Name of task.
	 * @return string Returns Cost center ID or empty string.
	 */
	private function guessCostCenterFromTaskName($taskname)
	{
		$name = trim($taskname);
		if ($name == '')
			return '';
		// let's cache cost center task prefixes
		if ($this->cachedCostCenterPrefixes === NULL)
		{
			$cost_centers = $this->context->costCenterRepository->findAllCostCenters();
			$this->cachedCostCenterPrefixes = array();
			foreach ($cost_centers as $center_id => $cost_center) 
				if (trim($cost_center->getTaskNamePrefix()) != '') // only cost centers with defined task name prefix
					$this->cachedCostCenterPrefixes[$center_id] = $cost_center->getTaskNamePrefix();
			// need to sort them by prefix length descending
			uasort($this->cachedCostCenterPrefixes, 'strcmp');
			$this->cachedCostCenterPrefixes = array_reverse($this->cachedCostCenterPrefixes, TRUE);
		}
        unset($center_id);
		foreach ($this->cachedCostCenterPrefixes as $center_id => $prefix)
		{
			if (strpos(strtolower($name), strtolower($prefix)) === 0) // prefix is divided from the rest of task name by a space
				return $center_id;
		}
		return '';
	}
	
	/**
	 * Check whether given file is correct transaction file according to tester.
	 * 
	 * @param resource $filehandle Handle to opened file.
	 * @param RegexHeaderFileTester $tester
	 * @param ILogger $errorlogger
	 * @return	mixed Returns FALSE if file is not right type of timestamp of file content generation. 
	 */
	private static function checkTransactionFile($filehandle, RegexHeaderFileTester $tester, ILogger $logger)
	{
		if (!is_resource($filehandle))
			throw new InvalidArgumentException('Given handle is not file resource.');
		try
		{
			$tester->testResource($filehandle);					
			return $tester->getLastFileTimestamp();
		}
		catch (IOException $e)
		{
			$logger->log($e->getMessage(), ILogger::ERROR);
			return false;
		}
	}
	
	/**
	 * Returns currency of PMB file.
	 * 
	 * @param resource $filehandle Handle to opened file.
	 * @return	mixed Returns FALSE if currency is not recognized or string of currency
	 */
	public static function getPMBCurrency($filehandle)
	{
		fseek($filehandle, 0);
		// Skip header to currency
		for ($i = 0; $i < 6; $i++)
			fgets($filehandle);
		$line = fgets($filehandle);
		if(strpos($line, ExchangeRatePresenter::CZK))
			return ExchangeRatePresenter::CZK;
		elseif(strpos($line, ExchangeRatePresenter::EUR))
			return ExchangeRatePresenter::EUR;
		else
			return false;
	}
	
	/**
	 * Get date object from given input string.
	 * Possible input formats are: d.m.Y or d/m/Y
	 * 
	 * @param string $date Input string containing date.
	 * @return	DateTime object
	 */
	private static function getDateTimeFromInputString($date)
	{
		if(strpos($date, '.') || strpos($date, '-'))
			return DateTime::createFromFormat('d.m.Y', $date);
		elseif(strpos($date, '/'))
			return DateTime::createFromFormat('m/d/Y', $date);
	}
	
	/**
	 * Check whether given file is SAP CN43N transaction file (WBS Elements) by content.
	 * 
	 * Checks file header.
	 * 
	 * @param resource $filehandle Handle to opened file.
	 * @param ILogger $errorlogger
	 * @return	mixed Returns FALSE if file is not right type of timestamp of file content generation. 
	 */
	public function checkCN43NTransactionFile($filehandle)
	{
		return MultipleProjectSAPDataImportService::checkTransactionFile(
				$filehandle, 
				new RegexHeaderFileTester($this->templateDir . DIRECTORY_SEPARATOR . 'CN43N.header-regex'),
				$this->logger
		);
	}

	/**
	 * Check whether given file is SAP CN47N transaction file (Tasks) by content.
	 * 
	 * Checks file header.
	 * 
	 * @param resource $filehandle Handle to opened file.
	 * @param ILogger $errorlogger
	 * @return	mixed Returns FALSE if file is not right type of timestamp of file content generation. 
	 */
	public function checkCN47NTransactionFile($filehandle)
	{
		return MultipleProjectSAPDataImportService::checkTransactionFile(
				$filehandle, 
				new RegexHeaderFileTester($this->templateDir . DIRECTORY_SEPARATOR . 'CN47N.header-regex'),
				$this->logger
		);
	}

	/**
	 * Check whether given file is SAP YPSDSNPLNYR_OC transaction file (TaskPlans and WBS structure with short names) by content.
	 * 
	 * Checks file header.
	 * 
	 * @param resource $filehandle Handle to opened file.
	 * @param ILogger $errorlogger
	 * @return	mixed Returns FALSE if file is not right type of timestamp of file content generation. 
	 */
	public function checkYPSDSNPLNYR_OCTransactionFile($filehandle)
	{
		return MultipleProjectSAPDataImportService::checkTransactionFile(
				$filehandle, 
				new RegexHeaderFileTester($this->templateDir . DIRECTORY_SEPARATOR . 'YPSDSNPLNYR_OC.header-regex'),
				$this->logger
		);
	}

	/**
	 * Imports WBS Element names from SAP CN43N transaction file.
	 * Messages from import are logged into logger.
	 * 
	 * @param resource $filehandle Handle to opened file.
	 * @return	boolean
	 */
	private function processCN43NTransactionFile($fhandle)
	{
		$this->logger->log('Parsing WBS Elements names.', ILogger::INFO);
		fseek($fhandle, 0);
		// Skip header - 4 lines
		// TODO: get rid of header line count somehow :(
		for ($i = 0; $i < 4; $i++)
			fgets($fhandle);
		// lines in input file are in format (with header):
		//-------------------------------------------------------------------------------------------------------------------------------------------------------
		//|Lev|Proj. def.|WBS element       |Name                                    |No. resp.|Plnt|Basic strt|Finish (B)|Created|Curr.|DlI|Text 2   |Text 3   |
		//-------------------------------------------------------------------------------------------------------------------------------------------------------
		//|  1|EC-000541 |EC-000541         |A350XWB WP1 EMC Test set-up and doc upd |   288842|3320|02/22/2010|05/12/2010|E454429|CZK  |   |MIL 123..|MIL 234..|
		//|.....
		$line_number = 0;
		$licences = array();
		$project_id = NULL;
		while (true)
		{
			$input_line = fgets($fhandle);
			$line_number++;
			$line = iconv(MultipleProjectSAPDataImportService::SAP_FILES_ENCODING, "UTF-8", $input_line);
			$wbs_data = explode('|', $line);
			if (count($wbs_data) == 15) // 13 data columns and 2 extra before first and after last delimiter |
			{
				$sap_id = trim($wbs_data[3]);
				$name = trim($wbs_data[4]);
				$wp = $this->context->workPackageRepository->findBySAPID($sap_id);
				if ($wp != NULL)
				{
					if ($name != $wp->getName())
					{
						$old_name = $wp->getName();
						$wp->setName($name);
						if ($this->context->workPackageRepository->update($wp))
							$this->logger->log('Work Package (' . $wp->SAPID . ') has been updated. Name: '.$old_name.' > ' . $name . '; ', ILogger::NOTICE);
						else
							$this->logger->log('Update of Work Package (' . $wp->SAPID . ') failed.', ILogger::ERROR);
					}
					
					// pick project id
					if ($wp->ProjectID)
					{
						$project_id = $wp->ProjectID;
						$licences[$project_id] = array();
					}
					// pick licence names as array keys - it ensures uniqueness of each licence name
					$licences[$project_id][$wbs_data[12]] = NULL;
					$licences[$project_id][$wbs_data[13]] = NULL;
				}
				// Update project plant
				if(trim($wbs_data[1]) == 1)
				{
					$projectSAP_ID = trim($wbs_data[2]);
					$project = $this->context->projectRepository->findProjectBySAPID($projectSAP_ID);
					if($project)
					{
						$plant = trim($wbs_data[6]);
						if($project->Plant != $plant)
						{
							$oldPlant = $project->Plant;
							$project->Plant = $plant;
							if($this->context->projectRepository->update($project))
							{
								$this->logger->log('Project ' . $project->Name . ' (' . $projectSAP_ID . ') plant has been updated from ' . $oldPlant . ' to ' . $project->Plant . '.', ILogger::NOTICE);
							}
							else
							{
								$this->logger->log('Cannot update project ' . $project->Name . ' (' . $projectSAP_ID . ') plant from ' . $oldPlant . ' to ' . $project->Plant . '.', ILogger::WARNING);
							}
						}
					}
				}
			}
			else
				break;
		}
		
		$project_updated = array();
		
		// save and assign licence names to each project
		foreach ($licences as $project_id => $licence_names)
		{
			$old_names = array();
			$old_licence_names = $this->context->ecUtility->findECLicenceNameByProjectID($project_id);
			foreach ($old_licence_names as $licence_name)
				$old_names[] = $licence_name->Name;
			
			// Delete previous assignment for project
			if (!$this->context->projectECLicenceNameRepository->deleteByProjectID($project_id))
				$this->logger->log('Cannot delete assigned EC licences for Project (' . $project_id . ').', ILogger::WARNING);
			
			foreach ($licence_names as $name => $null)
		    {
                if ((trim($name) != '') && !in_array($name, $old_names))
                    $project_updated[$project_id] = TRUE;

                if (!$this->context->ecUtility->assignLicence($name, $project_id))
                    $this->logger->log('Cannot assign EC licence "' . $name . '" to Project (' . $project_id . ').', ILogger::WARNING);
				
			}
		}
        
		$this->stats[self::STATS_PROJECT_UPDATED] += count($project_updated);
		
		// now we should be at the end of the data table 
		// last line should be dashed line
		if (!preg_match('/^\-+\s*$/', $line))
			$this->logger->log('End of data file was not reached. Stopped on line ' . $line_number . ' of the input table.', ILogger::WARNING);
		return true;
	}
	
	/**
	 * Imports Task names, cost centers and cost shares from SAP CN47N transaction file.
	 * Messages from import are logged into logger.
	 * 
	 * @param resource $filehandle Handle to opened file.
	 * @return	boolean
	 */
	private function processCN47NTransactionFile($fhandle)
	{
		$this->logger->log('Parsing names, cost share, work centers for tasks.', ILogger::INFO);
		fseek($fhandle, 0);
		// Skip header - 4 lines
		// TODO: get rid of header line count somehow :(
		for ($i = 0; $i < 4; $i++)
			fgets($fhandle);
		// lines in input file are in format (with header):
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
        //|Proj. def.|WBS element       |Network   |Act.|Plnt|Activity desc.                          |WorkCntr|Work center text             |Part Numbe|ActyT |     Work|       Amount|%Wk|Curr.|Act. start|Act.finish|ErlStrt(B)|ErlFin (B)|Basic strt|Finish (B)|
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
        //|EG-002251 |EG-002251-3.1     |7003208140|0010|3320|MAT T3.1-001  D-NOTAM 2012              |        |                             |54        |      |         |   38,380.00 |   |CZK  |          |          |01/01/2012|12/31/2012|01/01/2012|12/31/2012|
        //|...
        $line_number = 0;
		while (true)
		{
			$input_line = fgets($fhandle);
			$line_number++;
			$line = iconv(MultipleProjectSAPDataImportService::SAP_FILES_ENCODING, "UTF-8", $input_line);
			$task_data = explode('|', $line);
			if (count($task_data) == 22) // 20 data columns and 2 extra before first and after last delimiter |
			{
				$sap_id = trim($task_data[3]) . ' ' . trim($task_data[4]);
				$name = trim($task_data[6]);
				$cost_share = (float) (trim($task_data[9]) / 100);
				$cost_center_id = trim($task_data[7]);
				$start_date = $this->getDateTimeFromInputString(trim($task_data[19]));
				$finish_date = $this->getDateTimeFromInputString(trim($task_data[20]));
				$task = $this->context->taskRepository->findBySAPID($sap_id);
				if ($task != NULL)
				{
                    $temp_change_log = ''; $delim = '; ';
                    
                    // Task name
					if ($name != $task->Name)
					{
						$old_name = $task->Name;
						$task->Name = $name;
                        $temp_change_log .= 'Name: ' . $old_name . ' > ' . $name . $delim;
					}
                    
                    // Task cost share
					if ($cost_share != $task->SAPCostShare)
					{
						$old_cost_share = $task->SAPCostShare;
						$task->SAPCostShare = $cost_share;
                        $temp_change_log .= 'Cost Share: ' . $old_cost_share . ' > ' . $cost_share . $delim;
					}
                    
                    // Task work center
                    // Work center has been found
                    if (strlen($cost_center_id))
                    {
                        // Task should be marked as labor if work center is present
                        if (!$task->LabourTaskFlag)
                        {
                            $task->LabourTaskFlag = TRUE;
                            $temp_change_log .= 'Labor: false > true' . $delim;
                        }
                        
                        // Currently assigned work center is different from new one
                        if ($cost_center_id != $task->CostCenterID)
                        {
                            // Only work centers stored in db can be used
                            if ($this->context->costCenterRepository->findCostCenter($cost_center_id))
                            {
                                $old_cost_center_id = $task->CostCenterID;
                                $task->CostCenterID = $cost_center_id;
                                $temp_change_log .= 'Work Center: ' . $old_cost_center_id . ' > ' . $cost_center_id . $delim;
                            }
                            // Unknown work center
                            else
                            {
                                $this->logger->log('Work Center (' . $cost_center_id . ') found for Task "' . $task->Name . '" (' . $task->SAPID . ') is unknown therefore Task Work Center cannot be updated.', ILogger::WARNING);
                            }
                        }
                    }
                    // Work center has not been found and task (originally non-labor) is not considered as labor
					else if (!$task->CostAsLabour)
					{
                        // Current work center assignment is not empty
                        if (strlen($task->CostCenterID))
                        {
                            $this->logger->log('Work Center (' . $task->CostCenterID . ') currently assigned to Task "' . $task->Name . '" (' . $task->SAPID . ') has not been reassigned to new Work Center ([empty string]).', ILogger::WARNING);
                        }
                        // Task should be marked as non-labor if work center is absent
                        else if ($task->LabourTaskFlag)
                        {
                            $task->LabourTaskFlag = FALSE;
                            $temp_change_log .= 'Labor: true > false' . $delim;
                        }
					}
					// Work center cost as labour fast bug fix
					if($task->CostAsLabour)
					{
                        $task->LabourTaskFlag = TRUE;
						$task->CostCenterID = 'CZxxxxxx';
                        $temp_change_log .= 'Labor: false > true' . $delim . 'Work Center: NULL > ' . $task->CostCenterID . $delim;
					}
					
                    // Task start date
					if ($start_date && (!$task->StartDate || (($task->StartDate instanceof DateTime) && ($start_date->format('Y-m-d') !== $task->StartDate->format('Y-m-d')))))
					{
						$old_start_date = $task->StartDate;
						$task->StartDate = $start_date->format('Y-m-d');
                        $temp_change_log .= 'Start Date: ' . $old_start_date . ' > ' . $start_date->format('Y-m-d') . $delim;
					}
					
                    // Task finish date
					if ($finish_date && (!$task->FinishDate || (($task->FinishDate instanceof DateTime) && ($finish_date->format('Y-m-d') !== $task->FinishDate->format('Y-m-d')))))
					{
						$old_finish_date = $task->FinishDate;
						$task->FinishDate = $finish_date->format('Y-m-d');
                        $temp_change_log .= 'Finish Date: ' . $old_finish_date . ' > ' . $finish_date->format('Y-m-d') . $delim;
					}
                    
                    // Update task if any change has been made
                    if ($temp_change_log !== '')
                    {
						//we update just SAP tasks
						if($task->Fake == Task::TASK_TYPE_SAP)
						{
							if ($this->context->taskRepository->update($task))
								$this->logger->log('Task "' . $task->Name . '" (' . $task->SAPID . ') has been updated: "' . $temp_change_log, ILogger::NOTICE);
							else
								$this->logger->log('Update of Task "' . $task->Name . '" (' . $task->SAPID . ') failed.', ILogger::ERROR);
						}							
						// Otherwise we riase error message
						else
						{
							$this->logger->log('The task ' . $task->Name . ' (' . $task->SAPID . ') was not updated, because it is already set up as a Placeholder task by somebody else. Inform the stakeholders, remove the Placeholder task and run Bulk import again in order to get the task updated within a new project.', ILogger::ERROR);
						}
                    }
			    }
		    }
			else
				break;
		}
		// now we should be at the end of the data table 
		// last line should be dashed line
		if (!preg_match('/^\-+\s*$/', $line))
			$this->logger->log('End of data file was not reached. Stopped on line ' . $line_number . ' of the input table.', ILogger::WARNING);

		return true;
	}

	/**
	 * Updates Task Plans, adds New Tasks and WorkPackages for existing projects
	 * from YPSDSNPLNYR_OC SAP transaction output file.
	 * 
	 * @param resource $filehandle Handle to opened file.
	 * @param int $year Year for which the file has data.
	 * @return	boolean		Returns true if processing completed or false if it wasn't finished.
	 * @todo Simplify and clean up the code (detection of errors and footer which has to be ignored complicates the code).
	 */
	private function processYPSDSNPLNYR_OCTransactionFile($fhandle, $year)
	{
		if (!is_resource($fhandle))
			throw new InvalidArgumentException('Given handle is not file resource.');
		if (!NumericUtils::isPositiveInteger($year))
			throw new InvalidArgumentException('Given year is not positive integer.');
		$currency = MultipleProjectSAPDataImportService::getPMBCurrency($fhandle);
		if($currency == ExchangeRatePresenter::CZK)
		{
			$currency_exchange_rates = $this->context->ratesRepository->findYearRates('CZK', 'USD', $year);
			for ($i = MonthArray::LOWER_INDEX; $i <= MonthArray::UPPER_INDEX; $i++) // check if there are all exchange rates available
				if ($currency_exchange_rates[$i]->isUndefined())
				{
					$this->logger->log('Exchange rate for month ' . $i . ' is not set! Cannot process data!.', ILogger::ERROR);
					return false;
				}
		}
		if($currency == ExchangeRatePresenter::EUR)
		{
			$currency_exchange_rates = $this->context->ratesRepository->findYearRates('EUR', 'USD', $year);
			for ($i = MonthArray::LOWER_INDEX; $i <= MonthArray::UPPER_INDEX; $i++) // check if there are all exchange rates available
				if ($currency_exchange_rates[$i]->isUndefined())
				{
					$this->logger->log('Exchange rate for month ' . $i . ' is not set! Cannot process data!.', ILogger::ERROR);
					return false;
				}
		}
		$this->stats['projects_file_processed']++;
		$this->logger->log('Parsing task plans a new WBS elements for existing projects.', ILogger::INFO);
		fseek($fhandle, 0);
		// Skip header - 7 lines
		for ($i = 0; $i < 7; $i++)
			fgets($fhandle);
		// initialize 
		$data_timestamp = date("Y-m-d H:i:s");
		$line_number = 0;
		$parsed_project_sap_id = NULL;
		$next_project_sap_id = NULL;
		$end_of_file_reached = FALSE;
		$dropped_lines = array();
		$project_lines = array();
		$bufferred_lines = array();
		$parsing_on = TRUE;
		while ((count($bufferred_lines) > 0) || (count($dropped_lines) > 0) || !$end_of_file_reached)
		{
			$input_line = fgets($fhandle);
			if ($input_line === FALSE) // end of file or error in reading file
			{	
				$end_of_file_reached = TRUE;
				$line = $input_line;
			} 
			else
				$line = iconv(MultipleProjectSAPDataImportService::SAP_FILES_ENCODING, "UTF-8", $input_line);
			$line_number++;

			// simple test correctness of line data - correct first column data and correct number of columns
			$matches = array();
			if (!preg_match('/^[\d\* ]{3} ([A-Z]{3}) (.{25})[^\t]+\t.*/', $line, $matches) || (count(explode("\t", $line)) != 95))
			{
				// line is incorrect => remember dropped line and read next if not end of file
				$dropped_lines[] = 'Line ' . $line_number . ' is invalid. It is not Task/WP/Project line: ' . $line;
				$parsing_on = FALSE; // found invalid lines -> stop parsing until new project
				if (!$end_of_file_reached)
					continue;
			}
			if (isset($matches[1]) && ($matches[1] == 'PRJ')) // project line found
			{
				$next_project_sap_id = $matches[2];
				if ($parsed_project_sap_id === NULL) // only for the first one
					$parsed_project_sap_id = $next_project_sap_id;
				$parsing_on = TRUE;
			}
			if (count($dropped_lines) > 0) // if there were dropped lines...
			{
				if (!$end_of_file_reached) // ...in the middle of data file then drop current line buffer we have only part of the project data
				{
					$bufferred_lines = array();
					foreach ($dropped_lines as $dropped_line)
						$this->logger->log($dropped_line, ILogger::WARNING);
					$this->logger->log('Project containing invalid lines is not updated! ('.count($dropped_lines).' invalid lines found)', ILogger::ERROR);
				}
//				else // ...or at the end we silently drop them
//				{
//					$this->logger->log('Dropped '. count($dropped_lines) . ' lines at the end of the file.', ILogger::DEBUG);
//				}
				$dropped_lines = array();
			}
			// test another project start or end of line and last project in the line buffer
			if (((isset($matches[1]) && ($matches[1] == 'PRJ')) || ($end_of_file_reached)) && (count($bufferred_lines) > 0))
			{
				$project_lines = $bufferred_lines; // this will initiate parsing of project lines later
				$bufferred_lines = array();
			}
			if (($line !== FALSE) && ($parsing_on)) // buffer next valid line if parsing not stopped
				$bufferred_lines[] = $line;
			if (count($project_lines) > 0)
			{
				$this->stats['projects_processed']++;
				$project_array = $this->parseArrayFromYPSDSNPLNYR_OCTransaction($project_lines, $currency_exchange_rates, 0);
				$project_lines = array();
				if ($project_array === FALSE)
					$this->logger->log('('.$this->stats['projects_processed'].') Parsing of Project failed! Skipped.', ILogger::ERROR);
				else
				{
					$project = $this->context->projectRepository->findProjectBySAPID($project_array[0]['SAP_ID']);
					if ($project == NULL) // project is new
					{
						if ($this->processNewProjectInfo($project_array, $year))
						{
							$this->logger->log('('.$this->stats['projects_processed'].') Found new Project "' . $project_array[0]['Name'] . '" (' . $project_array[0]['SAP_ID'] . ').', ILogger::NOTICE);
							$this->stats['new_projects_found']++;
						}
					}
					else
					{
						$this->logger->log('('.$this->stats['projects_processed'].') Starting update of Project "' . $project->Name . '" (' . $project->SAPID . ').', ILogger::INFO);
						$this->updateProjectFromArray($project, $project_array, $year, $data_timestamp);
						$this->logger->log('('.$this->stats['projects_processed'].') Update of Project "' . $project->Name . '" (' . $project->SAPID . ') completed.', ILogger::INFO);

					}
				}
			}
			$parsed_project_sap_id = $next_project_sap_id;
		}
		return true;
	}		

 	/**
	 * Parses lines representing project definition from SAP input file
	 * into array that can be processed by updateProjectFromArray function.
	 * 
	 * How to setup $currency_exchange_rates:
	 * Example: CZK (source currency of SAP files) => USD (target currency of application)
	 * current exchange rate for January is: 17 CZK for 1 USD
	 * $currency_exchange_rates[1] = 17.0 
	 * 
	 * Result is an array of associative arrays containing values on keys:
	 * $array["Type"] : "PRJ" for Project or "WBS" for WorkPackage or "NWA" for Task with TaskPlan
	 * $array["SAP_ID"] : SAP ID of element e.g. "EC-001238" or "7002447847 0030"
	 * $array["Name"] : Shortened name of the element
	 * $array["Parent"] : SAP ID of parent element 
	 * $array["Level"] : level of nesting of the element
	 * $array["Hours"] : MonthArray of hours planned
	 * $array["Budget"] : MonthArray of money planned
	 * 
	 * Example of input file format including header:
		Object	Jan ValPln	Feb ValPln	Mar ValPln	Apr ValPln	May ValPln	Jun ValPln	Jul ValPln	Aug ValPln	Sep ValPln	Oct ValPln	Nov ValPln	Dec ValPln	Jan ValAct	Feb ValAct	Mar ValAct	Apr ValAct	May ValAct	Jun ValAct	Jul ValAct	Aug ValAct	Sep ValAct	Oct ValAct	Nov ValAct	Dec ValAct	Jan Comm	Feb Comm	Mar Comm	Apr Comm	May Comm	Jun Comm	Jul Comm	Aug Comm	Sep Comm	Oct Comm	Nov Comm	Dec Comm	Jan QtyAct	Feb QtyAct	Mar QtyAct	Apr QtyAct	May QtyAct	Jun QtyAct	Jul QtyAct	Aug QtyAct	Sep QtyAct	Oct QtyAct	Nov QtyAct	Dec QtyAct	Jan QtyPln	Feb QtyPln	Mar QtyPln	Apr QtyPln	May QtyPln	Jun QtyPln	Jul QtyPln	Aug QtyPln	Sep QtyPln	Oct QtyPln	Nov QtyPln	Dec QtyPln	Qtr 1 Val	Qtr 2 Val	Qtr 3 Val	Qtr 4 Val	Qtr 1 Qty	Qtr 2 Qty	Qtr 3 Qty	Qtr 4 Qty	Year Val	Year Qty	Jan ValTot	Feb ValTot	Mar ValTot	Apr ValTot	May ValTot	Jun ValTot	Jul ValTot	Aug ValTot	Sep ValTot	Oct ValTot	Nov ValTot	Dec ValTot	Jan QtyTot	Feb QtyTot	Mar QtyTot	Apr QtyTot	May QtyTot	Jun QtyTot	Jul QtyTot	Aug QtyTot	Sep QtyTot	Oct QtyTot	Nov QtyTot	Dec QtyTot
		Displayed in	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 CZK	1 CZK	1 CZK	1 CZK	1 H	1 H	1 H	1 H	1 CZK	1 H	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 CZK	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H	1 H
		6*  PRJ EC-001238                  SESAR MGMT noneligib	               8002	              66388	              29888	             421748	              29888	              29888	              29888	            3520549	              29888	              29888	              29888	              51773	               1356	              35268	              25708	             142151	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	             166610	             623675	            3580326	             111550	                  0	                  0	                  0	                  0	            4482161	                  0	               9358	             101656	              55596	             563899	              29888	              29888	              29888	            3520549	              29888	              29888	              29888	              51773	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0
		5*  WBS EC-001238                  SESAR MGMT noneligib	               8002	              66388	              29888	             421748	              29888	              29888	              29888	            3520549	              29888	              29888	              29888	              51773	               1356	              35268	              25708	             142151	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	             166610	             623675	            3580326	             111550	                  0	                  0	                  0	                  0	            4482161	                  0	               9358	             101656	              55596	             563899	              29888	              29888	              29888	            3520549	              29888	              29888	              29888	              51773	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0
		4*  WBS EC-001238-5                2011 SESAR MGMT none	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	               1356	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	               1356	                  0	                  0	                  0	                  0	                  0	                  0	                  0	               1356	                  0	               1356	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0
		*** WBS EC-001238-52               EXT+COST11 SESAR MGM	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	               1356	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	               1356	                  0	                  0	                  0	                  0	                  0	                  0	                  0	               1356	                  0	               1356	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0
		**  NWA 7001976449 0020            TRV11 SESAR MGMT non	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	               1356	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	               1356	                  0	                  0	                  0	                  0	                  0	                  0	                  0	               1356	                  0	               1356	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0
		4*  WBS EC-001238-6                2012 SESAR MGMT none	               8002	              66388	              29888	             421748	              29888	              29888	              29888	            3520549	              29888	              29888	              29888	              51773	                  0	              35268	              25708	             142151	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	             165254	             623675	            3580326	             111550	                  0	                  0	                  0	                  0	            4480805	                  0	               8002	             101656	              55596	             563899	              29888	              29888	              29888	            3520549	              29888	              29888	              29888	              51773	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0
		*** WBS EC-001238-62               EXT+COST12 SESAR MGM	               8002	              66388	              29888	             421748	              29888	              29888	              29888	            3520549	              29888	              29888	              29888	              51773	                  0	              35268	              25708	             142151	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	             165254	             623675	            3580326	             111550	                  0	                  0	                  0	                  0	            4480805	                  0	               8002	             101656	              55596	             563899	              29888	              29888	              29888	            3520549	              29888	              29888	              29888	              51773	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0
		**  NWA 7002447847 0010            TLS12 SESAR MGMT non	                  0	              21886	              21886	              21886	              21886	              21886	              21886	              21886	              21886	              21886	              21886	              43771	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	              43772	              65658	              65658	              87543	                  0	                  0	                  0	                  0	             262631	                  0	                  0	              21886	              21886	              21886	              21886	              21886	              21886	              21886	              21886	              21886	              21886	              43771	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0
		**  NWA 7002447847 0020            TRV12 SESAR MGMT non	               8002	               8002	               8002	             247877	               8002	               8002	               8002	               8002	               8002	               8002	               8002	               8002	                  0	                  0	              25708	               8186	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	              49715	             272068	              24007	              24007	                  0	                  0	                  0	                  0	             369796	                  0	               8002	               8002	              33710	             256063	               8002	               8002	               8002	               8002	               8002	               8002	               8002	               8002	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0
		**  NWA 7002447847 0030            OTC12 SESAR MGMT non	                  0	                  0	                  0	                  0	                  0	                  0	                  0	            3490661	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	            3490661	                  0	                  0	                  0	                  0	                  0	            3490661	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	            3490661	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0
		**  NWA 7002447847 0040            NLR12 SESAR MGMT non	                  0	              36500	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	              35268	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	              71768	                  0	                  0	                  0	                  0	                  0	                  0	                  0	              71768	                  0	                  0	              71768	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0
		**  NWA 7002447847 0050            MAT12 SESAR MGMT non	                  0	                  0	                  0	             151985	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	             133965	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	             285950	                  0	                  0	                  0	                  0	                  0	                  0	             285950	                  0	                  0	                  0	                  0	             285950	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0	                  0
	 * 
	 * 
	 * @param array $lines Lines (fragment of input file) representing project.
	 * @param MonthArray $currency_exchange_rates array of twelve CurrencyExchangeRate for converting source budget values to target
	 * @return array Returns array of WBS element data.
	 * @throws InvalidArgumentException on wrong usage or invalid input parameters.
	 */
	private function parseArrayFromYPSDSNPLNYR_OCTransaction($lines, MonthArray $currency_exchange_rates, $rounding_prec = 2) 
	{
        if (count($lines))
		{
			$missing_rates = array();
            $project_wbs_elements = array();
            $parents = array();
            $actual_level = -1;
            foreach ($lines as $line_of_text) 
			{
                $wbs_element = array();

				if (preg_match("/^([\d\* ]{3}) ([A-Z]{3}[ ].{25}[^\t]+)[\t](.*)$/", $line_of_text, $values)) 
				{
                    // Level (   ,*  ,** ,***,4* ,5* , ...)
                    if (preg_match("/([\d])[\*][ ]/", $values[1], $level)) 
                        $wbs_element["Level"] = $level[1];
                    else
                        $wbs_element["Level"] = preg_match_all("/\*/", $values[1], $level);
                    if ($actual_level == -1) // level initialized
                        $actual_level = $wbs_element["Level"];
                    // Name: TYPE SAP_ID SAP_NAME_SHORT
                    // Name [PRJ|WBS|NWA] {25}{1,20}\t
                    preg_match("/^([A-Z]{3})[ ](.{25})(.*)$/", $values[2], $name_column);
                    $wbs_element["Type"] = trim($name_column[1]);
                    $wbs_element["SAP_ID"] = trim($name_column[2]);
                    $wbs_element["Name"] = trim($name_column[3]);
					
					// Search for labour tasks by guessing cost center
//					$cost_center = $this->guessCostCenterFromTaskName($wbs_element["Name"]);
//					if ($cost_center)
//					{
//						// Check existence of labour rates for current year
//						$rates = $this->context->costCenterRepository->findCostCenterLabourRates($cost_center, (int) date('Y'));
//						if ($rates === NULL)
//							$missing_rates[$cost_center] = $cost_center;
//					}
					
                    // Values are delimited by TABs ( ... \t ... \t ... \t ... )
					$value_arr = explode("\t", $values[3]);
                    // Planned budgets (ValPln columns): String to Int Conversion
					$budget = new MonthArray(0, TRUE);
					for ($i = MonthArray::LOWER_INDEX; $i <= MonthArray::UPPER_INDEX; $i++) // 1..12
					{
						$budget[$i] = (int) preg_replace("/[^\-\d]/", "", trim($value_arr[$i - 1]));
						// !!! rounding "changes" value in target currency so when counting backwards to source currency, numbers do not match
						$budget[$i] = (int) round($budget[$i] * (1.0 / $currency_exchange_rates[$i]->Value), $rounding_prec);
					}
					$wbs_element["Budget"] = $budget;
                    // Planned hours (QtyPln columns): String to Int Conversion
					$hours = new MonthArray(0, TRUE);
					for ($i = MonthArray::LOWER_INDEX; $i <= MonthArray::UPPER_INDEX; $i++) // 1..12
						$hours[$i] = (int) preg_replace("/[^\-\d]/", "", trim($value_arr[47 + $i]));
					$wbs_element["Hours"] = $hours;
                    // Levels And Parents
                    if ($wbs_element["Type"] != "PRJ") 
					{
                        if ($actual_level > $wbs_element["Level"]) 
						{
                            // Cannot Go "Down" By More Than One Step
                        } 
						else
						{
							if ($actual_level < $wbs_element["Level"]) 
								for ($i = 0; $i < $wbs_element["Level"] - $actual_level; $i++)
									array_pop($parents);
							array_pop($parents);
						}
                        $wbs_element["Parent"] = $parents[count($parents) - 1];
                    }
                    $actual_level = $wbs_element["Level"];
                    array_push($parents, $wbs_element["SAP_ID"]);
					//$this->logger->log(var_export($wbs_element, TRUE), ILogger::DEBUG);
                    $project_wbs_elements[] = $wbs_element;
                }
            }
			
			// Inform user about missing labour rates in current year
			foreach ($missing_rates as $cost_center)
			    $this->logger->log('Work Center (' . $cost_center . ') has undefined Labour Rates for year ' . date('Y') . '.', Logger::WARNING);
			
			// all projects in input file have same level which means that task levels (leaves of WBS tree)
			// often have higher level than 0 => normalize/reduce levels
			$min_level = $project_wbs_elements[0]["Level"]; // Project level
			foreach ($project_wbs_elements as $element)
				$min_level = ($element["Level"] < $min_level) ? $element["Level"] : $min_level;
			if ($min_level > 0)
			{
				//$this->logger->log('Reducing levels of project "' . $project_wbs_elements[0]['Name'] . '" with SAP ID "' . $project_wbs_elements[0]['SAP_ID'] . '" by ' . $min_level, ILogger::DEBUG);
				foreach ($project_wbs_elements as &$element)
				{
					$element["Level"] = $element["Level"] - $min_level;
					//$this->logger->log(var_export($element, TRUE), ILogger::DEBUG);
				}
			}
            return $project_wbs_elements;
        } 
		else 
			throw new InvalidArgumentException("Input array of lines is empty. Invalid function usage.");
    }	

	/**
	 * Creates new project from project stub.
	 * 
	 * @param ProjectFound $project_stub Project stub.
	 * @return mixed Returns ID of new project or FALSE on failure.
	 */
	public function runNewProjectImport(ProjectFound $project_stub)
	{
		$project = new Project();
		$project->Name = $project_stub->Name;
		$project->SAP_ID = $project_stub->SAP_ID;		
		if ($this->context->projectRepository->save($project))
		{
			$this->logger->log('Project "' . $project->Name . '" (' . $project->SAP_ID . ') was created.', ILogger::NOTICE);
			// process plans
			$plans = $project_stub->getPlans();
			foreach ($plans as $plan)
			{
				// TODO: rollback inserts ;*(
				$plan_array = unserialize($plan->getSerializedPlan());
				if (!$this->updateProjectFromArray($project, $plan_array, $plan->getYear(), $plan->getLastFoundDate()))
					return FALSE;
			}
			return $project->ID;
		}
		else
		{
			$this->logger->log('Insertion of Project "' . $project_stub->Name . '" (' . $project_stub->SAP_ID . ') failed.', ILogger::ERROR);
			return FALSE;
		}
	}
	
	/**
	 * Updates project its WBS and task plans from array 
	 * created by parseArrayFromYPSDSNPLNYR_OCTransaction() method.
	 * 
	 * Works only if project already exists. Does not create new projects.
	 * 
	 * @param array $project_array Input array representing project.
	 * @param int $year Year for which this project plan was defined.
	 * @param string $data_timestamp Database formatted file timestamp to set the same timestamp for added elements.
	 * @return boolean Returns TRUE on success.
	 * @todo Fix work package structure if it changes.
	 * @todo If WorkPackage in existing structure is missing in input log warning of missing workpackage.
	 */
	private function updateProjectFromArray(Project $project, $project_array, $year, $data_timestamp)
	{
		if ($project === NULL)
			throw new InvalidArgumentException('Invalid project argument. Cannot be NULL.');
		if (($project_array == NULL) || (count($project_array) == 0) || ($project_array[0]["Type"] != "PRJ"))
			throw new InvalidArgumentException('Invalid project_array argument.');
		$task_adepts_for_plans_deletion = $this->context->taskRepository->findPlannedProjectTasksInYear($project->getID(), $year);
		//TODO: create temporary SAP_ID => ID map for easier searching of ID by SAP_ID (no need to peek into DB everytime)
		foreach ($project_array as $record)
		{
			switch ($record["Type"])
			{
                // Update project
				case "PRJ":
					if ($project->getName() != $record["Name"])
					{
						$old_project_name = $project->getName();
						$project->setName($record["Name"]);
						if ($this->context->projectRepository->update($project))
							$this->logger->log('Project "' . $project->Name . '" (' . $project->SAP_ID . ') has been updated: Name: ' . $old_project_name . ' > ' . $record["Name"] . '; ', ILogger::NOTICE);
						else
							$this->logger->log('Update of Project "' . $old_project_name . '" (' . $project->SAP_ID . ') failed.', ILogger::ERROR);
					}
					break;
                // Update work package
				case "WBS":
					$work_package = NULL;
					$work_package = $this->context->workPackageRepository->findBySAPID($record["SAP_ID"]);
					if ($work_package != NULL) // update workpackage name if necessary
					{
						if (($work_package->getName() != $record["Name"])
							&& (strpos($work_package->getName(), $record["Name"])) !== 0) // update only if new name is not prefix of current
						{
							$old_wp_name = $work_package->getName();
							$work_package->setName($record["Name"]);
							if ($this->context->workPackageRepository->update($work_package))
								$this->logger->log('Work Package "' . $work_package->Name . '" (' . $work_package->SAPID . ') updated: Name: ' . $old_wp_name . ' > ' . $record["Name"] . '; ', ILogger::NOTICE);
							else
								$this->logger->log('Update of Work Package "' . $old_wp_name . '" (' . $work_package->SAPID . ') failed.', ILogger::ERROR);
						}
					}
					else // new work package found
					{
						$work_package = new WorkPackage();
						$work_package->SAPID = $record["SAP_ID"];
						$work_package->Name = $record["Name"];
						if ($record["SAP_ID"] == $project->SAP_ID) // Top level WorkPackage is the same as project
							$work_package->ProjectID = $project->ID;
						else
						{
							$parent_work_package = $this->context->workPackageRepository->findBySAPID($record["Parent"]);
							if ($parent_work_package == NULL) // TODO: use temporary SAP_ID => ID map
							{
								$this->logger->log('Failed to find parent Work Package (' . $record["Parent"] . ') to Work Package "' . $record["Name"] . '" (' . $record["SAP_ID"] . '). New Work Package has not been inserted!', ILogger::ERROR);
								break;
							}
							$work_package->ParentID = $parent_work_package->ID;
						}
						$work_package->Date = $data_timestamp;
						if ($this->context->workPackageRepository->save($work_package))
							$this->logger->log('New Work Package "' . $work_package->Name . '" ('.$work_package->SAPID.') has been added to Project (' . $project->SAPID . ').', ILogger::NOTICE);
						else
							$this->logger->log('Insertion of new Work Package "' . $record["Name"] . '" (' . $record["SAP_ID"] . ') failed.', ILogger::ERROR);
					}
					break;
                // Update task
				case "NWA":
                    $prefixes_cost_as_labor = (array) Environment::getConfig('cost_as_labour');
                    
					$task = $this->context->taskRepository->findBySAPID($record["SAP_ID"]);
                    // Task does not exist - new one should be created
					if ($task == NULL)
					{
                        $parent_work_package = $this->context->workPackageRepository->findBySAPID($record["Parent"]);
						if ($parent_work_package == NULL)
						{
                            // TODO Use temporary SAP_ID => ID mapping
							$this->logger->log('Failed to find parent Work Package (' . $record["Parent"] . ') for Task "' . $record["Name"] . '" ('.$record["SAP_ID"].'). New Task has not been inserted!', ILogger::ERROR);
							break;
						}
                        
						$task = new Task();
						$task->Name = $record["Name"];
						$task->SAPID = $record["SAP_ID"];
						$task->ProjectID = $project->ID;
						$task->WorkPackageID = $parent_work_package->ID;
						$task->Date = $data_timestamp;
						$task->CostShare = $project->RealCostShare;

						// set CostAsLabour flag accordingly
						if (array_key_exists($task->SAPID, $this->CostAsLabourCandidateTasks))
						{
							$costCenterID = $this->CostAsLabourCandidateTasks[$task->SAPID];
							if ($costCenterID === '')
							{
								// cost as labour task -> set CostCenter to HTS Virtual
								$task->CostCenterID = 'CZxxxxxx';
								$task->CostAsLabour = TRUE;
								$task->LabourTaskFlag = TRUE;
							}
							else
							{
								$costCenter = $this->context->costCenterRepository->findCostCenter($costCenterID);
								if ($costCenter !== NULL)
								{
									// it's just ordinary labour task,
									// we've found the WC as expected
									$task->CostCenterID = $costCenterID;
								}
								else
								{
									$this->logger->log('Cannot find Work Center (' . $costCenter . ') for new Task "' . $record["Name"] . '" (' . $record["SAP_ID"] . '). New Task will be inconsistent!', ILogger::ERROR);
								}
								$task->CostAsLabour = FALSE;
								$task->LabourTaskFlag = TRUE;
							}
						}
						
						// TODO Build fake WBS structure if necessary
                        
						if ($this->context->taskRepository->save($task))
                        {
							$this->logger->log('New Task "' . $task->Name . '" (' . $task->SAPID . ') has been added to Project "' . $project->Name . '" (' . $project->SAPID . ')', ILogger::NOTICE);
                        }
						else
						{
							$this->logger->log('Insertion of new Task "' . $record["Name"] . '" (' . $record["SAP_ID"] . ') failed.', ILogger::ERROR);
							break;
						}
					}
                    // Task exists - it should be updated
					else
					{
						$temp_change_log = '';  $delim = '; ';
                        
                        // Update only if new name is not prefix of current name
						if (($task->getName() != $record["Name"]) && (strpos($task->getName(), $record["Name"]) !== 0))
						{
                            $task->Name = $record["Name"];
							$temp_change_log .= 'Name: ' . $task->Name . ' > ' . $record["Name"] . $delim;
						}
                        
                        // Work center for task which is considered as labor
						$cost_center_id = $this->guessCostCenterFromTaskName($task->Name);
                        if ($cost_center_id != '')
                        {
                            $cost_center = $this->context->costCenterRepository->findCostCenter($cost_center_id);
                            if ($cost_center)
                            {
								$hasCostAsLabourPrefix = in_array($cost_center->TaskNamePrefix, $prefixes_cost_as_labor);
								
                                // Should be this task (according to name prefix) considered as labor?
						        if ($hasCostAsLabourPrefix && !$task->CostAsLabour && isset($this->CostAsLabourCandidateTasks[$record['SAP_ID']]) && 
										$this->CostAsLabourCandidateTasks[$record['SAP_ID']] === '')
                                {
									// this task should be definitely flagged as CostAsLabour
									// beware: not all tasks with CostAsLabour prefix should have the CostAsLabour flag set!
									// they must also have blank workcenter column (when processing Tasks_*.txt)
                                    $task->CostCenterID = $cost_center_id;
                                    $task->CostAsLabour = TRUE;
                                    $task->LabourTaskFlag = TRUE;
                                    $temp_change_log .= 'Work Center: ' . (($task->CostCenterID) ? $task->CostCenterID : '[empty string]') . ' > ' . $cost_center_id . $delim;
                                    $temp_change_log .= 'Cost as Labor: false > true' . $delim;
                                    $temp_change_log .= 'Labor: false > true' . $delim;
                                }
								// this task has prefix, but hasn't empty work centrum, which means it should be set to
								// ->CostAsLabour = 0 and ->LabourTaskFlag = 1
								else if ($hasCostAsLabourPrefix && $task->CostAsLabour &&
										$this->CostAsLabourCandidateTasks[$record['SAP_ID']] !== '')
								{
									$cost_center = $this->context->costCenterRepository->findCostCenter($this->CostAsLabourCandidateTasks[$record['SAP_ID']]);
									
									if ($cost_center === NULL)
									{
										$this->logger->log('Cannot find Work Center (' . $this->CostAsLabourCandidateTasks[$record['SAP_ID']] . ') for Task "' . $task->Name . '" (' . $task->SAPID . '). Task will be inconsistent!', ILogger::ERROR);
									}
									else
									{
										$oldLabourFlag = $task->LabourTaskFlag;
										$oldLabourFlag = ($oldLabourFlag) ? "true" : "false";
										
										$task->CostCenterID = $cost_center->ID;
										$task->CostAsLabour = FALSE;
										$task->LabourTaskFlag = TRUE;
										$temp_change_log .= 'Work Center: ' . (($task->CostCenterID) ? $task->CostCenterID : '[empty string]') . ' > ' . $cost_center_id . $delim;
										$temp_change_log .= 'Cost as Labor: true > false' . $delim;
										$temp_change_log .= 'Labor: ' . $oldLabourFlag . ' > true' . $delim;
									}
								}
                                // Should not be this task (according to name prefix) considered as labor?
                                else if (!$hasCostAsLabourPrefix && $task->CostAsLabour)
                                {
                                    $old_cost_center_id = $task->CostCenterID;
                                    $task->CostCenterID = NULL;
                                    $task->CostAsLabour = FALSE;
                                    $task->LabourTaskFlag = FALSE;
                                    $temp_change_log .= 'Work Center: ' . (($old_cost_center_id) ? $old_cost_center_id : '[empty string]') . ' > [empty string]' . $delim;
                                    $temp_change_log .= 'Cost as Labor: true > false' . $delim;
                                    $temp_change_log .= 'Labor: true > false' . $delim;
                                }
                            }
                            else
                            {
                                $this->logger->log('Cannot find Work Center (' . $cost_center_id . ') for Task "' . $task->Name . '" (' . $task->SAPID . '). Task will be inconsistent!', ILogger::ERROR);
                            }
                        }
                        
						// Update task if any change has been made
                        if ($temp_change_log !== '')
                        {
							//we update just SAP tasks
							if($task->Fake == Task::TASK_TYPE_SAP)
							{
								if ($this->context->taskRepository->update($task))
									$this->logger->log('Task "' . $task->Name . '" (' . $task->SAPID . ') has been updated: "' . $temp_change_log, ILogger::NOTICE);
								else
									$this->logger->log('Update of Task "' . $task->Name . '" (' . $task->SAPID . ') failed.', ILogger::ERROR);
							}
							// Otherwise we riase error message
							else
							{
								$this->logger->log('The task ' . $task->Name . ' (' . $task->SAPID . ') was not updated, because it is already set up as a Placeholder task by somebody else. Inform the stakeholders, remove the Placeholder task and run Bulk import again in order to get the task updated within a new project.', ILogger::ERROR);
							}
                        }
					}
                    
					// If task plan is still active for a task -> remove it from drop list
					if (isset($task_adepts_for_plans_deletion[$task->ID]))
						unset($task_adepts_for_plans_deletion[$task->ID]);
                    
					$task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task->ID, $year);
                    // Task plan does not exist - new one should be created
					if ($task_plan === NULL)
					{
						$task_plan = new TaskYearPlan($year);
						$task_plan->setDate($data_timestamp);
						$task_plan->setTaskID($task->ID);
					}
                    
					$task_plan_changes = ''; // Temporary change log
                    
                    // Recalculate plan for tasks that are considered as labor
					if($task->CostAsLabour)
					{
						$cost_center = $this->context->costCenterRepository->findCostCenter($task->CostCenterID);
                        if ($cost_center)
                        {
						    $rate = $cost_center->getYearLabourRate($year);
                            if ($rate)
                            {
                                // For all months
                                for ($i = 1; $i <= 12; $i++)
                                {
                                    // Store new budget
                                    if ($record["Budget"][$i] != $task_plan->getMonthPlan($i)->getBudget())
                                    {
                                        // Budget is extra information for labor tasks but it is saved anyway
                                        $task_plan_changes .= 'Budget('.FiscalCalendar::fiscalMonthIndexToName($i - 1, false).'): '.$task_plan->getMonthPlan($i)->getBudget().' > '.$record["Budget"][$i].'; ';
                                        $task_plan->getMonthPlan($i)->setBudget($record["Budget"][$i]);
                                    }
                                    
                                    // Recalculate cost budget to hours using labour rate
                                    $hours = round($record["Budget"][$i]/$rate->getMonthlyHourRate($i), 1);
                                    
                                    // Skip january - plan for tasks that are considered as labor comes with month delay
                                    if($i != 1)
                                    {
                                        // Store new plan
                                        if ($hours != $task_plan->getMonthPlan($i-1)->getHours())
                                        {
                                            // Store hours plan for all months except last one
                                            if ($i != 12)
                                            {
                                                $task_plan->getMonthPlan($i-1)->setHours($hours);
                                            }
                                            // Split december plan into last two months - december includes plan for itself and month before
                                            else
                                            {
                                                $half_hours = ($hours)/2;
                                                $task_plan->getMonthPlan($i-1)->setHours($half_hours);
                                                $task_plan->getMonthPlan($i)->setHours($half_hours);
                                            }                                            
                                            $task_plan_changes .= 'Hours('.FiscalCalendar::fiscalMonthIndexToName($i - 2, false).'): '.$task_plan->getMonthPlan($i-1)->getHours().' > '.$hours.'; ';
                                        }
                                    }
                                    // Special care about january
                                    else
                                    {
                                        // Store new plan
                                        if ($hours != 0)
                                        {                                    
                                            $this->logger->log('Billing of Task "' . $task->Name . '" (' . $task->SAPID . ') probably contains mistake. Unexpected plan has been found in January ' . $year . '.', ILogger::WARNING);
                                        }                                        
                                    }
                                }
                            }
                            else
                            {
                                $this->logger->log('Calculation of Task Plan for year ' . $year . ' for Task "' . $task->Name . '" (' . $task->SAPID . ') failed. Missing Labour Rate for year ' . $year . ' for Work Center "' . $cost_center->Name.'" (' . $cost_center->ID . ').', ILogger::WARNING);
                            }
                        }
                        else
                        {
                            $this->logger->log('Calculation of Task Plan for year ' . $year . ' for Task "' . $task->Name . '" (' . $task->SAPID . ') failed. Unknown Work Center (' . (($task->CostCenterID) ? $task->CostCenterID : '[empty string]') . ').', ILogger::WARNING);
                        }
					}
                    // Handle plan for other tasks
					else
					{
                        // For all months
						for ($i = 1; $i <= 12; $i++)
						{
                            // Store new budget
							if ($record["Budget"][$i] != $task_plan->getMonthPlan($i)->getBudget())
							{
                                // TODO Find potential problem - saving budget for labor tasks
								// budget is stored but irrelevant for labour tasks - can change for labor task even if hours do not change based on current exchange rate
								// for new task plans for task that do not have yet decided whether they are labor both budget and hours are logged
								if (!$task->LabourTaskFlag)
									$task_plan_changes .= 'Budget('.FiscalCalendar::fiscalMonthIndexToName($i - 1, false).'): '.$task_plan->getMonthPlan($i)->getBudget().' > '.$record["Budget"][$i].'; ';
								$task_plan->getMonthPlan($i)->setBudget($record["Budget"][$i]);
							}
                            
                            // Store new plan 
							if ($record["Hours"][$i] != $task_plan->getMonthPlan($i)->getHours())
							{
								$task_plan_changes .= 'Hours('.FiscalCalendar::fiscalMonthIndexToName($i - 1, false).'): '.$task_plan->getMonthPlan($i)->getHours().' > '.$record["Hours"][$i].'; ';
								$task_plan->getMonthPlan($i)->setHours($record["Hours"][$i]);
							}
						}
					}
					
                    // Store task plan in case of changes on existing task plan or new task plan
					if (($task_plan_changes != '') || ($task_plan->ID == 0))
					{
                        // New task plan
						if ($task_plan->ID == 0)
						{
							if ($this->context->taskPlanRepository->save($task_plan))
								$this->logger->log('New Task Plan (' . $task_plan->ID . ') for year ' . $year . ' for Task "' . $task->Name . '" (' . $task->SAPID . ') was added.', ILogger::NOTICE);
							else
								$this->logger->log('Insertion of new Task Plan for year ' . $year . ' for Task  "' . $task->Name . '" (' . $task->SAPID . ') failed.', ILogger::ERROR);
						}
                        // Update task plan
						else
						{
							// We are updating just SAP tasks
							if($task->Fake == Task::TASK_TYPE_SAP)
							{
								if ($this->context->taskPlanRepository->update($task_plan))
									$this->logger->log('Task Plan (' . $task_plan->ID . ') for year ' . $year . ' for Task "' . $task->Name . '" (' . $task->SAPID . ') has been updated: ' . $task_plan_changes, ILogger::NOTICE);
								else
									$this->logger->log('Update of Task Plan (' . $task_plan->ID . ') for year ' . $year . ' for Task  "' . $task->Name . '" (' . $task->SAPID . ') failed.', ILogger::ERROR);
							}
							// Otherwise we riase error message
							else
							{
								$this->logger->log('Task plan of the task ' . $task->Name . ' (' . $task->SAPID . ') was not updated, because it is already set up as a Placeholder task by somebody else. Inform the stakeholders, remove the Placeholder task and run Bulk import again in order to get the task updated within a new project.', ILogger::ERROR);
							}
						}
					}
					break;
			}
		}
		// delete no longer existing task plans
		if (!empty($task_adepts_for_plans_deletion))
		{
			foreach ($task_adepts_for_plans_deletion as $task_id => $task)
			{
                // Skip placeholder & planning tasks
                if ($task->Fake)
                    continue;
                
				$task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task_id, $year);
				if ($task_plan == NULL)
					$this->log('Failed to find (for deletion) Task Plan for Task "' . $task->Name . '" (' . $task->SAPID . ')  for year ' . $year . '.', ILogger::WARNING);
				else
				{
					if ($this->context->taskPlanRepository->delete($task_plan->ID))
						$this->logger->log('Task Plan (' . $task_plan->ID . ') for year ' . $year . ' for Task "' . $task->Name . '" (' . $task->SAPID . ') has been deleted.', ILogger::NOTICE);
					else
						$this->logger->log('Deletion of Task Plan (' . $task_plan->ID . ') for year ' . $year . ' for Task  "' . $task->Name . '" (' . $task->SAPID . ') failed.', ILogger::ERROR);
				}
			}
		}
		return true;
	}

	/**
	 * Inserts project stub about new project found.
	 * Returns true if project is new and doesn't exist as stub yet.
	 * 
	 * @param array	$project_array	Parsed project data array.
	 * @param int	$year			Year of the plan in the project data array.
	 * @return	boolean	Returns true if project should be reported as new.
	 */
	private function processNewProjectInfo($project_array, $year)
	{
		if ($this->context->projectFoundRepository->isBlacklisted($project_array[0]['SAP_ID']))
		{
//			$this->logger->log('Blacklisted project stub (' . $project_array[0]['SAP_ID'] . ') found. Ignored.', ILogger::DEBUG);				
			return false;
		}
		$project_found_stub_check = $this->context->projectFoundRepository->findInfoBySAPID($project_array[0]['SAP_ID']);
		$report_new = ($project_found_stub_check == NULL);
		$project_found_stub = new ProjectFound();
		$project_found_stub->SAP_ID = $project_array[0]['SAP_ID'];
		$project_found_stub->Name = $project_array[0]['Name'];
		$project_found_plan = new ProjectFoundPlan($year);
		// TODO: better solution would be transformation to XML or some other "standard" format
		$project_found_plan->SerializedPlan = serialize($project_array);
		$project_found_plan->touch();
		$project_found_stub->setPlan($project_found_plan);
		if ($this->context->projectFoundRepository->save($project_found_stub))
		{
			$this->logger->log('New Project "' . $project_found_stub->Name . '" (' . $project_found_stub->SAPID . ') was found. Plan for year '.$year.' was remembered.', ILogger::NOTICE);
			return $report_new;
		}
		else
		{
			$this->logger->log('Failed to add newly found Project "' . $project_found_stub->Name . '" ('.$project_found_stub->SAPID.').', ILogger::ERROR);
			return false;
		}
	}

	/**
	 * Returns statistics array from import.
	 * 
	 * @return	array
	 */
	public function getProjectImportStats()
	{
		return $this->stats;
	}
	
	/*
	 * Fills $this->CostAsLabourCandidateTasks list, which contains information
	 * about tasks that should receive the CostAsLabour flag.
	 * 
	 * This needs to be done, because we can't collect sufficient info from processing the
	 * first file (PMB_*_YYYY.dat), as we're missing the |WorkCntr| column.
	 * Work center is determined when processing Tasks_*.txt, so we just quickly peek into it.
	 * 
	 * @param array $file An array with path to the Tasks_*.txt file.
	 * @return
	 */
	private function prepareTaskCostAsLabourFlags($file)
	{
		$fhandle = fopen($this->workDir . '/'. $file['path'], 'r');
		if ($fhandle === FALSE)
			throw new IOException('Cannot open file for importing data: '. $file['path']);
		
		fseek($fhandle, 0);
		// Skip header - 4 lines
		// TODO: get rid of header line count somehow :(
		for ($i = 0; $i < 4; $i++)
			fgets($fhandle);
		// lines in input file are in format (with header):
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
        //|Proj. def.|WBS element       |Network   |Act.|Plnt|Activity desc.                          |WorkCntr|Work center text             |Part Numbe|ActyT |     Work|       Amount|%Wk|Curr.|Act. start|Act.finish|ErlStrt(B)|ErlFin (B)|Basic strt|Finish (B)|
        //----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
        //|EG-002251 |EG-002251-3.1     |7003208140|0010|3320|MAT T3.1-001  D-NOTAM 2012              |        |                             |54        |      |         |   38,380.00 |   |CZK  |          |          |01/01/2012|12/31/2012|01/01/2012|12/31/2012|
        //|...
        $line_number = 0;
		while (true)
		{
			$input_line = fgets($fhandle);
			$line_number++;
			$line = iconv(MultipleProjectSAPDataImportService::SAP_FILES_ENCODING, "UTF-8", $input_line);
			$task_data = explode('|', $line);
			if (count($task_data) == 22) // 20 data columns and 2 extra before first and after last delimiter |
			{
				$sap_id = trim($task_data[3]) . ' ' . trim($task_data[4]);
				$name = trim($task_data[6]);
				$cost_share = (float) (trim($task_data[9]) / 100);
				$cost_center_id = trim($task_data[7]);
				$start_date = $this->getDateTimeFromInputString(trim($task_data[19]));
				$finish_date = $this->getDateTimeFromInputString(trim($task_data[20]));
				
				$costAsLabourPrefixes = (array) Environment::getConfig('cost_as_labour');
				foreach ($costAsLabourPrefixes as $key => $prefix)
				{
					// cost as labour prefix + work center = ordinary labour task, use this work center
					// later during PMB import (updateProjectFromArray function)
					//
					// cost as labour prefix + blank work center = cost as labour task
					if (strpos(strtolower($name), strtolower($prefix)) === 0)
					{
						$this->CostAsLabourCandidateTasks[$sap_id] = $cost_center_id;
					}
				}
		    }
			else
				break;
		}
		// now we should be at the end of the data table 
		// last line should be dashed line
		if (!preg_match('/^\-+\s*$/', $line))
			$this->logger->log('End of data file was not reached. Stopped on line ' . $line_number . ' of the input table.', ILogger::WARNING);

		fclose($fhandle);
		return true;
	}
}	
