<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class provides methods for communication with LDAP server.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class LDAPQuery
{
	/** server parameters for LDAP authentication */
    private $server = '';
    private $port = '';
    private $basedn = '';
    private $basepass = '';
    private $searchdn = '';
    private $filter = '';

	/**
	 * Options injection from application configuration.
	 */
	public function setOptions($options)
	{
		$this->server = (isset($options['server'])) ? $options['server'] : '';
		$this->port  = (isset($options['port'])) ? $options['port'] : '';
		$this->basedn = (isset($options['basedn'])) ? $options['basedn'] : '';
		$this->basepass = (isset($options['basepass'])) ? $options['basepass'] : '';
		$this->searchdn = (isset($options['searchdn'])) ? $options['searchdn'] : '';
		$this->filter = (isset($options['filter'])) ? $options['filter'] : '';
	}
	
	/**
	 * Establishes connection to LDAP server.
	 * @return resource LDAP link identifier.
	 * @throws Exception
	 */
	private function connect()
	{
		$connect = ldap_connect($this->server, $this->port);
		if (!$connect)
			throw new Exception("LDAP connect failed.");
		$bind = ldap_bind($connect, $this->basedn, $this->basepass);
		if (!$bind)
			throw new Exception("LDAP bind failed.");
		
		return $connect;
	}
	
	/**
	 * Closes connection to LDAP server.
	 * @param resource $connect LDAP link identifier.
	 * @throws Exception
	 */
	private function disconnect($connect)
	{
		if(!ldap_unbind($connect))
			throw new Exception("LDAP disconnect failed.");
	}
	
	/**
	 * Finds out personal information according to person EID
	 * @param string $eid Person identification in Honeywell company.
	 * @throws Exception
	 */
	public function getPersonalInfo($eid)
	{
		$connect = $this->connect();
		
		$search = ldap_search($connect, $this->searchdn, $this->filter.$eid);
		$info = ldap_get_entries($connect, $search);
		if ($info['count'] == 0)
			throw new Exception("Wrong EID. ".$eid." not found via LDAP.");
		
		$this->disconnect($connect);
		
		return $this->loadLDAPPersonalInfo($info);
	}
	
	/**
	 * Creates LDAPPersonalInfo instance from result of LDAP query. 
	 * @param array $info Complete result of function ldap_get_entries().
	 * @return \LDAPPersonalInfo
	 */
	private function loadLDAPPersonalInfo($info)
	{
		$ldapPersonalInfo = new LDAPPersonalInfo();
		if (isset($info[0]['uid'][0]))
			$ldapPersonalInfo->setEID($info[0]['uid'][0]);
		if (isset($info[0]['givenname'][0]))
			$ldapPersonalInfo->setName($info[0]['givenname'][0]);
		if (isset($info[0]['sn'][0]))
			$ldapPersonalInfo->setSurname($info[0]['sn'][0]);
		if (isset($info[0]['mail'][0]))
			$ldapPersonalInfo->setEmail($info[0]['mail'][0]);
		if (isset($info[0]['telephonenumber'][0]))
			$ldapPersonalInfo->setPhoneNumber($info[0]['telephonenumber'][0]);
		if (isset($info[0]['o'][0]))
			$ldapPersonalInfo->setEmployer($info[0]['o'][0]);
		if (isset($info[0]['departmentnumber'][0]))
			$ldapPersonalInfo->setDepartmentNumber($info[0]['departmentnumber'][0]);
		return $ldapPersonalInfo;
	}
}
