<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing POAttributesAndBudget from database using dibi.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class POAttributesAndBudgetDibiMapper extends DibiMapper implements IPOAttributesAndBudgetMapper
{	
	/**
	 * Creates POAttributesAndBudget instance from DibiRow.
	 * @param DibiRow $row
	 * @return POAttributesAndBudget
	 */
    private function load(DibiRow $row)
    {
		$poAttr = new POAttributesAndBudget();
		$poAttr->POAttributesAndBudgetID = $row->POAttributesAndBudget_ID;
		$poAttr->ProjectID = $row->Project_ID;
		$poAttr->Year = $row->Year;
		$poAttr->POValue = $row->POValue;
		$poAttr->PONumber = $row->PONumber;
		$poAttr->Budget = $row->Budget;
		return $poAttr;
    }
	
	/**
	 * Saves POAttributesAndBudget.
	 * @param POAttributesAndBudget $poAttr
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(POAttributesAndBudget $poAttr)
	{
        try
        {
            $array = array();
            $array['Project_ID'] = $poAttr->ProjectID;
            $array['Year'] = $poAttr->Year;
            $array['POValue'] = $poAttr->POValue;
            $array['PONumber'] = $poAttr->PONumber;
            $array['Budget'] = $poAttr->Budget;
            $db_result = $this->conn->insert('POAttributesAndBudget', $array)->execute();
                        $poAttr->POAttributesAndBudgetID = $this->conn->insertId();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving POAttributesAndBudget with ProjectID = " . $poAttr->ProjectID . "and Budget = " . $poAttr->Budget . " failed. " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Updates POAttributesAndBudget.
	 * @param POAttributesAndBudget $poAttr
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(POAttributesAndBudget $poAttr)
	{
        try
        {
            $array = array();
            $array['Year'] = $poAttr->Year;
            $array['POValue'] = $poAttr->POValue;
            $array['PONumber'] = $poAttr->PONumber;
            $array['Budget'] = $poAttr->Budget;
            $db_result = $this->conn->update('POAttributesAndBudget', $array)->where('[POAttributesAndBudget_ID] = %i', $poAttr->POAttributesAndBudgetID)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("POAttributesAndBudget update with ID = " . $poAttr->POAttributesAndBudgetID . " failed. " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Deletes existing POAttributesAndBudget.
	 * @param int $id POAttributesAndBudget ID.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($id)
	{
		try
		{
		    $db_result = $this->conn->delete('POAttributesAndBudget')->where('[POAttributesAndBudget_ID] = %i', $id)->execute();
			if ($db_result)
				Debugger::log("POAttributesAndBudget with ID = " . $id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of POAttributesAndBudget with ID = " . $id . " failed.", Debugger::ERROR);
			return $db_result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of POAttributesAndBudget with ID = " . $id . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Finds POAttributesAndBudget by ID.
	 * @param int $id POAttributesAndBudget ID.
	 * @return POAttributesAndBudget or NULL if not found.
	 */
	public function find($id)
	{
		$db_result = $this->conn->select('*')->from('POAttributesAndBudget')->where('[POAttributesAndBudget_ID] = %i', $id)->fetch();
		if ($db_result !== FALSE)
			return $this->load($db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds all POAttributesAndBudgets.
	 * @return array Array of POAttributesAndBudget or empty array.
	 */
	public function findAll()
	{
		$db_result = $this->conn->select('*')->from('POAttributesAndBudget')->orderBy('Project_ID', 'Year')->fetchAll();     
		$poAttr_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $row)
                $poAttr_array[$row->POAttributesAndBudget_ID] = $this->load($row);
        }		
		return $poAttr_array;
	}
	
	/**
	 * Finds all POAttributesAndBudgets.
	 * @param int $id Project ID
	 * @return array Array of POAttributesAndBudget or empty array.
	 */
	public function findByProjectID($id)
	{
		$db_result = $this->conn->select('*')->from('POAttributesAndBudget')->where('[Project_ID] = %i', $id)->orderBy('[Year], [POAttributesAndBudget_ID]')->fetchAll();     
		$poAttr_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $row)
                $poAttr_array[$row->POAttributesAndBudget_ID] = $this->load($row);
        }		
		return $poAttr_array;
	}
	
	/**
	 * Sum of budget for project in given year.
	 * @param int $id Project ID
	 * @param int $year Year
	 * @return float Or NULL if error occured
	 */
	public function budgetByProjectIDAndYear($id, $year)
	{
		$db_result = $this->conn->select('SUM([Budget]) AS Sum')->from('POAttributesAndBudget')->where('[Project_ID] = %i AND [Year] = %s', $id, $year)->fetch();
        if ($db_result !== FALSE)
            return $db_result->Sum;
		return NULL;
	}
	
	/**
	 * Sum of PO value for project in given year.
	 * @param int $id Project ID
	 * @param int $year Year
	 * @return float Or NULL if error occured
	 */
	public function poValueByProjectIDAndYear($id, $year)
	{
		$db_result = $this->conn->select('SUM([POValue]) AS Sum')->from('POAttributesAndBudget')->where('[Project_ID] = %i AND [Year] = %s', $id, $year)->fetch();
        if ($db_result !== FALSE)
            return $db_result->Sum;
		return NULL;
	}
	
	/**
	 * Sum of PO number for project in given year.
	 * @param int $id Project ID
	 * @param int $year Year
	 * @return string
	 */
	public function poNumberByProjectIDAndYear($id, $year)
	{
		$db_result = $this->conn->select('[PONumber]')->from('POAttributesAndBudget')->where('[Project_ID] = %i AND [Year] = %s AND [PONumber] IS NOT NULL', $id, $year)->fetchAll();
        $poNumbers = '';
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $row)
			{
				if ($poNumbers)
					$poNumbers .= ' | ';
                $poNumbers .= $row->PONumber;
			}
        }		
		return $poNumbers;
	}
}
