<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Person is company employee that can be assigned
 * to tasks and has his/hers time planned.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class Person extends Object
{
   
	/** Perl formatted regular expression for ID validation */
	const ID_PREG_EXP = '[E|e|H|h|T|e][0-9]{6}|[E|e|H|h|T|t][N|n]{2}[0-9]{4}|^E0$|[TG][0-9]{5}';
    
    const ROLE_PIPM = 'PI',
          ROLE_PMOE = 'PM',
          ROLE_PPaC = 'PP&C',
          ROLE_RESOURCE_OWNER = 'RO',
          ROLE_CLUSTER_LEADER = 'CL',
          ROLE_DEPARTMENT_LEADER = 'DL';
	
    /** @var string Employee identification code */
    private $id;

    /** @var string Employee lastname */
    private $surname;

    /** @var string Employee firstname */
    private $name;

    /** @var string Code of cost center assigned to the employee */
    private $costCenterID;

    /** @var string Exmployee email address */
    private $email;
    
    /** @var int Id of organization unit */ 
    private $OrganizationUnitID;
	
	/** @var string Comment to person */
	private $comment;
	
	/** @var boolean Person activity status */
	private $status  = TRUE;
	
    /** @var int Nationality id */
	private $nationalityID;
	
    /** @var string Employee identification code */
    private $supervisorID;
	
	/** @var boolean Person assignment lock */
	private $locked  = FALSE;

    public function  __construct() 
    {
    }
	
    /**
     * Test validity of EID.
     * 
     * @param $eid string
     * @return boolean Returns TRUE if EID is valid.
     */
	public static function validEID($eid)
	{
		return (preg_match('/' . self::ID_PREG_EXP . '/i', trim($eid)) > 0);
	}

    /**
     * Processes EID and validates it.
     * @param string $eid
     * @return string EID string if valid. NULL if EID is empty.
     * @throws InvalidArgumentException if EID is filled and not valid .
     */
    public static function processEID($eid)
    {
        $eid = trim($eid);

        if ($eid !== '')
        {
            if (Person::validEID($eid))
                return strtoupper($eid);
            // else
            //     throw new InvalidArgumentException('EID is not of valid format.');
        }

        return NULL;
    }
	
    /**
     * Sets person ID.
     * 
     * @param $id string
     */
	public function setID($id)
	{
		$this->id = Person::processEID($id);
	}

    /**
     * Gets person ID.
     * 
     * @return string
     */
	public function getID()
	{
		return $this->id;
	}	
	
    /**
     * Sets firstname.
     * 
     * @param string $name
     */
	public function setFirstname($name)
	{
		$this->name = $name;
	}

    /**
     * Gets firstname.
     * 
     * @return string
     */
	public function getFirstname()
	{
		return $this->name;
	}	
	
    /**
     * Sets surname.
     * 
     * @param string $name
     */
	public function setSurname($name)
	{
		$this->surname = $name;
	}

    /**
     * Gets surname.
     * 
     * @return string
     */
	public function getSurname()
	{
		return $this->surname;
	}	


    /**
     * Returns reversed full name e.g. "Tom Clancy" => "Clancy, Tom".
     * 
     * @return string
     */
	public function getReversedFullName()
	{
		return $this->surname . ((($this->name) != '') ? ', ' . $this->name : '');
	}	


    /**
     * Returns reversed full name with EID e.g. "Tom Clancy E123456" => "Clancy, Tom (E123456)".
     * 
     * @return string
     */
	public function getReversedFullNameWithEID()
	{
		return $this->surname . ((($this->name) != '') ? ', ' . $this->name : '') . ' (' . strtoupper($this->id) . ')';
	}	
	
    /**
     * Returns reversed full name with firtname shortened e.g. "Tom Clancy" => "Clancy, T.".
     * 
     * @return string
     */
	public function getReversedShortName()
	{
		return $this->surname . ((($this->name) != '') ? ', ' . substr($this->name, 0, 1) . '.' : '');
	}	
	
    /**
     * Returns full name formated as usual in western world e.g. "Tom Clancy".
     * 
     * @return string
     */
	public function getFullName()
	{
		return ((($this->name) != '') ? $this->name . ' ' : '') . $this->surname;
	}
	
    /**
     * Returns full name formated as usual in western world with EID e.g. "Tom Clancy (E12345)".
     * 
     * @return string
     */
	public function getFullNameWithEID()
	{
		return ((($this->name) != '') ? $this->name . ' ' : '') . $this->surname . ' (' . $this->id . ')';
	}
	
    /**
     * Returns full name with firtname shortened e.g. "Tom Clancy" => "T. Clancy".
     * 
     * @return string
     */
	public function getShortName()
	{
		return ((($this->name) != '') ? substr($this->name, 0, 1) . '. ' : '') . $this->surname;
	}	

    /**
     * Returns only first characters of name e.g. "Tom Clancy" => "TC".
     * 
     * @return string
     */
	public function getNameInicials()
	{
		return substr($this->name, 0, 1) . substr($this->surname, 0, 1);
	}	
	
	/**
     * Sets cost center ID.
     * 
     * @param string $id
     */
	public function setCostCenterID($id)
	{
		$this->costCenterID = $id;
	}

    /**
     * Gets cost center.
     * 
     * @return string
     */
	public function getCostCenterID()
	{
		return $this->costCenterID;
	}		
	
    /**
     * Sets email.
     * 
     * @param string $email
     */
	public function setEmail($email)
	{
		$this->email = $email;
	}

    /**
     * Gets email.
     * 
     * @return string
     */
	public function getEmail()
	{
		return $this->email;
	}	
    
    /** 
     * Sets OrganizationUnit ID. 
     *  
     * @param int $id  
     */ 
    public function setOrganizationUnitID($id) 
    { 
            $this->OrganizationUnitID = $id; 
    } 

    /** 
     * Gets OrganizationUnit ID. 
     *  
     * @return int 
     */ 
    public function getOrganizationUnitID() 
    { 
            return $this->OrganizationUnitID; 
    } 
    
    /** 
     * Sets Comment. 
     *  
     * @param string $comment  
     */ 
    public function setComment($comment) 
    { 
            $this->comment = $comment; 
    } 

    /** 
     * Gets Comment. 
     *  
     * @return string 
     */ 
    public function getComment() 
    { 
            return $this->comment; 
    } 
	
    /**
	 * Sets person activity flag.
     * @param boolean $status
     */
    public function setStatus($status)
    {
        $this->status = ($status) ? TRUE : FALSE;
    }

    /**
	 * Gets person activity flag.
     * @return boolean
     */
    public function getStatus()
    {
        return $this->status;
    }
	
	/**
	 * Sets person assignment lock.
     * @param boolean $locked
     */
    public function setLocked($locked)
    {
        $this->locked = ($locked) ? TRUE : FALSE;
    }

    /**
	 * Gets person assignment lock.
     * @return boolean
     */
    public function getLocked()
    {
        return $this->locked;
    }	
	
	/**
     * Sets Nationality id.
     * 
     * @param int $id
     */
	public function setNationalityID($id)
	{
		if ($id > 0)
			$this->nationalityID = $id;
		else
			throw new InvalidArgumentException('Licence ID must be integer greater than 0. Wrong param: ' . $id);
	}

    /**
     * Gets Nationality id.
     * 
     * @return int
     */
	public function getNationalityID()
	{
		return $this->nationalityID;
	}
	
    /**
     * Sets person supervisor ID.
     * 
     * @param $id string
     */
	public function setSupervisorID($id)
	{
		$this->supervisorID = $id;
	}

    /**
     * Gets person supervisor ID.
     * 
     * @return string
     */
	public function getSupervisorID()
	{
		return $this->supervisorID;
	}	
}