<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Person repository access class.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 */
class PersonRepository extends Object
{
    /** @var IPersonMapper */
 	private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(IPersonMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'Person-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Person"));
    }	

	/**
	 * Gets Person by id.
	 * 
	 * @param int $id
	 * @return Person or NULL if not found.
	 */
	public function find($id)
	{
		return $this->mapper->find($id);
	}

	/**
	 * Gets All Persons.
	 * 
	 * @return array Array of Persons where EID is key.
	 */
	public function findAll()
	{
		return $this->mapper->findAll();
	}
	
	/**
	 * Finds persons by workcenter.
	 * 
	 * @param string $id workcenter ID.
	 * @return	Person or NULL if not found.
	 */
	public function findAllByWorkCenter($id)
	{
		return $this->mapper->findAllByWorkCenter($id);
	}
	
	/**
	 * Finds all Person by given sbe path.
	 * 
	 * @param int $path organization unit path.
	 * @return array Array of Persons where EID is key.
	 */
	public function findAllByOrgUnitPath($path)
	{
		return $this->mapper->findAllByOrgUnitPath($path);
	}
	
	/**
	 * Finds all persons included into specific organization unit.
	 * 
	 * @param int $id ID of Organization unit.
	 * @return array Array of Persons where EID is key.
	 */
	public function findByOrganizationUnitID($id)
	{
		return $this->mapper->findByOrganizationUnitID($id);
	}

    /**
     * Finds all active people included into specific organization unit.
     * 
     * @param int $id ID of Organization unit.
     * @return array Array of Persons where EID is key.
     */
    public function findActiveByOrganizationUnitID($id)
    {
        return $this->mapper->findActiveByOrganizationUnitID($id);
    }

    /**
     * Gets Persons by PersonFilter.
     *
     * @param PersonFilter $personFilter
     * @return array of Person Objects
     */
    public function getPersonsByFilter(PersonFilter $personFilter)
    {
        return $this->mapper->getPersonsByFilter($personFilter);
    }
    
    /**
     * Gets all persons assigned to project from given year to future.
     * @param id $projectID project ID.
	 * @param int $year year.
     * @return array of Persons. Return array if find succesfull, otherwise returns null. 
     */
    public function findAllPersonsAssignedToProjectInYearToFuture($projectID, $year)
    {
        return $this->mapper->findAllPersonsAssignedToProjectInYearToFuture($projectID, $year);
    }
    
    /**
     * Finds active persons by their role on project.
     * @param string $role Should be in range of Person::ROLE_*
     * @param array $org_units Array of IDs of organization unit where people are members
     * @param array $project_phases Array of IDs of responsible organization unit for project
     * @return array Of Person or empty array.
     */
    public function findByProjectRole($role, $org_units = array(), $project_phases = array())
    {
        return $this->mapper->findByProjectRole($role, $org_units, $project_phases);
    }
    
    /**
     * Finds active persons who own resources
     * @param array $org_units Array of IDs of organization unit where people are members
     * @return array Of Person or empty array.
     */
    public function findResourceOwners($org_units = array())
    {
        return $this->mapper->findResourceOwners($org_units);
    }
	
    /**
     * Gets Persons by PersonFilter. Returns every person that matches
	 * at least one set condition in the filter.
     *
     * @param PersonFilter $personFilter
     * @return array of Person
     */
    public function getPersonsByOrFilter(PersonFilter $personFilter)
    {
        return $this->mapper->getPersonsByOrFilter($personFilter);
	}
	
    /**
     * Updates Person.
     * 
     * @param Person $person
     * @return boolean Returns true if update successfull, otherwiste returns false.
     */
    public function update(Person $person)
    {
		$oldObject = $this->find($person->ID);
        if ($this->mapper->update($person))
		{
			$this->multilog->log('Person with EID: ' . $person->ID . ' updated. ' . $this->multilog->differences($oldObject, $person), ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
    }
	
	/**
	 * Updates person EID.
	 * 
	 * @param string $originEID person ID.
	 * @param string $newEID person ID.
	 */
	public function updateEID($originEID, $newEID)
    {
        if ($this->mapper->updateEID($originEID, $newEID))
		{
			$this->multilog->log('Place holder person EID: ' . $originEID . '  changed to EID: ' . $newEID . ' .', ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
    }
    
    /**
     * Saves new Person.
     * 
     * @param Person $person
     * @return boolean Returns true if save successfull, otherwiste returns false.
     */
    public function save(Person $person)
    {
        if ($this->mapper->save($person))
		{
			$this->multilog->log('Person with EID: ' . $person->ID . ' saved.', ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
    }
	
    /**
     * Deletes Person.
     * 
     * @param string $id
     * @return boolean Returns true if delete successfull, otherwiste returns false.
     */
    public function delete($id)
    {
        if ($this->mapper->delete($id))
		{
			$this->multilog->log('Person with EID: ' . $id . ' deleted.', ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
    }
    
    /**
     * Returns leaders of organization units according to type of organization units.
     * 
     * @param array $orgUnitType Should be array of values in range of ORG_UNIT_TYPE_* defined in OrganizationUnit.
     * @return array of Person or empty array in case of empty result.
     */
    public function getLeadersByOrgUnitType(array $orgUnitType)
    {
        return $this->mapper->getLeadersByOrgUnitType($orgUnitType);
    }
	
/**
     * Returns leaders of organization units according to type of organization units and SBE.
     * @param array $orgUnitType Should be array of values in range of ORG_UNIT_TYPE_* defined in OrganizationUnit.
	 * @param array $sbeIDs Array of Organizaiton unit(SBE) IDs
     * @return array of Person or empty array in case of empty result.
     */
    public function getLeadersByOrgUnitTypeAndSBE(array $orgUnitType, array $sbeIDs)
    {
        return $this->mapper->getLeadersByOrgUnitTypeAndSBE($orgUnitType, $sbeIDs);
    }
	
	/**
     * Gets all persons assigned to task in given year.
     * @param int $taskID task ID.
	 * @param int $year year.
     * @return array of Persons. Return array if find succesfull, otherwise returns null. 
     */
    public function findAllPersonsAssignedToTaskInYear($taskID, $year)
    {
        return $this->mapper->findAllPersonsAssignedToTaskInYear($taskID, $year);
    }
	
	/**
     * Gets all persons assigned to task.
     * @param int $taskID task ID.
     * @return array of Persons. Return array if find succesfull, otherwise returns empty array. 
     */
    public function findAllPersonsAssignedToTask($taskID)
    {
		return $this->mapper->findAllPersonsAssignedToTask($taskID);
    }
	
	/**
     * Gets all persons eid charged on task in given year.
     * @param string $chargeNumber Charge number.
     * @return array of EID. Return array if find succesfull, otherwise returns empty array. 
     */
    public function findAllPersonsChargedOnTask($chargeNumber)
    {
		return $this->mapper->findAllPersonsChargedOnTask($chargeNumber);
    }
	
	/**
     * Gets all persons eid charged on task in given year.
     * @param string $chargeNumber Charge number.
	 * @param int $year year.
     * @return array of EID. Return array if find succesfull, otherwise returns empty array. 
     */
    public function findAllPersonsChargedOnTaskInYear($chargeNumber, $year)
    {
		return $this->mapper->findAllPersonsChargedOnTaskInYear($chargeNumber, $year);
    }
	
	/**
	 * Gets all people, who have the 'TG' prefix in their EID.
	 * In fact, they are placeholders to shadow Technical Groups.
	 * 
	 * @return array 
	 */
	public function findAllTGs()
	{
		return $this->mapper->findAllTGs();
	}
}
