<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package model\ProjectCharter
 */

/**
 * Service performs automatic approval of project charters 
 * which final date for approval has passed.
 * 
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class ProjectCharterAutoApprovalService {
	
    /** @const value of maximum execution time if it is not setup in GlobalSettings in seconds */
    const MAXIMUM_EXECUTION_TIME = 300; // 5 minutes

	/** @var DIContainer DI context with application repositories access */
    private $context;

	/** @var ILogger Logging facility */
    private $logger;

	/**
	 * Creates service.
	 *  
	 * @param $context Application context.
	 * @param ILogger Object for logging the import progress.
	 */
    public function  __construct(DIContainer $context, ILogger $logger)
    {
        $this->context  = $context;
		$this->logger   = $logger;
    }
    
	/**
	 * Perfoms auto-approval on all projects pending approval (final approval date over).
	 *  
	 * @return boolean True if auto-approval was successful for all projects after final approval date and false if at least one of them failed to be approved.
	 */
	public function runAutoApproval() 
	{
		$success = true;
		
        $maximum_execution_time  = GlobalSettingReaderHelper::get('charters.autoApprovalMaxExecTimeSeconds');
		if ($maximum_execution_time === NULL)
			$maximum_execution_time = ProjectCharterAutoApprovalService::MAXIMUM_EXECUTION_TIME;
		ini_set('max_execution_time', $maximum_execution_time);
        if (ini_get('max_execution_time') != $maximum_execution_time)
            throw new EProjectCharterAutoApprovalServiceException("Setting maximum execution time to '$maximum_execution_time' failed, check server (php.ini) settings");
		
		$lock = $this->context->createLock('CHARTER_AUTO_APPROVAL');
		if (!$lock->lock())
            throw new EProjectCharterAutoApprovalServiceException("Charter automatical approval is already running (active lock detected).");
		
		try 
		{
			$filter = new ProjectCharterFilter();
			$filter->setStatus(array(ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_FINAL_APPROVAL));
			$charters = $this->context->projectCharterRepository->getChartersByFilter($filter);
			//Debugger::log("Number of charters for auto approval: " . count($charters), ILogger::DEBUG);
			foreach ($charters as $charter) 
			{
				$approval_request = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($charter->ID);
				if (is_null($approval_request))
					throw new EProjectCharterAutoApprovalServiceException("Failed to read final approval request (history) for charter with ID = " . $charter->ID . " in status '".$charter->Status."'.", ILogger::DEBUG);
				//Debugger::log("Picked for auto approval: Charter ID=".$charter->getID().", Name='".$charter->getName()."', Status: " . $charter->getStatus(). ", FinalApprovalDate: " . date("c", $approval_request->getFinalApprovalDate()), ILogger::DEBUG);
				$approval_limit_ts = $approval_request->getFinalApprovalDate(); 
				if (time() > $approval_limit_ts) { // auto approve
					$approvers = $this->context->projectCharterApproverRepository->findAllApprover($approval_request->ProjectCharterID, $approval_request->Version);
					if (is_null($approvers))
						throw new EProjectCharterAutoApprovalServiceException("Failed to get list of approvers for charter with ID = " . $charter->ID . " in status '".$charter->Status."'.");
					// if nobody is against approval of charter, it will be approved
					$all_agree = true;
					foreach ($approvers as $approver)
					{
						if ($approver->getApproval() == ProjectCharterApprover::DECISION_PENDING) 
						{
							$approver->setApproval(ProjectCharterApprover::DECISION_APPROVED);
							$approver->setAutoApproved(true);
							$approver->setDecisionDateTime(date("Y-m-d H:i:s"));
							if ($this->context->projectCharterApproverRepository->update($approver))
								$this->logger->log("Charter for project \"".$charter->Name."\" (ID=".$charter->ID.") was auto-approved for approver \"".$approver->Surname.",".$approver->Name."\" (EID=".$approver->ApproverEID.") in role \"".$approver->Role."\".", ILogger::NOTICE);
							else {
								$this->logger->log("Failed auto-approval of charter for project \"".$charter->Name."\" (ID=".$charter->ID.") for of approver \"".$approver->Surname.",".$approver->Name."\" (EID=".$approver->ApproverEID.") in role \"".$approver->Role."\".", ILogger::ERROR);
								// skipping auto approval of this project charter
								$success = false;
								continue 2;
							}
						}
						elseif ($approver->getApproval() == ProjectCharterApprover::DECISION_DISAPPROVED)
						{
							$all_agree = false;
						}
					}
					$charter->setStatus($all_agree ? ProjectCharter::PROJECT_CHARTER_STATUS_APPROVED : ProjectCharter::PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED);
					if ($this->context->projectCharterRepository->update($charter))
						$this->logger->log("Charter for project '".$charter->Name."' (ID=".$charter->ID.") was approved automatically.", ILogger::NOTICE);
					else
						$this->logger->log("Charter auto-approval: Failed to change state for charter for project '".$charter->Name."' (ID=".$charter->ID.").", ILogger::ERROR);
				}
			}
			$lock->unlock();
			return $success;
		}
        catch (EProjectCharterAutoApprovalServiceException $e)
        {
            $this->logger->log($e->getMessage(), ILogger::ERROR);
            Debugger::log($e->getMessage(), Debugger::ERROR);
			$lock->unlock();
            return FALSE;
        }
	}
}

class EProjectCharterAutoApprovalServiceException extends Exception {}
