<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Project charter budget definition.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterBudget extends Object
{
	/* possible project charter budget types */
	const PROJECT_CHARTER_BUDGET_HOURS = 'HRS';
	const PROJECT_CHARTER_BUDGET_TRAVEL = 'TRV';
	const PROJECT_CHARTER_BUDGET_MATERIAL = 'MAT';
        const PROJECT_CHARTER_BUDGET_SUBCONTRACTING = 'SUB';

    /** @var int ID */
    private $id;
    /** @var int ID of project charter */
    private $projectCharterID;
    /** @var int Version of project charter budget */
    private $version;
    /** @var string Resource type */
    private $resourceType;
    /** @var int ID of organizaiton unit(only tech. group). */
    private $technicalGroup;
    /** @var int ID of subcontractor */
    private $subcontractor;
    /** @var float Frozen labor rate */
    private $rate;
    /** @var float Value of project charter budget for first quarter. */
    private $budgetQ1;
    /** @var float Value of project charter budget for second quarter. */
    private $budgetQ2;
    /** @var float Value of project charter budget for third quarter. */
    private $budgetQ3;
    /** @var float Value of project charter budget for fourth quarter. */
    private $budgetQ4;    
    
   /**
     * Sets ID of project charter budget.
     * @param int $id Must be positive integer.
     */
	public function setID($id)
	{
		if ($id > 0)
			$this->id = $id;
		else
			throw new InvalidArgumentException('Project charter budget ID must be integer greater than 0. Wrong param: '.$id);
	}
	
    /**
     * Gets ID of project charter budget.
     * @return int
     */
	public function getID()
	{
		return $this->id;
	}
        
    /**
     * Set ID of project charter.
     * @param int $id Must be positive integer.
     * @throws InvalidArgumentException if parameter is not integer greater than 0.
     */
        public function setProjectCharterID($id)
        {
            if ($id > 0)
                $this->projectCharterID = $id;
            else
                throw new InvalidArgumentException('Project charter ID must be integer greater than 0. Wrong param: '.$id);
        }
        
    /**
     * Gets ID of project charter.
     * @return int
     */
        public function getProjectCharterID()
        {
            return $this->projectCharterID;
        }
    
    /**
     * Sets version of project charter budget.
     * @param int $version Must be positive integer.
     */
	public function setVersion($version)
	{
		if ($version > 0)
			$this->version = $version;
		else
			throw new InvalidArgumentException('Project charter budget version must be positive integer. Wrong param: '.$version);
	}
	
    /**
     * Gets version of project charter budget.
     * @return int
     */
	public function getVersion()
	{
		return $this->version;
	}
    
	/**
     * Sets resource of project charter budget.
     * @param string $resourceType
     */
	public function setResourceType($resourceType)
	{
		if (in_array($resourceType, array_keys(ProjectCharterBudget::getResources())))
			$this->resourceType = $resourceType;
		else
			throw new InvalidArgumentException('Unknown project charter budget resource.');
	}

    /**
     * Gets resource of project charter budget.
     * @return string
     */
	public function getResourceType()
	{
		return $this->resourceType;
	}
    
   /**
     * Sets organization unit(only tech. group) ID of project charter budget.
     * @param int $technicalGroup Must be positive integer or NULL.
     */
	public function setTechnicalGroup($technicalGroup)
	{
		if ((NumericUtils::isPositiveInteger($technicalGroup)) || ($technicalGroup === NULL))
			$this->technicalGroup = $technicalGroup;
		else
			throw new InvalidArgumentException('Technical group ID must be positive integer or NULL. Wrong param: '.$technicalGroup);
	}
	
    /**
     * Gets organization unit(only tech. group) ID of project charter budget.
     * @return int
     */
	public function getTechnicalGroup()
	{
		return $this->technicalGroup;
	}
    
   /**
     * Sets subcontractor ID of project charter budget.
     * @param int $subcontractor Must be positive integer or NULL.
     */
	public function setSubcontractor($subcontractor)
	{
		if ((NumericUtils::isPositiveInteger($subcontractor)) || ($subcontractor === NULL))
			$this->subcontractor = $subcontractor;
		else
			throw new InvalidArgumentException('Subcontractor ID must be positive integer or NULL. Wrong param: '.$subcontractor);
	}
	
    /**
     * Gets subcontractor ID of project charter budget.
     * @return int
     */
	public function getSubcontractor()
	{
		return $this->subcontractor;
	}

    /**
     * Sets rate value
     * @param float $rate
     * @throws InvalidArgumentException
     */
    public function setRate($rate)
    {
        if ($rate === NULL)
            $this->rate = $rate;
        else if (is_numeric($rate))
            $this->rate = 0.0 + $rate;
        else
            throw new InvalidArgumentException('Rate must be numeric value. Wrong param: ' . $rate);
    }

    /**
     * Gets rate value
     * @return float Or NULL if rate is undefined
     */
    public function getRate()
    {
        return $this->rate;
    }
        
    /**
     * Sets values of project charter budget.
     * @param float $value Must be positive integer or NULL.
     */
    public function setBudgetQ1($value)
    {
                if ($value === NULL)
                    $this->budgetQ1 = $value;
                else if (NumericUtils::isNonNegativeInteger($value))
		    $this->budgetQ1 = 0.0 + $value; // to float
		else
                    throw new InvalidArgumentException('Value must be integer number.');
    }
    
    /**
     * Gets value of project charter budget.
     * @return float
     */
    public function getBudgetQ1()
    {
        return $this->budgetQ1;
    }
        
    /**
     * Sets values of project charter budget.
     * @param float $value Must be positive integer or NULL.
     */
    public function setBudgetQ2($value)
    {
		if ($value === NULL)
                    $this->budgetQ2 = $value;
                else if (NumericUtils::isNonNegativeInteger($value))
		    $this->budgetQ2 = 0.0 + $value; // to float
		else
                    throw new InvalidArgumentException('Value must be integer number.');
    }
    
    /**
     * Gets value of project charter budget.
     * @return float
     */
    public function getBudgetQ2()
    {
        return $this->budgetQ2;
    }
        
    /**
     * Sets values of project charter budget.
     * @param float $value Must be positive integer or NULL.
     */
    public function setBudgetQ3($value)
    {
		if ($value === NULL)
                    $this->budgetQ3 = $value;
                else if (NumericUtils::isNonNegativeInteger($value))
		    $this->budgetQ3 = 0.0 + $value; // to float
		else
                    throw new InvalidArgumentException('Value must be integer number.');
    }
    
    /**
     * Gets value of project charter budget.
     * @return float
     */
    public function getBudgetQ3()
    {
        return $this->budgetQ3;
    }
        
    /**
     * Sets values of project charter budget.
     * @param float $value Must be positive integer or NULL.
     */
    public function setBudgetQ4($value)
    {
		if ($value === NULL)
                    $this->budgetQ4 = $value;
                else if (NumericUtils::isNonNegativeInteger($value))
		    $this->budgetQ4 = 0.0 + $value; // to float
		else
                    throw new InvalidArgumentException('Value must be integer number.');
    }
    
    /**
     * Gets value of project charter budget.
     * @return float
     */
    public function getBudgetQ4()
    {
        return $this->budgetQ4;
    }
    
    /**
     * Returns array of available resources.
     * @return array
     */
	public static function getResources()
	{
		return array(	self::PROJECT_CHARTER_BUDGET_HOURS => self::PROJECT_CHARTER_BUDGET_HOURS,
                                self::PROJECT_CHARTER_BUDGET_TRAVEL => self::PROJECT_CHARTER_BUDGET_TRAVEL,
                                self::PROJECT_CHARTER_BUDGET_MATERIAL => self::PROJECT_CHARTER_BUDGET_MATERIAL,
                                self::PROJECT_CHARTER_BUDGET_SUBCONTRACTING => self::PROJECT_CHARTER_BUDGET_SUBCONTRACTING
                            );
	}
}