<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Project charter budget repository access class.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterBudgetRepository  extends Object
{
	/** @var IProjectCharterBudgetMapper */
	private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

	public function __construct(IProjectCharterBudgetMapper $mapper)
	{
		$this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'ProjectCharterBudget-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Project Charter Budget"));
	}

	/**
	 * Saves project charter budget.
	 * 
	 * @param ProjectCharterBudget $projectCharterBudget
	 */
	public function save(ProjectCharterBudget $projectCharterBudget)
	{
		if($this->mapper->save($projectCharterBudget))
		{
			$this->multilog->log('Project Charter Budget for charter with ID: ' . $projectCharterBudget->ProjectCharterID . ' and version: ' . $projectCharterBudget->Version . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Updates project charter budget.
	 * 
	 * @param ProjectCharterBudget $projectCharterBudget
	 */
	public function update(ProjectCharterBudget $projectCharterBudget)
	{
		$oldObject = $this->find($projectCharterBudget->ID);
		if($this->mapper->update($projectCharterBudget))
		{
			$this->multilog->log('Project Charter Budget for charter with ID: ' . $projectCharterBudget->ProjectCharterID . ' and version: ' . $projectCharterBudget->Version . ' updated. ' . $this->multilog->differences($oldObject, $projectCharterBudget), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes existing project charter budget.
	 * 
	 * @param int $projectCharterBudget_id
	 * @return boolean
	 */
	public function delete($projectCharterBudget_id)
	{
		if($this->mapper->delete($projectCharterBudget_id))
		{
			$this->multilog->log('Project Charter Budget with ID: ' . $projectCharterBudget_id . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes all Project Charter Budgets with specified Project Charter ID and Version.
	 * 
	 * @param int $projectCharterID
	 * @param int $version
	 * @return boolean
	 */
	public function deleteByProjectCharterIDAndVersion($projectCharterID, $version)
	{
		if($this->mapper->deleteByProjectCharterIDAndVersion($projectCharterID, $version))
		{
			$this->multilog->log('Project Charter Budgets with Charter ID = ' . $projectCharterID . ' and Version = ' . $version . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Finds project charter buget by id.
	 * 
	 * @param int $id project charter budget ID.
	 * @return	ProjectCharterBudget or NULL if not found.
	 */
	public function find($id)
	{
				return $this->mapper->find($id);
	}

	/**
	 * Finds all project charter budgets.
	 * 
	 * @param int $projectCharterID Should be ID of existing project charter.
	 * @param int $version Version of ProjectCharterBudgetHistory.
	 * @return array Array of ProjectCharterBudget.
	 */
	public function findByProjecCharterIDAndVersion($projectCharterID, $version)
	{
		return $this->mapper->findByProjecCharterIDAndVersion($projectCharterID, $version);
	}
	
	/**
	 * Finds latest veriosn of project charter budget by project charter id.
	 * 
	 * @param int $projectCharterID Project charter ID.
	 * @return	array Array of ProjectCharterBudget where ProjectCharterBudget_Id is key or NULL if not found.
	 */
	public function findProjectCharterBudgetByProjectCharterID($projectCharterID)
	{
		return $this->mapper->findProjectCharterBudgetByProjectCharterID($projectCharterID);
	}

	/**
	 * Finds project charter budget by Organization unit id.
	 * 
	 * @param int $OrgUnit_ID Organizaiton unit ID.
	 * @return	array Array of ProjectCharterBudget where ProjectCharterBudget_Id is key or NULL if not found.
	 */
	public function findProjectCharterBudgetByOrgUnitId($OrgUnit_ID)
	{
		return $this->mapper->findProjectCharterBudgetByOrgUnitId($OrgUnit_ID);
	}

	/**
	 * Finds latest veriosn of project charter budget where is resource type HRS by Organization unit id.
	 * 
	 * @param int $OrgUnit_ID Organizaiton unit ID.
	 * @return	array Array of ProjectCharterBudget where ProjectCharterBudget_Id is key or NULL if not found.
	 */
	public function findProjectCharterBudgetHRSByOrgUnitId($OrgUnit_ID)
	{
		return $this->mapper->findProjectCharterBudgetHRSByOrgUnitId($OrgUnit_ID);
	}

	/**
	 * Finds all budget items which was created in latest version of given responsible person
	 *
	 * @param int $charterId
	 * @param string $eid
	 * @return array array(int[project charter budget id] => ProjecCharterBudget)
	 */
	public function findLatestByCharterIdAndResponsible($charterId, $eid)
	{
		return $this->mapper->findLatestByCharterIdAndResponsible($charterId, $eid);
	}
}
