<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * ProjectPlanningCharterState is used to hold the state of the planning module.
 * 
 * Planning is ongoing, if there is any record with 'State' set to PROJECT_PLANNING_CHARTER_STATE_PLANNINGONGOING.
 *
 * @author Leos Navratil <leos.navratil@honeywell.com>
 */
class ProjectPlanningCharterState extends Object
{
	/* If there isn't a record in the state table, then we aren't planning this
	 * project in this year.
	 * The planning life cycle is as follows:
	 * 1. No entry in the DB
	 * 2. The planning is ongoing (started by 'Enter Planning')
	 * 3. Charter is created from the planning module (using the create button) and it can be updated from planning.
	 * 4. Planning is finished. This PlanningCharterState is the same as 3. state, but in addition it states that the finish
	 *	  button has been clicked, because then the ProjectCharter will be unlocked for all changes.
	 */
	const PROJECT_PLANNING_CHARTER_STATE_PLANNINGONGOING = 'PlanningOngoing';
	const PROJECT_PLANNING_CHARTER_STATE_CHARTEREXISTS = 'CharterExists';
	const PROJECT_PLANNING_CHARTER_STATE_FINISHED = 'Finished';
	
	/** @var int Project ID */
	private $projectID;
	/** @var int Planning year */
	private $year;
	/** @var int Workpackage ID */
	private $workpackageID;
	/** @var int ProjectCharter ID */
	private $projectCharterID;
	/** @var string The state, in which the planning for project in this year currently is */
	private $state;
	/** @var bool Flag, which states, whether the planning finished at least once. */
	private $atLeastOnceFinished;
	
	public function __construct()
	{
	}
	
	/**
	 * Sets Project ID.
	 * 
	 * @param int $id
	 */
	public function setProjectID($id)
	{
		if (is_int($id + 0))
		{
			$this->projectID = $id;
		}
		else
		{
			throw new InvalidArgumentException('Project ID must be an integer. Wrong param: ' . $id);
		}
	}
	
	/**
	 * Gets Project ID.
	 * 
	 * @return int
	 */
	public function getProjectID()
	{
		return $this->projectID;
	}
	
	/**
	 * Sets Year.
	 * 
	 * @param int $year
	 */
	public function setYear($year)
	{
		if (is_int($year + 0))
		{
			$this->year = $year;
		}
		else
		{
			throw new InvalidArgumentException('Year must be an integer. Wrong param: ' . $year);
		}
	}

	/**
	 * Gets Year.
	 * 
	 * @return int
	 */
	public function getYear()
	{
		return $this->year;
	}

	/**
	 * Sets Workpackage ID.
	 * 
	 * @param int $wp
	 */
	public function setWorkpackageID($wp)
	{
		if (is_int($wp + 0))
		{
			$this->workpackageID = $wp;
		}
		else
		{
			throw new InvalidArgumentException('Workpackage ID must be an integer. Wrong param: ' . $wp);
		}
	}
	
	/**
	 * Gets Workpackage ID.
	 * 
	 * @return int
	 */
	public function getWorkpackageID()
	{
		return $this->workpackageID;
	}
	
	/**
	 * Sets ProjectCharter ID.
	 * 
	 * @param int $projectCharterID
	 */
	public function setProjectCharterID($projectCharterID)
	{
		if (is_int($projectCharterID + 0))
		{
			$this->projectCharterID = $projectCharterID;
		}
		else
		{
			throw new InvalidArgumentException('ProjectCharter ID must be an integer. Wrong param: ' . $projectCharterID);
		}
	}
	
	/**
	 * Gets ProjectCharter ID.
	 * 
	 * @return int
	 */
	public function getProjectCharterID()
	{
		return $this->projectCharterID;
	}
	
	/**
	 * Sets State.
	 * 
	 * @param string $state
	 */
	public function setState($state)
	{
		if (in_array($state, array_keys(ProjectPlanningCharterState::getStates())))
		{
			$this->state = $state;
		}
		else
		{
			throw new InvalidArgumentException('State must be from the enum. Wrong param: ' . $state);
		}
	}
	
	/**
	 * Gets State, in which the planning module for this (Project_ID, Year) is.
	 * 
	 * @return string
	 */
	public function getState()
	{
		return $this->state;
	}

	/**
	 * Returns an array of available States.
	 * 
	 * @return array
	 */
	public static function getStates()
	{
		return array(
			self::PROJECT_PLANNING_CHARTER_STATE_PLANNINGONGOING => self::PROJECT_PLANNING_CHARTER_STATE_PLANNINGONGOING,
			self::PROJECT_PLANNING_CHARTER_STATE_CHARTEREXISTS => self::PROJECT_PLANNING_CHARTER_STATE_CHARTEREXISTS,
			self::PROJECT_PLANNING_CHARTER_STATE_FINISHED => self::PROJECT_PLANNING_CHARTER_STATE_FINISHED
		);
	}
	
	/**
	 * Sets AtLeastOnceFinished flag.
	 * 
	 * @param bool $atLeastOnceFinished
	 */
	public function setAtLeastOnceFinished($atLeastOnceFinished)
	{
		if (!is_bool($atLeastOnceFinished))
			throw new InvalidArgumentException('ProjectPlanningCharterState must be a boolean.');

		$this->atLeastOnceFinished = $atLeastOnceFinished;
	}
	
	/**
	 * Gets AtLeastOnceFinished flag.
	 * 
	 * @return bool
	 */
	public function getAtLeastOnceFinished()
	{
		return $this->atLeastOnceFinished;
	}
	
}