<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Project planning charter state repository access class.
 *
 * @author Leos Navratil <leos.navratil@honeywell.com>
  */
class ProjectPlanningCharterStateRepository extends Object
{
	/** @var IProjectPlanningCharterStateMapper */
	private $mapper;
	
	/** @var MultiLogger */
	private $multilog;
	
	public function __construct(IProjectPlanningCharterStateMapper $mapper)
	{
		$this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'ProjectPlanningCharterState-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Project Planning Charter State"));
	}

	/**
	 * Saves project planning state.
	 * 
     * @param ProjectPlanningCharterState $projectPlanningCharterState Project planning charter state
	 * @return boolean
	 */
	public function save(ProjectPlanningCharterState $projectPlanningCharterState)
	{
		if($this->mapper->save($projectPlanningCharterState))
		{
			$this->multilog->log('Project Planning Charter State with Project ID = ' . $projectPlanningCharterState->getProjectID() . ' and Year = ' . $projectPlanningCharterState->getYear()
					. ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Updates project planning state.
	 * 
     * @param ProjectPlanningCharterState $projectPlanningCharterState Project planning charter state
	 * @return boolean
	 */
	public function update(ProjectPlanningCharterState $projectPlanningCharterState)
	{
		$oldObject = $this->find($projectPlanningCharterState->ProjectID, $projectPlanningCharterState->Year);
		if($this->mapper->update($projectPlanningCharterState))
		{
			$this->multilog->log('Project Planning Charter with Project ID = ' . $projectPlanningCharterState->getProjectID() . ' and Year = ' . $projectPlanningCharterState->getYear()
					. ' updated. ' . $this->multilog->differences($oldObject, $projectPlanningCharterState), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes existing project planning state.
	 * 
	 * @param int $projectID Project's ID
	 * @param int $year Year
	 * @return boolean
	 */
	public function delete($projectID, $year)
	{
		if($this->mapper->delete($projectID, $year))
		{
			$this->multilog->log('Project planning charter with Project ID = ' . $projectID . ' and Year = ' . $year
					. ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Returns project planning charter state by its Project ID and Year.
	 * 
	 * @param int $projectID Project's ID
	 * @param int $year Year
	 * @return ProjectPlanningCharterState
	 */
	public function find($projectID, $year)
	{
		return $this->mapper->find($projectID, $year);
	}
	
	/**
	 * Returns project planning charter state by Project Charter ID and Year.
	 * 
	 * @param int $projectCharterID Project Charter's ID
	 * @param int $year Year
	 * @return ProjectPlanningCharterState
	 */
	public function findByProjectCharterIDAndYear($projectCharterID, $year)
	{
		return $this->mapper->findByProjectCharterIDAndYear($projectCharterID, $year);
	}
	
	/**
	 * Returns all project planning charter states.
	 * 
	 * @return array of ProjectPlanningCharterState or empty array if not found
	 */
	public function findAll()
	{
		return $this->mapper->findAll();
	}
}
