<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Project charter repository access class.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterRepository extends Object
{
    
    /** @var IProjectCharterMapper */
 	private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(IProjectCharterMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'ProjectCharter-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Project Charter"));
    }
    
	/**
	 * Saves project charter.
	 * 
	 * @param ProjectCharter $projectCharter
	 */
	public function save(ProjectCharter $projectCharter)
	{
		if($this->mapper->save($projectCharter))
		{
			$this->multilog->log('Project Charter with ID: ' . $projectCharter->ID . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Updates project charter.
	 * 
	 * @param ProjectCharter $projectCharter
	 */
	public function update(ProjectCharter $projectCharter)
	{
		$oldObject = $this->find($projectCharter->ID);
		if($this->mapper->update($projectCharter))
		{
			$this->multilog->log('Project Charter with ID: ' . $projectCharter->ID . ' updated. ' . $this->multilog->differences($oldObject, $projectCharter), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes existing project charter.
	 * 
	 * @param int $projectCharter_id
	 * @return boolean
	 */
	public function delete($projectCharter_id)
	{
		if($this->mapper->delete($projectCharter_id))
		{
			$this->multilog->log('Project Charter with ID: ' . $projectCharter_id . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
    
	/**
	 * Finds project charter by id.
	 * 
	 * @param int $id project charter ID.
	 * @return	ProjectCharter or NULL if not found.
	*/
	public function find($id)
	{
		return $this->mapper->find($id);
	}
        
    /**
     * Gets ProjectCharter by ProjectCharterFilter. Returns every project charters that matches all
     * set conditions.
     *
     * @param ProjectCharterFilter $charterFilter
     * @return array of ProjectCharter
    */
    public function getChartersByFilter(ProjectCharterFilter $charterFilter)
    {
            return $this->mapper->getChartersByFilter($charterFilter);
    }

	/**
	 * Finds project charters by Organization Unit id.
	 * 
	 * @param int $id Organization Unit ID.
     * @param int $year Year.
	 * @return	array Array of ProjectCharter where key is ProjectCharter_ID or NULL if not found.
	*/
	public function findProjectCharterByOrgUnitID($id, $year)
	{
        return $this->mapper->findProjectCharterByOrgUnitID($id, $year);
    }
        
    /**
     * Returns all years for that has been planned.
     * @return array arr[$year] == $year or empty array
    */
    public function getYears()
    {
        return $this->mapper->getYears();
    }
        
	/**
	 * Finds project charters by line manager.
	 * 
	 * @param int $id EID of line manager.
     * @param int $year Year.
	 * @return	array Array of ProjectCharter where key is ProjectCharter_ID or NULL if not found.
	*/
	public function findProjectCharterByLM($id, $year)
	{
        return $this->mapper->findProjectCharterByLM($id, $year);
	}
	
	/**
	 * Finds planning year of charter specified by its ID.
	 * 
	 * @param int $projectCharterID
	 * @return mixed ProjectPlanningCharter or NULL
	 */
	public function findYearByProjectCharterID($projectCharterID)
	{
		return $this->mapper->findYearByProjectCharterID($projectCharterID);
	}
	
	/**
	 * Finds all project charters in year.
	 * 
     * @param int $year Year.
	 * @return	array Array of ProjectCharter where key is ProjectCharter_ID or NULL if not found.
	*/
	public function findAll($year)
	{
        return $this->mapper->findAll($year);
	}
}
