<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * Service for STRAP report.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class STRAPReportService extends object
{
    /** @var DIContainer DI context with application repositories access */
    private $context;

	/** @var ILogger Logging facility */
    private $logger;
	
	/** @var String Value for freeze user interface during report generation. */
    private $uiFreezeToken;
	
    /** @var string Path to storage TT files. */
    private $strap_storage_dir;
	
	/**
	 * Creates service for tracking table export.
	 *  
	 * @param $context Application context.
	 * @param ILogger Object for logging the import progress.
	 */
    public function  __construct(DIContainer $context, ILogger $logger, $uiFreezeToken = null)
    {
        $this->context = $context;
		$this->logger = $logger;
		$this->uiFreezeToken = $uiFreezeToken;
		$this->strap_storage_dir = Environment::getConfig('strap_storage_dir');
    }
	
    /**
     * Generates STRAP report as a XLSX (MS-EXCEL file).
     */
    public function generateReport()
    {
		$objPHPExcel = new PHPExcel();
		
        $memory_limit = "512M";
        if (!ini_set("memory_limit", $memory_limit)) 
		{
			$this->flashMessage('Setup of memory limit failed. Contact administrator.', 'error');
			trigger_error('Unable to setup memory limit to ' . $memory_limit . '.', E_USER_ERROR);
		}
		
        //increase executon time limit
        $exec_time = 420;
		//lock check
		$lock = $this->context->createLock('STRAP_Report', $exec_time);
		if (!$lock->lock())
		{
			$this->logger->log('STRAP report generation start failed - active lock detected: ' . date('m-d-Y H:i:s'), ILogger::INFO);
			throw new StrapReportGenerationException("Export of STRAP source data is already running.");
		}
		//increase executon time limit
	    if (!ini_set("max_execution_time", $exec_time)) 
		{
			$this->flashMessage('Setup of maximum execution time failed. Contact administrator.', 'error');
			trigger_error('Unable to setup maximum execution time to '.$exec_time.'.', E_USER_ERROR);
		}
		$this->logger->log('STRAP reprot generation start: ' . date('m-d-Y H:i:s'), ILogger::INFO);
		
		// excel file creation
		$sheet = $objPHPExcel->getActiveSheet();
		$sheet->setTitle('STRAP source data');
		$rowIterator = $sheet->getRowIterator();
		$cellIterator = $rowIterator->current()->getCellIterator();
		
		// STRAP source data part
		// Header
		$cellIterator->current()->setValue("STRAP Item");
		$cellIterator->next();
		$cellIterator->current()->setValue("Item ID");
		$cellIterator->next();
		$cellIterator->current()->setValue("Department");
		$cellIterator->next();
		$cellIterator->current()->setValue("Cluster");
		$cellIterator->next();
		$cellIterator->current()->setValue("Item Target TRL");
		$cellIterator->next();
		$cellIterator->current()->setValue("Item TRL Year");
		$cellIterator->next();
		$cellIterator->current()->setValue("Activity");
		$cellIterator->next();
		$cellIterator->current()->setValue("Activity ID");
		$cellIterator->next();
		$cellIterator->current()->setValue("Type");
		$cellIterator->next();
		$cellIterator->current()->setValue("Status");
		$cellIterator->next();
		$cellIterator->current()->setValue("Customer");
		$cellIterator->next();
		$cellIterator->current()->setValue("Activity Target TRL");
		$cellIterator->next();
		$cellIterator->current()->setValue("Activity TRL Year");
		$cellIterator->next();
		$cellIterator->current()->setValue("Technology Roadmap Domain");
		$cellIterator->next();
		$cellIterator->current()->setValue("Produc Line");
		$cellIterator->next();
		$cellIterator->current()->setValue("M&PM Sub-Portfolio");
		$cellIterator->next();
		$cellIterator->current()->setValue("M&PM Portfolio");
		$cellIterator->next();
		$cellIterator->current()->setValue("RTC");
		$cellIterator->next();
		$cellIterator->current()->setValue("CostShare");
		$cellIterator->next();
		$cellIterator->current()->setValue("Project budget usage");
		$cellIterator->next();
		$cellIterator->current()->setValue("Potential Customer");
		$cellIterator->next();
		$cellIterator->current()->setValue("Probability of Win");
		$cellIterator->next();
		$cellIterator->current()->setValue("Year");
		$cellIterator->next();
		$cellIterator->current()->setValue("Labor");
		$cellIterator->next();
		$cellIterator->current()->setValue("Subcontracting");
		$cellIterator->next();
		$cellIterator->current()->setValue("Travel");
		$cellIterator->next();
		$cellIterator->current()->setValue("Material");
		$cellIterator->next();
		$cellIterator->current()->setValue("Total");
		$cellIterator->next();
		$cellIterator->current()->setValue("RDE");
		$cellIterator->next();
		$cellIterator->current()->setValue("ActivityID");
		$cellIterator->next();
		$cellIterator->current()->setValue("Department short");
		$cellIterator->next();
		$cellIterator->current()->setValue("Multiple statuses");
		$cellIterator->next();
		$cellIterator->current()->setValue("Multiple customers");
		$cellIterator->next();
		$cellIterator->current()->setValue("Project");
		$cellIterator->next();
		$cellIterator->current()->setValue("Licence");
		$cellIterator->next();
		$cellIterator->current()->setValue("Remarks");
		$cellIterator->next();
		$cellIterator->current()->setValue("First Pursuit");
		$cellIterator->next();
		$cellIterator->current()->setValue("Generation time");
		$row = $cellIterator->current()->getRow();
		$col = $cellIterator->current()->getColumn();
		$sheet->setCellValue($col . ($row + 1), date('m-d-Y H:i:s'));
		$rowIterator->next();
        $cellIterator = $rowIterator->current()->getCellIterator();
		// Data generation
		$items = $this->context->itemRepository->findAll();
		foreach($items as $item)
		{
			if($item->Active == 1)
			{
				// get data from STRAP item
				$clus = $this->context->organizationUnitRepository->find($item->ResponsibleOrgUnitID);
				$dep = $this->context->organizationUnitRepository->find($clus->Parent_ID);
				//find all activities
				$activities = $this->context->itemActivityRepository->findByItemID($item->ID);
				foreach($activities as $activity)
				{
					if($activity->Active == 1)
					{
						//get data from STRAP activity
						$TRD = $this->context->technologyRoadmapDomainRepository->find($activity->TechnologyRoadmapDomain);
						if($TRD == null)
							$TRD = '#N/A';
						else
							$TRD = $TRD->Domain;
						$PL = $this->context->productLineRepository->find($activity->ProductLine);
						if($PL == null)
							$PL = '#N/A';
						else
							$PL = $PL->ProductName;
						$MaPM = $this->context->advisorRepository->find($activity->MarketingProductManager);
						if($MaPM == null)
							$MaPM = '#N/A';
						else
							$MaPM = $MaPM->Name;
						$PM = $this->context->advisorRepository->find($activity->PortfolioManager);
						if($PM == null)
							$PM = '#N/A';
						else
							$PM = $PM->Name;
						$RTCD = $this->context->advisorRepository->find($activity->RTCDirector);
						if($RTCD == null)
							$RTCD = '#N/A';
						else
							$RTCD = $RTCD->Name;
						switch($activity->Type)
						{
							//project
							case $activity::ITEM_ACTIVITY_TYPE_PROJECT :
								$costShare = $activity->CostShare*100;
								$projectBudgetUsage = $activity->ProjectBudgetUsage*100;
								$potentialCustomer = '';
								$probabilityOfWin = 100;
							break;
							//pursuit
							case $activity::ITEM_ACTIVITY_TYPE_PURSUIT :
								$pursuit = $this->context->pursuitRepository->findAllPursuitsByActivityID($activity->ID);
								if (count($pursuit) == 1)
								{
									$key = key($pursuit);
									$probabilityOfWin = $pursuit[$key]->ProbabilityOfWin*100;
									$costShare = $pursuit[$key]->CostShare*100;
									$projectBudgetUsage = 0;
									$potentialCustomer = '';
								}
								else
								{
									$probabilityOfWin = 0;
									$costShareSum = 0;
									$costshareCount = 0;
									foreach($pursuit as $pur)
									{
										$probabilityOfWin += $pur->ProbabilityOfWin;
										$costshareCount++;
										$costShareSum += $pur->CostShare*100;
									}
									if($probabilityOfWin > 1)
										$probabilityOfWin = 100;
									else
										$probabilityOfWin = $probabilityOfWin * 100;
									// aggregate average for costshare
									$costShare = $costShareSum / $costshareCount;
									$finalCostShare = 0;
									switch ($costShare)
									{
										case ($costShare >= 0 && $costShare < 25) :
											$finalCostShare = 0;
											break;
										case ($costShare >= 25 && $costShare <= 75) :
											$finalCostShare = 50;
											break;
										case ($costShare > 75 && $costShare <= 100) :
											$finalCostShare = 100;
											break;
										default:
											$finalCostShare = 0;
									}
									$costShare = $finalCostShare;
									$projectBudgetUsage = 0;
									$potentialCustomer = '';
								}

							break;
							//need to pursue
							case $activity::ITEM_ACTIVITY_TYPE_NEED_TO_PURSUE :
								$costShare = $activity->CostShare*100;
								$projectBudgetUsage = 0;
								$potentialCustomer = $activity->PotentialCustomer;
								$probabilityOfWin = 0;
							break;
						}
						// get budget values
						for($year = Date('Y'); $year <= (Date('Y')+10); $year++)
						{
							if($activity->Type == $activity::ITEM_ACTIVITY_TYPE_PROJECT && $year == Date('Y'))
							{
								$project = $this->context->projectRepository->find($activity->ProjectID);
								$reporter = new ProjectEACReporter($this->context, $project, Date('Y'));
								try
								{
									$labor = $reporter->getProjectLabourPlannedCostsMonthTotals()->sum();
								}
								catch (InvalidStateException $e)
								{
									$lock->unlock();
									$this->flashMessage($e->getMessage(), 'error');
									$this->redirect('default');
								}
								$material = $reporter->getProjectTravelPlannedCostsMonthTotals()->sum();
								$travel = $reporter->getProjectMaterialPlannedCostsMonthTotals()->sum();
								$subcontracting = $reporter->getProjectSubcontractingPlannedCostsMonthTotals()->sum();
							}
								else
								{
								$budget = $this->context->itemActivityYearlyBudgetRepository->find($activity->ID, $year);
								if($budget != null)
								{
									$labor = $budget->LaborCosts;
									$subcontracting = $budget->SubcontractingCosts;
									$travel = $budget->TravelCosts;
									$material = $budget->MaterialCosts;
								}
								else
								{
									$labor = 0;
									$subcontracting = 0;
									$travel = 0;
									$material = 0;
								}
							}
							$licence = $this->context->ecLicenceRepository->find($activity->ECLicenceID);
							$activityProject = $this->context->projectRepository->find($activity->ProjectID);
							$activityProjectName = ($activityProject === NULL) ? '' : $activityProject->Name;
							// assumes there's just one pursuit to each activity
							// can be more, but we have an agreement there won't be, remember?
							if ($activity->Type === ItemActivity::ITEM_ACTIVITY_TYPE_PURSUIT)
								$firstPursuitName = $pursuit[key($pursuit)]->Name;
							else
								$firstPursuitName = '';
							
							$budgetSum = $labor + $subcontracting + $travel + $material;
							// EXPORT DATA TO EXCEL OBJECT
							$cellIterator->current()->setValue($item->Name);
							$cellIterator->next();
							$cellIterator->current()->setValue($item->CustomID);
							$cellIterator->next();
							$cellIterator->current()->setValue($dep->Name);
							$cellIterator->next();
							$cellIterator->current()->setValue($clus->Name);
							$cellIterator->next();
							$cellIterator->current()->setValue($item->TargetTRL);
							$cellIterator->next();
							$cellIterator->current()->setValue($item->TargetTRLYear);
							$cellIterator->next();
							$cellIterator->current()->setValue($activity->Name);
							$cellIterator->next();
							$cellIterator->current()->setValue($activity->CustomID);
							$cellIterator->next();
							$cellIterator->current()->setValue($activity->Type);
							$cellIterator->next();
							$cellIterator->current()->setValue($activity->Status);
							$cellIterator->next();
							$cellIterator->current()->setValue($activity->Customer);
							$cellIterator->next();
							if($activity->TargetTRL == null)
								$cellIterator->current()->setValue(0);
							else
								$cellIterator->current()->setValue($activity->TargetTRL);
							$cellIterator->next();
							if($activity->TargetTRLYear == null)
								$cellIterator->current()->setValue(0);
							else
								$cellIterator->current()->setValue($activity->TargetTRLYear);
							$cellIterator->next();
							$cellIterator->current()->setValue($TRD);
							$cellIterator->next();
							$cellIterator->current()->setValue($PL);
							$cellIterator->next();
							$cellIterator->current()->setValue($MaPM);
							$cellIterator->next();
							$cellIterator->current()->setValue($PM);
							$cellIterator->next();
							$cellIterator->current()->setValue($RTCD);
							$cellIterator->next();
							$cellIterator->current()->setValue($costShare);
							$cellIterator->next();
							$cellIterator->current()->setValue($projectBudgetUsage);
							$cellIterator->next();
							$cellIterator->current()->setValue($potentialCustomer);
							$cellIterator->next();
							$cellIterator->current()->setValue($probabilityOfWin);
							$cellIterator->next();
							$cellIterator->current()->setValue($year);
							$cellIterator->next();
							$cellIterator->current()->setValue($labor);
							$cellIterator->next();
							$cellIterator->current()->setValue($subcontracting);
							$cellIterator->next();
							$cellIterator->current()->setValue($travel);
							$cellIterator->next();
							$cellIterator->current()->setValue($material);
							$cellIterator->next();
							$cellIterator->current()->setValue($budgetSum);
							$cellIterator->next();
							$cellIterator->current()->setValue($budgetSum*(100 - $costShare)/100);
							$cellIterator->next();
							$cellIterator->current()->setValue($activity->ID);
							$cellIterator->next();
							$cellIterator->current()->setValue($dep->Short);
							$cellIterator->next();
							if($activity->Status == ItemActivity::ITEM_ACTIVITY_STATUS_MULTPLE)
								$cellIterator->current()->setValue($activity->getMultipleStatuses());
							else
								$cellIterator->current()->setValue();
							$cellIterator->next();
							if($activity->Customer == ItemActivity::ITEM_ACTIVITY_CUSTOMER_MULTIPLE)
								$cellIterator->current()->setValue($activity->getMultipleCustomers());
							else
								$cellIterator->current()->setValue();
							$cellIterator->next();
							$cellIterator->current()->setValue($activityProjectName);
							$cellIterator->next();
							$cellIterator->current()->setValue($licence->Name);
							$cellIterator->next();
							$cellIterator->current()->setValue($activity->Remarks);
							$cellIterator->next();
							$cellIterator->current()->setValue($firstPursuitName);
							$rowIterator->next();
							$cellIterator = $rowIterator->current()->getCellIterator();
							// EXPORT DATA TO EXCEL OBJECT END
						}
					}
				}
			}
		}
		// Data generation end
		// STRAP source data part end
		
		// B&P Pursuits part
        $sheet5 = $objPHPExcel->createSheet(1);
		$sheet5->setTitle('B&P Pursuits');
		$rowIterator = $sheet5->getRowIterator();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// Header
		$cellIterator->current()->setValue("Name");
		$cellIterator->next();
		$cellIterator->current()->setValue("Status");
		$cellIterator->next();
		$cellIterator->current()->setValue("Manager");
		$cellIterator->next();
		$cellIterator->current()->setValue("Investigator");
		$cellIterator->next();
		$cellIterator->current()->setValue("SFDC Number");
		$cellIterator->next();
		$cellIterator->current()->setValue("Probability of Win");
		$cellIterator->next();
		$cellIterator->current()->setValue("Cost Share");
		$cellIterator->next();
		$cellIterator->current()->setValue("Document Storage");
		$cellIterator->next();
		$cellIterator->current()->setValue("Customer");
		$cellIterator->next();
		$cellIterator->current()->setValue("Program");
		$cellIterator->next();
		$cellIterator->current()->setValue("Call");
		$cellIterator->next();
		$cellIterator->current()->setValue("Call Closing Date");
		$cellIterator->next();
		$cellIterator->current()->setValue("Remarks");
		$cellIterator->next();
		$milestones = PursuitPlan::getMilestones();
		foreach($milestones as $milestone)
		{
			$cellIterator->current()->setValue('Milestone ' . $milestone);
			$cellIterator->next();
			$cellIterator->current()->setValue("Planned Date");
			$cellIterator->next();
			$cellIterator->current()->setValue("Actual Date");
			$cellIterator->next();
			$cellIterator->current()->setValue("B&P Plan USD");
			$cellIterator->next();
			$cellIterator->current()->setValue("B&P Actuals USD");
			$cellIterator->next();
		}
		$cellIterator->current()->setValue("Total Budget");
		$cellIterator->next();
		$cellIterator->current()->setValue("ActivityID");
		$cellIterator->next();
		$cellIterator->current()->setValue("Department short");
		$rowIterator->next();
        $cellIterator = $rowIterator->current()->getCellIterator();
		
		/** SECOND ROW NEEEDED FOR EXCEL DATA TYPE DEFINITION **/
		$cellIterator->current()->setValue("Name");
		$cellIterator->next();
		$cellIterator->current()->setValue("Status");
		$cellIterator->next();
		$cellIterator->current()->setValue("Manager");
		$cellIterator->next();
		$cellIterator->current()->setValue("Investigator");
		$cellIterator->next();
		$cellIterator->current()->setValue("SFDC Number");
		$cellIterator->next();
		$cellIterator->current()->setValue(0);
		$cellIterator->next();
		$cellIterator->current()->setValue(0);
		$cellIterator->next();
		$cellIterator->current()->setValue("Document Storage");
		$cellIterator->next();
		$cellIterator->current()->setValue("Customer");
		$cellIterator->next();
		$cellIterator->current()->setValue("Program");
		$cellIterator->next();
		$cellIterator->current()->setValue("Call");
		$cellIterator->next();
		$cellIterator->current()->setValue("2014-04-16");
		$cellIterator->next();
		$cellIterator->current()->setValue("Remarks");
		$cellIterator->next();
		$milestones = PursuitPlan::getMilestones();
		foreach($milestones as $milestone)
		{
			$cellIterator->current()->setValue('Milestone ' . $milestone);
			$cellIterator->next();
			$cellIterator->current()->setValue("2014-04-16");
			$cellIterator->next();
			$cellIterator->current()->setValue("2014-04-16");
			$cellIterator->next();
			$cellIterator->current()->setValue(0);
			$cellIterator->next();
			$cellIterator->current()->setValue(0);
			$cellIterator->next();
		}
		$cellIterator->current()->setValue(0);
		$cellIterator->next();
		$cellIterator->current()->setValue(0);
		$cellIterator->next();
		$cellIterator->current()->setValue("Department short");
		$rowIterator->next();
        $cellIterator = $rowIterator->current()->getCellIterator();
		/** END - SECOND ROW NEEEDED FOR EXCEL DATA TYPE DEFINITION - END **/

		//Data generation and export
		$pursuits = $this->context->pursuitRepository->findAll();
		foreach($pursuits as $pursuit)
		{
			$activities = $this->context->itemActivityRepository->findAllActivitiesByPursuitID($pursuit->PursuitID);
			$pursuitPlans = $this->context->pursuitPlanRepository->findByPursuitID($pursuit->PursuitID);
			$totalBudget = 0;
			foreach($activities as $activity)
			{
				$activityBudgets = $this->context->itemActivityYearlyBudgetRepository->findAllByID($activity->ID);
				foreach($activityBudgets as $budget)
				{
					$totalBudget += $budget->LaborCosts + $budget->SubcontractingCosts + $budget->TravelCosts + $budget->MaterialCosts;
				}
			}
			//
			if(empty($activities))
			{
				$cellIterator->current()->setValue($pursuit->Name);
				$cellIterator->next();
				if($pursuit->StatusFullName == Pursuit::FULL_NAME_STATUS_CLOSED)
					$cellIterator->current()->setValue($pursuit->StatusFullName . ' - ' . $pursuit->ClosureResultFullName);
				else
					$cellIterator->current()->setValue($pursuit->StatusFullName);
				$cellIterator->next();
				if($pursuit->PursuitManager)
					$cellIterator->current()->setValue($this->context->personRepository->find($pursuit->PursuitManager)->ReversedFullName);
				else
					$cellIterator->current()->setValue('#N/A');
				$cellIterator->next();
				if($pursuit->PursuitPI)
					$cellIterator->current()->setValue($this->context->personRepository->find($pursuit->PursuitPI)->ReversedFullName);
				else
					$cellIterator->current()->setValue('#N/A');
				$cellIterator->next();
				$cellIterator->current()->setValue($pursuit->SFDCNumber);
				$cellIterator->next();
				$cellIterator->current()->setValue($pursuit->ProbabilityOfWin*100);
				$cellIterator->next();
				$cellIterator->current()->setValue($pursuit->CostShare*100);
				$cellIterator->next();
				$cellIterator->current()->setValue($pursuit->DocumentStorage);
				$cellIterator->next();
				if($pursuit->CustomerID)
					$cellIterator->current()->setValue($this->context->customerRepository->find($pursuit->CustomerID)->Name);
				else
					$cellIterator->current()->setValue('#N/A');
				$cellIterator->next();
				if($pursuit->ProgramID)
					$cellIterator->current()->setValue($this->context->customerProgramRepository->find($pursuit->ProgramID)->Program);
				else
					$cellIterator->current()->setValue('#N/A');
				$cellIterator->next();
				if($pursuit->CallID)
					$cellIterator->current()->setValue($this->context->customerProgramCallRepository->find($pursuit->CallID)->Call);
				else
					$cellIterator->current()->setValue('#N/A');
				$cellIterator->next();
				if($pursuit->CallID)
				{
					if($this->context->customerProgramCallRepository->find($pursuit->CallID)->ClosingDate)
						$cellIterator->current()->setValue($this->context->customerProgramCallRepository->find($pursuit->CallID)->ClosingDate);
					else
						$cellIterator->current()->setValue('#N/A');
				}
				else
					$cellIterator->current()->setValue('#N/A');
				$cellIterator->next();
				$cellIterator->current()->setValue($pursuit->Remarks);
				$cellIterator->next();				
				foreach($pursuitPlans as $key => $plan)
				{
					$cellIterator->current()->setValue($plan->Milestone);
					$cellIterator->next();
					if($plan->PlannedDate)
						$cellIterator->current()->setValue($plan->PlannedDate);
					else
						$cellIterator->current()->setValue('#N/A');
					$cellIterator->next();
					if($plan->ActualDate)
						$cellIterator->current()->setValue($plan->ActualDate);
					else
						$cellIterator->current()->setValue('#N/A');
					$cellIterator->next();
					$cellIterator->current()->setValue($plan->BnPPlan);
					$cellIterator->next();
					$cellIterator->current()->setValue($plan->BnPActuals);
					$cellIterator->next();
				}
				$cellIterator->current()->setValue($totalBudget);
				$cellIterator->next();
				$cellIterator->current()->setValue('#N/A');
				$cellIterator->next();
				$cellIterator->current()->setValue('#N/A');
				$rowIterator->next();
				$cellIterator = $rowIterator->current()->getCellIterator();				
			}
			else
			{
				foreach($activities as $activity)
				{
					if($activity->Active == 1)
					{
						$cellIterator->current()->setValue($pursuit->Name);
						$cellIterator->next();
						if($pursuit->StatusFullName == Pursuit::FULL_NAME_STATUS_CLOSED)
							$cellIterator->current()->setValue($pursuit->StatusFullName . ' - ' . $pursuit->ClosureResultFullName);
						else
							$cellIterator->current()->setValue($pursuit->StatusFullName);
						$cellIterator->next();
						if($pursuit->PursuitManager)
							$cellIterator->current()->setValue($this->context->personRepository->find($pursuit->PursuitManager)->ReversedFullName);
						else
							$cellIterator->current()->setValue('#N/A');
						$cellIterator->next();
						if($pursuit->PursuitPI)
							$cellIterator->current()->setValue($this->context->personRepository->find($pursuit->PursuitPI)->ReversedFullName);
						else
							$cellIterator->current()->setValue('#N/A');
						$cellIterator->next();
						$cellIterator->current()->setValue($pursuit->SFDCNumber);
						$cellIterator->next();
						$cellIterator->current()->setValue($pursuit->ProbabilityOfWin*100);
						$cellIterator->next();
						$cellIterator->current()->setValue($pursuit->CostShare*100);
						$cellIterator->next();
						$cellIterator->current()->setValue($pursuit->DocumentStorage);
						$cellIterator->next();
						if($pursuit->CustomerID)
							$cellIterator->current()->setValue($this->context->customerRepository->find($pursuit->CustomerID)->Name);
						else
							$cellIterator->current()->setValue('#N/A');
						$cellIterator->next();
						if($pursuit->ProgramID)
							$cellIterator->current()->setValue($this->context->customerProgramRepository->find($pursuit->ProgramID)->Program);
						else
							$cellIterator->current()->setValue('#N/A');
						$cellIterator->next();
						if($pursuit->CallID)
							$cellIterator->current()->setValue($this->context->customerProgramCallRepository->find($pursuit->CallID)->Call);
						else
							$cellIterator->current()->setValue('#N/A');
						$cellIterator->next();
						if($pursuit->CallID)
						{
							if($this->context->customerProgramCallRepository->find($pursuit->CallID)->ClosingDate)
								$cellIterator->current()->setValue($this->context->customerProgramCallRepository->find($pursuit->CallID)->ClosingDate);
							else
								$cellIterator->current()->setValue('#N/A');
						}
						else
							$cellIterator->current()->setValue('#N/A');
						$cellIterator->next();
						$cellIterator->current()->setValue($pursuit->Remarks);
						$cellIterator->next();				
						foreach($pursuitPlans as $key => $plan)
						{
							$cellIterator->current()->setValue($plan->Milestone);
							$cellIterator->next();
							if($plan->PlannedDate)
								$cellIterator->current()->setValue($plan->PlannedDate);
							else
								$cellIterator->current()->setValue('#N/A');
							$cellIterator->next();
							if($plan->ActualDate)
								$cellIterator->current()->setValue($plan->ActualDate);
							else
								$cellIterator->current()->setValue('#N/A');
							$cellIterator->next();
							$cellIterator->current()->setValue($plan->BnPPlan);
							$cellIterator->next();
							$cellIterator->current()->setValue($plan->BnPActuals);
							$cellIterator->next();
						}
						$cellIterator->current()->setValue($totalBudget);
						$cellIterator->next();
						$cellIterator->current()->setValue($activity->ID);
						$cellIterator->next();
						$item = $this->context->itemRepository->find($activity->ItemID);
						$orgUnit = $this->context->organizationUnitRepository->find($item->ResponsibleOrgUnitID);
						$dep = $this->context->organizationUnitRepository->find($orgUnit->Parent_ID);
						$cellIterator->current()->setValue($dep->Short);
						$rowIterator->next();
						$cellIterator = $rowIterator->current()->getCellIterator();
					}
				}
			}
		}
		// B&P Pursuits part END

		// LABOR part
        $sheet2 = $objPHPExcel->createSheet(2);
		$sheet2->setTitle('Labor');
		$rowIterator = $sheet2->getRowIterator();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// Header
		$cellIterator->current()->setValue("SBU");
		$cellIterator->next();
		$cellIterator->current()->setValue("Department");
		$cellIterator->next();
		$cellIterator->current()->setValue("Department short");
		$cellIterator->next();
		$cellIterator->current()->setValue("Cluster");
		$cellIterator->next();
		$cellIterator->current()->setValue("Cluster short");
		$cellIterator->next(); 
		$cellIterator->current()->setValue("Technical Group"); 
		$cellIterator->next();
		$cellIterator->current()->setValue("EID");
		$cellIterator->next();
		$cellIterator->current()->setValue("Surname");
		$cellIterator->next();
		$cellIterator->current()->setValue("Name");
		$cellIterator->next();
		$cellIterator->current()->setValue("Work Center");
		$cellIterator->next();
		$cellIterator->current()->setValue("Comment");
		$cellIterator->next();
		$cellIterator->current()->setValue("Status");
		$cellIterator->next();
		$cellIterator->current()->setValue("Year");
		$cellIterator->next();
		$cellIterator->current()->setValue("Yield");
		$cellIterator->next();
		$cellIterator->current()->setValue("Contract");
		$cellIterator->next();
		$cellIterator->current()->setValue("Jan");
		$cellIterator->next();
		$cellIterator->current()->setValue("Feb");
		$cellIterator->next();
		$cellIterator->current()->setValue("Mar");
		$cellIterator->next();
		$cellIterator->current()->setValue("Apr");
		$cellIterator->next();
		$cellIterator->current()->setValue("May");
		$cellIterator->next();
		$cellIterator->current()->setValue("Jun");
		$cellIterator->next();
		$cellIterator->current()->setValue("Jul");
		$cellIterator->next();
		$cellIterator->current()->setValue("Aug");
		$cellIterator->next();
		$cellIterator->current()->setValue("Sep");
		$cellIterator->next();
		$cellIterator->current()->setValue("Oct");
		$cellIterator->next();
		$cellIterator->current()->setValue("Nov");
		$cellIterator->next();
		$cellIterator->current()->setValue("Dec");
		$cellIterator->next();
		$cellIterator->current()->setValue("Total");
		$cellIterator->next();
		$cellIterator->current()->setValue("Unit");
		$rowIterator->next();
        $cellIterator = $rowIterator->current()->getCellIterator();
		
		//Data generation and export
        $persons = $this->context->personRepository->findAll();		
        foreach ($persons as $person)
		{
			$orgUnit = $this->context->organizationUnitRepository->find($person->OrganizationUnitID);
			// SBE
			$sbe = null;
			if($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_SBE)
				$sbe = $orgUnit->Name;
			elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_DEP)
				$sbe = $this->context->organizationUnitRepository->find($orgUnit->Parent_ID)->Name;
			elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_CLUS)
				$sbe = $this->context->organizationUnitRepository->find($this->context->organizationUnitRepository->find($orgUnit->Parent_ID)->Parent_ID)->Name;
			elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_TG)
				$sbe = $this->context->organizationUnitRepository->find($this->context->organizationUnitRepository->find($this->context->organizationUnitRepository->find($orgUnit->Parent_ID)->Parent_ID)->Parent_ID)->Name;
			// Departmnet
			$dep = null;
			if($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_SBE)
				$dep = 'none';
			elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_DEP)
				$dep = $orgUnit->Name;
			elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_CLUS)
				$dep = $this->context->organizationUnitRepository->find($orgUnit->Parent_ID)->Name;
			elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_TG)
				$dep = $this->context->organizationUnitRepository->find($this->context->organizationUnitRepository->find($orgUnit->Parent_ID)->Parent_ID)->Name;
			// Departmnet Short
			$depShort = null;
			if($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_SBE)
				$depShort = 'none';
			elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_DEP)
			{
				if($orgUnit->Short)
					$depShort = $orgUnit->Short;
				else
					$depShort = 'Not defined';
			}
			elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_CLUS)
			{
				if($this->context->organizationUnitRepository->find($orgUnit->Parent_ID)->Short)
					$depShort = $this->context->organizationUnitRepository->find($orgUnit->Parent_ID)->Short;
				else
					$depShort = 'Not defined';
			}
			elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_TG)
			{
				if($this->context->organizationUnitRepository->find($this->context->organizationUnitRepository->find($orgUnit->Parent_ID)->Parent_ID)->Short)
					$depShort = $this->context->organizationUnitRepository->find($this->context->organizationUnitRepository->find($orgUnit->Parent_ID)->Parent_ID)->Short;
				else
					$depShort = 'Not defined';
			}
			// Cluster
			$clus = null;
			if($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_SBE)
				$clus = 'none';
			elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_DEP)
				$clus = 'none';
			elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_CLUS)
				$clus = $orgUnit->Name;
			elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_TG)
				$clus = $this->context->organizationUnitRepository->find($orgUnit->Parent_ID)->Name;
			// Cluster short
			$clusShort = null;
			if($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_SBE)
				$clusShort = 'none';
			elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_DEP)
				$clusShort = 'none';
			elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_CLUS)
			{
				if($orgUnit->Short)
					$clusShort = $orgUnit->Short;
				else
					$clusShort = 'Not defined';
			}
			elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_TG)
			{
				if($this->context->organizationUnitRepository->find($orgUnit->Parent_ID)->Short)
					$clusShort = $this->context->organizationUnitRepository->find($orgUnit->Parent_ID)->Short;
				else
					$clusShort = 'Not defined';
			}
			// Technical Group
			$tg = null;
			if($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_SBE)
				$tg = 'none';
			elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_DEP)
				$tg = 'none';
			elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_CLUS)
				$tg = 'none';
			elseif($orgUnit->Type == OrganizationUnit::ORG_UNIT_TYPE_TG)
				$tg = $orgUnit->Name;
			for($year = Date('Y'); $year <= (Date('Y')+10); $year++)
			{
				$cellIterator->current()->setValue($sbe);
				$cellIterator->next();
				$cellIterator->current()->setValue($dep);
				$cellIterator->next();
				$cellIterator->current()->setValue($depShort);
				$cellIterator->next();
				$cellIterator->current()->setValue($clus);
				$cellIterator->next();
				$cellIterator->current()->setValue($clusShort);
				$cellIterator->next(); 
				$cellIterator->current()->setValue($tg);
				$cellIterator->next();
				$cellIterator->current()->setValue($person->ID);
				$cellIterator->next();
				$cellIterator->current()->setValue($person->Surname);
				$cellIterator->next();
				$cellIterator->current()->setValue($person->Firstname);
				$cellIterator->next();
				$cellIterator->current()->setValue($person->CostCenterID);
				$cellIterator->next();
				$cellIterator->current()->setValue($person->Comment);
				$cellIterator->next();
				if($person->Status)
					$cellIterator->current()->setValue('Active');
				else
					$cellIterator->current()->setValue('Inactive');
				$cellIterator->next();
				$cellIterator->current()->setValue($year);
				$cellIterator->next();
				if(!$this->context->workableHourRepository->isAOP($year, $person->ID))
					$cellIterator->current()->setValue($this->context->workableHourRepository->getYield($person->ID, $year)*100);
				else
					$cellIterator->current()->setValue('null');
				$cellIterator->next();
				if(!$this->context->workableHourRepository->isAOP($year, $person->ID))
					$cellIterator->current()->setValue($this->context->workableHourRepository->getContract($person->ID, $year)*100);
				else
					$cellIterator->current()->setValue('null');
				$cellIterator->next();
				$aop_hours = $this->context->aopRepository->findForPerson($year, $person->ID);
				if ($aop_hours != NULL)
				{
					for ($i = 1; $i <= 12; $i++)
					{
						$cellIterator->current()->setValue($aop_hours->getMonthAOPHours($i));
						$cellIterator->next();
					}
					$cellIterator->current()->setValue(array_sum($aop_hours->getAOP()));
					$cellIterator->next();
				}
				else
				{			
					for ($i = 1; $i <= 12; $i++)
					{
						$cellIterator->current()->setValue(0);
						$cellIterator->next();	
					}
					$cellIterator->current()->setValue(0);
					$cellIterator->next();	
				}
				$cellIterator->current()->setValue("hrs");
				$rowIterator->next();
				$cellIterator = $rowIterator->current()->getCellIterator();
			}			
		}
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// LABOR part END
		
		// Labor Rates part
        $sheet3 = $objPHPExcel->createSheet(3);
		$sheet3->setTitle('Labor Rates');
		$rowIterator = $sheet3->getRowIterator();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// Header
		$cellIterator->current()->setValue("Work Center");
		$cellIterator->next();
		$cellIterator->current()->setValue("ID");
		$cellIterator->next();
		$cellIterator->current()->setValue("Year");
		$cellIterator->next();
		$cellIterator->current()->setValue("Jan");
		$cellIterator->next();
		$cellIterator->current()->setValue("Feb");
		$cellIterator->next();
		$cellIterator->current()->setValue("Mar");
		$cellIterator->next();
		$cellIterator->current()->setValue("Apr");
		$cellIterator->next();
		$cellIterator->current()->setValue("May");
		$cellIterator->next();
		$cellIterator->current()->setValue("Jun");
		$cellIterator->next();
		$cellIterator->current()->setValue("Jul");
		$cellIterator->next();
		$cellIterator->current()->setValue("Aug");
		$cellIterator->next();
		$cellIterator->current()->setValue("Sep");
		$cellIterator->next();
		$cellIterator->current()->setValue("Oct");
		$cellIterator->next();
		$cellIterator->current()->setValue("Nov");
		$cellIterator->next();
		$cellIterator->current()->setValue("Dec");
		$cellIterator->next();
		$cellIterator->current()->setValue("Unit");
		$cellIterator->next();
		$rowIterator->next();
        $cellIterator = $rowIterator->current()->getCellIterator();
		
		//Data generation and export
        $labour_centers = $this->context->costCenterRepository->findAllCostCenters();		
        foreach ($labour_centers as $center)
		{
			for($year = Date('Y'); $year <= (Date('Y')+10); $year++)
			{
				$cellIterator->current()->setValue($center->Name);
				$cellIterator->next();
				$cellIterator->current()->setValue($center->ID);
				$cellIterator->next();
				$rate = $center->getYearLabourRate($year);
				if ($rate != NULL)
				{
					$cellIterator->current()->setValue($year);
					$cellIterator->next();			
					for ($i = 1; $i <= 12; $i++)
					{
						$cellIterator->current()->setValue($rate->getMonthlyHourRate($i));
						$cellIterator->next();	
					}
				}
				else
				{
					$cellIterator->current()->setValue($year);
					$cellIterator->next();			
					for ($i = 1; $i <= 12; $i++)
					{
						$cellIterator->current()->setValue(0);
						$cellIterator->next();	
					}
				}
				$cellIterator->current()->setValue("USD/hrs");
				$rowIterator->next();
				$cellIterator = $rowIterator->current()->getCellIterator();
			}
		}
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// Labor Rates part END
		
		// AOP calendar part
        $sheet4 = $objPHPExcel->createSheet(4);
		$sheet4->setTitle('AOP calendar');
		$rowIterator = $sheet4->getRowIterator();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// Header
		$cellIterator->current()->setValue("Year");
		$cellIterator->next();
		$cellIterator->current()->setValue("Jan");
		$cellIterator->next();
		$cellIterator->current()->setValue("Feb");
		$cellIterator->next();
		$cellIterator->current()->setValue("Mar");
		$cellIterator->next();
		$cellIterator->current()->setValue("Apr");
		$cellIterator->next();
		$cellIterator->current()->setValue("May");
		$cellIterator->next();
		$cellIterator->current()->setValue("Jun");
		$cellIterator->next();
		$cellIterator->current()->setValue("Jul");
		$cellIterator->next();
		$cellIterator->current()->setValue("Aug");
		$cellIterator->next();
		$cellIterator->current()->setValue("Sep");
		$cellIterator->next();
		$cellIterator->current()->setValue("Oct");
		$cellIterator->next();
		$cellIterator->current()->setValue("Nov");
		$cellIterator->next();
		$cellIterator->current()->setValue("Dec");
		$cellIterator->next();
		$cellIterator->current()->setValue("Total");
		$cellIterator->next();
		$cellIterator->current()->setValue("Unit");
		$rowIterator->next();
        $cellIterator = $rowIterator->current()->getCellIterator();
		
		//Data generation and export	
		for($year = Date('Y'); $year <= (Date('Y')+10); $year++)
		{
			$cellIterator->current()->setValue($year);
			$cellIterator->next();	
			$aop_hours = $this->context->aopRepository->find($year);
			if ($aop_hours != NULL)
			{		
				for ($i = 1; $i <= 12; $i++)
				{
					$cellIterator->current()->setValue($aop_hours->getMonthAOPHours($i));
					$cellIterator->next();
				}
				$cellIterator->current()->setValue(array_sum($aop_hours->getAOP()));
				$cellIterator->next();
			}
			else
			{			
				for ($i = 1; $i <= 12; $i++)
				{
					$cellIterator->current()->setValue(0);
					$cellIterator->next();	
				}
				$cellIterator->current()->setValue(0);
				$cellIterator->next();	
			}
			$cellIterator->current()->setValue("hrs");
			$rowIterator->next();
			$cellIterator = $rowIterator->current()->getCellIterator();
		}
		// AOP calendar part END
		
		//UI freeze
		if(!is_null($this->uiFreezeToken))
			setcookie('fileDownloadToken', $this->uiFreezeToken);

        $objWriter = PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel2007');
        if (!file_exists($this->strap_storage_dir)) 
          mkdir($this->strap_storage_dir, 0777, TRUE);
		
		$objWriter->save($this->strap_storage_dir . 'STRAPSourceFile.xlsx');

		$this->logger->log('STRAP report generation end: ' . date('m-d-Y H:i:s'), ILogger::INFO);
		$lock->unlock();
    }
}

class ESTRAPReportServiceException extends Exception {}

