<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository of STRAP Item.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ItemRepository extends Object
{
	/** @var IItemMapper */
    private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(IItemMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'Item-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "STRAP Item"));
    }
    
	/**
	 * Saves new STRAP item.
	 * 
	 * @param Item $item
	 * @return boolean
	 */
	public function save(Item $item)
	{
		if($this->mapper->save($item))
		{
			$this->multilog->log('Strap Item with Custom ID: ' . $item->CustomID . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Updates existing STRAP item.
	 * 
	 * @param Item $item
	 * @return boolean
	 */
	public function update(Item $item)
	{
		$oldObject = $this->find($item->ID);
		if($this->mapper->update($item))
		{
			$this->multilog->log('Strap Item with ID: ' . $item->CustomID . ' updated. ' . $this->multilog->differences($oldObject, $item), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes existing STRAP Item.
	 * 
	 * @param int $item
	 * @return boolean
	 */
	public function delete($item_id)
	{
		if($this->mapper->delete($item_id))
		{
			$this->multilog->log('Strap Item with ID: ' . $item_id . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Gets STRAP Item by id.
	 * 
	 * @param int $item
	 * @return Item
	 */
	public function find($item_id)
	{
		return $this->mapper->find($item_id);
	}

	/**
	 * Gets STRAP Item by custom id.
	 * 
	 * @param string $custom_id
	 * @return Item or null if not found
	 */
	public function findByCustomID($custom_id)
	{
		return $this->mapper->findByCustomID($custom_id);
	}
	
	/**
	 * Gets all Strap Items.
	 * 
	 * @return array Array of Items where strapItem_ID is key.
	 */
	public function findAll()
	{
		return $this->mapper->findAll();
	}
	
	/**
	 * Gets all Strap Items by responsible organization unit.
	 * 
	 * @param int $tgID
	 * @return array Array of Item where strapItem_ID is key or null if not found.
	 */
	public function findAllByTechnicalGroupID($tgID)
	{
		return $this->mapper->findAllByTechnicalGroupID($tgID);
	}
	
    /**
     * Gets Item by ItemFilter. Returns every items that matches all
     * set conditions.
     *
     * @param ItemFilter $itemFilter
     * @return array of Item
    */
    public function getItemsByFilter(ItemFilter $itemFilter)
    {
		return $this->mapper->getItemsByFilter($itemFilter);
    }
	
	/**
	 * Gets unique number for new item where CustomID has passed prefix.
	 * Number expresses order of items with the same prefix.
	 * 
	 * @param string $prefix May be defined as '[department short]-[technical group short]-'
	 * @return string '1' if prefix does not exist, first free number for passed prefix otherwise.
	 */
	public function getNextCustomIDNumber($prefix)
	{
		return $this->mapper->getNextCustomIDNumber($prefix);
	}
}