<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Pursuit definition.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class Pursuit extends Object
{
	/** possible pursuit statuses */
	const PURSUIT_STATUS_PLAN = 0, // new pursuit planned
		  PURSUIT_STATUS_PROPOSED = 1, // triggers approval process
		  PURSUIT_STATUS_ACTIVE = 2, // when approved and running
		  PURSUIT_STATUS_COMMITTED = 3, // when there is a 100% chance to win a project
		  PURSUIT_STATUS_CLOSED = 4; // when pursuit has been closed
	/** full names of possible pursuit statuses */
	const FULL_NAME_STATUS_PLAN = 'Plan',
		  FULL_NAME_STATUS_PROPOSED = 'Proposed',
		  FULL_NAME_STATUS_ACTIVE = 'Active',
		  FULL_NAME_STATUS_COMMITTED = 'Committed',
		  FULL_NAME_STATUS_CLOSED = 'Closed';
	/** possible closure results */
	const PURSUIT_CLOSURE_RESULT_ON_HOLD = 0,
	      PURSUIT_CLOSURE_RESULT_WIN = 1,
	      PURSUIT_CLOSURE_RESULT_LOST = 2,
	      PURSUIT_CLOSURE_RESULT_STOPPED = 3;
	/** full names of closures results */
	const FULL_NAME_CLOSURE_RESULT_ON_HOLD = 'On Hold',
	      FULL_NAME_CLOSURE_RESULT_WIN = 'Win',
	      FULL_NAME_CLOSURE_RESULT_LOST = 'Lost',
	      FULL_NAME_CLOSURE_RESULT_STOPPED = 'Stopped';
	/** possible thresholds of probability of win */
	const PROBABILITY_OF_WIN_LOW = 35,
		  PROBABILITY_OF_WIN_MIDDLE = 65,
		  PROBABILITY_OF_WIN_HIGH = 100;
	/** full names of thresholds of probability of win */
	const FULL_NAME_PROBABILITY_OF_WIN_LOW = 'Low (0 - 34)',
	      FULL_NAME_PROBABILITY_OF_WIN_MIDDLE = 'Middle (35 - 64)',
	      FULL_NAME_PROBABILITY_OF_WIN_HIGH = 'High (65 - 100)';
	
	/** @var int ID of record */
	private $pursuitID;
	/** @var int ID of customer */
	private $customerID;
	/** @var int ID of program */
	private $programID;
	/** @var int ID of call */
	private $callID;
	/** @var float Amount covered by customer */
	private $costShare;
	/** @var string Name of pursuit */
	private $name;
	/** @var int Current status of pursuit */
	private $status;
	/** @var datetime Date of creation */
	private $createdDate;
	/** @var datetime Date of last update */
	private $lastUpdatedDate;
	/** @var string EID of person responsible for update */
	private $updatedBy;
	/** @var string Business code */
	private $sfdcNumber;
	/** @var string EID of pursuit manager */
	private $pursuitManager;
	/** @var string EID of pursuit PI */
	private $pursuitPI;
	/** @var string Remarks for pursuit */
	private $remarks;
	/** @var int Reason why pursuit may be closed. */
	private $closureResult;
	/** @var float Probability of win */
	private $probabilityOfWin;
	/** @var string URL of document storage */
	private $documentStorage;
	
	/**
     * Sets ID of pursuit.
     * @param int $id Must be positive integer.
	 * @throws InvalidArgumentException
     */
	public function setPursuitID($id)
	{
		if ($id > 0)
			$this->pursuitID = $id;
		else
			throw new InvalidArgumentException('Pursuit ID must be integer greater than 0. Wrong param: ' . $id);
	}
	
    /**
     * Gets ID of pursuit.
     * @return int
     */
	public function getPursuitID()
	{
		return $this->pursuitID;
	}
	
	/**
     * Sets ID of customer.
     * @param int $id Must be positive integer.
	 * @throws InvalidArgumentException
     */
	public function setCustomerID($id)
	{
		if ($id > 0)
			$this->customerID = $id;
		else
			throw new InvalidArgumentException('Customer ID must be integer greater than 0. Wrong param: ' . $id);
	}
	
    /**
     * Gets ID of customer.
     * @return int
     */
	public function getCustomerID()
	{
		return $this->customerID;
	}
	
	/**
     * Sets ID of program.
     * @param int $id Must be positive integer.
	 * @throws InvalidArgumentException
     */
	public function setProgramID($id)
	{
		if (($id > 0) || ($id === NULL))
			$this->programID = $id;
		else
			throw new InvalidArgumentException('Program ID must be integer greater than 0. Wrong param: ' . $id);
	}
	
    /**
     * Gets ID of program.
     * @return int
     */
	public function getProgramID()
	{
		return $this->programID;
	}
	
	/**
     * Sets ID of call.
     * @param int $id Must be positive integer.
	 * @throws InvalidArgumentException
     */
	public function setCallID($id)
	{
		if (($id > 0) || ($id === NULL))
			$this->callID = $id;
		else
			throw new InvalidArgumentException('Call ID must be integer greater than 0. Wrong param: ' . $id);
	}
	
    /**
     * Gets ID of call.
     * @return int
     */
	public function getCallID()
	{
		return $this->callID;
	}
	
	/**
     * Sets costs sharing percentage between customer and supplier.
	 * @param float $number Should be in range <0;1>.
	 * @throws InvalidArgumentException
     */
	public function setCostShare($number)
	{
		if ((is_numeric($number) && ($number >= 0) && ($number <= 1.0)) || ($number === NULL))
			$this->costShare = $number;
		else
			throw new InvalidArgumentException('Cost Share must be integer in range <0;1>. Wrong param: ' . $number);
	}
    
    /**
     * Gets cost share.
     * @return float
     */
	public function getCostShare()
	{
		return $this->costShare;
	}
    
    /**
     * Gets cost share as percentage.
     * @return int Cost share expressed as percentage or NULL.
     */
	public function getCostSharePercentage()
	{
		if (is_numeric($this->costShare) && ($this->costShare >= 0))
			return $this->costShare * 100;
		
		return NULL;
	}
	
	/**
     * Sets name of pursuit.
	 * @param string Name of pursuit.
     */
	public function setName($name)
	{
	    $this->name = $name;
	}
    
    /**
     * Gets name of pursuit.
     * @return string
     */
	public function getName()
	{
		return $this->name;
	}
	
	/**
     * Sets status of pursuit.
	 * @param int Status of pursuit.
	 * @throws InvalidArgumentException
     */
	public function setStatus($status)
	{
	    if (in_array($status, self::getStatuses()))
			$this->status = $status;
		else
			throw new InvalidArgumentException('Pursuit status must be in set of pre defined statuses. Wrong param: ' . $status);
	}
    
    /**
     * Gets status of pursuit.
     * @return string
     */
	public function getStatus()
	{
		return $this->status;
	}
	
	/**
	 * Gets Full name of status.
	 * @return string Full name of status or empty string.
	 */
	public function getStatusFullName()
	{
		$name = '';
		switch($this->status)
		{
			case self::PURSUIT_STATUS_PLAN:
				$name = self::FULL_NAME_STATUS_PLAN;
				break;
			case self::PURSUIT_STATUS_PROPOSED:
				$name = self::FULL_NAME_STATUS_PROPOSED;
				break;
			case self::PURSUIT_STATUS_ACTIVE:
				$name = self::FULL_NAME_STATUS_ACTIVE;
				break;
			case self::PURSUIT_STATUS_COMMITTED:
				$name = self::FULL_NAME_STATUS_COMMITTED;
				break;
			case self::PURSUIT_STATUS_CLOSED:
				$name = self::FULL_NAME_STATUS_CLOSED;
				break;
			default:
				$name = '';
				break;
		}
		return $name;
	}
	
	/**
	 * Gets all statuses as array.
	 * @return array Array of possible statuses.
	 */
	public static function getStatuses()
	{
		return array(
			self::PURSUIT_STATUS_PLAN,
			self::PURSUIT_STATUS_PROPOSED,
			self::PURSUIT_STATUS_ACTIVE,
			self::PURSUIT_STATUS_COMMITTED,
			self::PURSUIT_STATUS_CLOSED,
		);
	}
	
	/**
	 * Gets all statuses as array.
	 * @return array Array of full names of possible statuses indexed by status ID.
	 */
	public static function getStatusesFullNames()
	{
		return array(
			self::PURSUIT_STATUS_PLAN => self::FULL_NAME_STATUS_PLAN,
			self::PURSUIT_STATUS_ACTIVE => self::FULL_NAME_STATUS_ACTIVE,
			self::PURSUIT_STATUS_PROPOSED => self::FULL_NAME_STATUS_PROPOSED,
			self::PURSUIT_STATUS_COMMITTED => self::FULL_NAME_STATUS_COMMITTED,
			self::PURSUIT_STATUS_CLOSED => self::FULL_NAME_STATUS_CLOSED,
		);
	}
	
	/**
	 * Gets all constants describing status as array.
	 * @return array Array of possible statuses indexed by names of constants of statuses.
	 */
	public static function getStatusesConstantNames()
	{
		return array(
			'PURSUIT_STATUS_PLAN' => self::PURSUIT_STATUS_PLAN,
			'PURSUIT_STATUS_PROPOSED' => self::PURSUIT_STATUS_PROPOSED,
			'PURSUIT_STATUS_ACTIVE' => self::PURSUIT_STATUS_ACTIVE,
			'PURSUIT_STATUS_COMMITTED' => self::PURSUIT_STATUS_COMMITTED,
			'PURSUIT_STATUS_CLOSED' => self::PURSUIT_STATUS_CLOSED,
		);
	}
	
	/**
	 * Sets created date.
	 * @param string $datetime
	 */
	public function setCreatedDate($datetime)
	{
		$this->createdDate = $datetime;
	}
	
	/**
	 * Gets created date.
	 * @return string
	 */
	public function getCreatedDate()
	{
		return $this->createdDate;
	}
	
	/**
	 * Sets last update date.
	 * @param string $datetime
	 */
	public function setLastUpdatedDate($datetime)
	{
		$this->lastUpdatedDate = $datetime;
	}
	
	
	/**
	 * Gets last update date
	 * @return string
	 */
	public function getLastUpdatedDate()
	{
		return $this->lastUpdatedDate;
	}
	
	/**
	 * Sets eid of person responsible for update
	 * @param string $eid
	 */
	public function setUpdatedBy($eid)
	{
		$this->updatedBy = Person::processEID($eid);
	}
	
	/**
	 * Gets eid of person responsible for update
	 * @return string
	 */
	public function getUpdatedBy()
	{
		return $this->updatedBy;
	}
	
	/**
	 * Sets SFDC number.
	 * @param string $number
	 */
	public function setSFDCNumber($number)
	{
		$this->sfdcNumber = $number;
	}
	
	/**
	 * Gets SFDC number.
	 * @return string
	 */
	public function getSFDCNumber()
	{
		return $this->sfdcNumber;
	}
	
	/**
	 * Sets pursuit manager.
	 * @param string $eid
	 */
	public function setPursuitManager($eid)
	{
		$this->pursuitManager = Person::processEID($eid);
	}
	
	/**
	 * Gets pursuit manager.
	 * @return string
	 */
	public function getPursuitManager()
	{
		return $this->pursuitManager;
	}
	
	/**
	 * Sets pursuit PI.
	 * @param string $eid
	 */
	public function setPursuitPI($eid)
	{
		$this->pursuitPI = Person::processEID($eid);
	}
	
	/**
	 * Gets pursuit PI.
	 * @return string
	 */
	public function getPursuitPI()
	{
		return $this->pursuitPI;
	}
	
	/**
	 * Sets remarks.
	 * @param string $text
	 */
	public function setRemarks($text)
	{
		$this->remarks = $text;
	}
	
	/**
	 * Gets remarks.
	 * @return string
	 */
	public function getRemarks()
	{
		return $this->remarks;
	}
	
	/**
     * Sets closure result of pursuit.
	 * @param int $result Closure result of pursuit.
	 * @throws InvalidArgumentException
     */
	public function setClosureResult($result)
	{
	    if (in_array($result, self::getClosureResults()))
			$this->closureResult = $result;
		else
			throw new InvalidArgumentException('Pursuit closure result must be in set of pre defined results. Wrong param: ' . $result);
	}
    
    /**
     * Gets closure result of pursuit.
     * @return int
     */
	public function getClosureResult()
	{
		return $this->closureResult;
	}
	
	/**
	 * Gets Full name of closure result.
	 * @return string Full name of closure result or empty string.
	 */
	public function getClosureResultFullName()
	{
		$name = '';
		switch($this->closureResult)
		{
			case self::PURSUIT_CLOSURE_RESULT_ON_HOLD:
				$name = self::FULL_NAME_CLOSURE_RESULT_ON_HOLD;
				break;
			case self::PURSUIT_CLOSURE_RESULT_WIN:
				$name = self::FULL_NAME_CLOSURE_RESULT_WIN;
				break;
			case self::PURSUIT_CLOSURE_RESULT_LOST:
				$name = self::FULL_NAME_CLOSURE_RESULT_LOST;
				break;
			case self::PURSUIT_CLOSURE_RESULT_STOPPED:
				$name = self::FULL_NAME_CLOSURE_RESULT_STOPPED;
				break;
			default:
				$name = '';
				break;
		}
		return $name;
	}
	
	/**
	 * Gets all closure results as array.
	 * @return array Array of closure results.
	 */
	public static function getClosureResults()
	{
		return array(
			self::PURSUIT_CLOSURE_RESULT_ON_HOLD,
			self::PURSUIT_CLOSURE_RESULT_WIN,
			self::PURSUIT_CLOSURE_RESULT_LOST,
			self::PURSUIT_CLOSURE_RESULT_STOPPED,
		);
	}
	
	/**
	 * Gets all closure results as array.
	 * @return array Array of full names of closure results indexed by closure result ID.
	 */
	public static function getClosureResultsFullNames()
	{
		return array(
			self::PURSUIT_CLOSURE_RESULT_ON_HOLD => self::FULL_NAME_CLOSURE_RESULT_ON_HOLD,
			self::PURSUIT_CLOSURE_RESULT_WIN => self::FULL_NAME_CLOSURE_RESULT_WIN,
			self::PURSUIT_CLOSURE_RESULT_LOST => self::FULL_NAME_CLOSURE_RESULT_LOST,
			self::PURSUIT_CLOSURE_RESULT_STOPPED => self::FULL_NAME_CLOSURE_RESULT_STOPPED,
		);
	}
	
	/**
     * Sets probability of win.
	 * @param float $number Should be in range <0;1>.
	 * @throws InvalidArgumentException
     */
	public function setProbabilityOfWin($number)
	{
		if ((is_numeric($number) && ($number >= 0) && ($number <= 1.0)) || ($number === NULL))
			$this->probabilityOfWin = $number;
		else
			throw new InvalidArgumentException('Probability of win must be integer in range <0;1>. Wrong param: ' . $number);
	}
    
    /**
     * Gets probability of win.
     * @return float
     */
	public function getProbabilityOfWin()
	{
		return $this->probabilityOfWin;
	}
    
    /**
     * Gets probability of win as percentage.
     * @return int Probability of win expressed as percentage or NULL.
     */
	public function getProbabilityOfWinPercentage()
	{
		if (is_numeric($this->probabilityOfWin) && ($this->probabilityOfWin >= 0))
			return $this->probabilityOfWin * 100;
		
		return NULL;
	}
	
	/**
	 * Gets all thresholds of probability of win as array.
	 * @return array Array of possible thresholds.
	 */
	public static function getProbabilityOfWinThresholds()
	{
		return array(
			self::PROBABILITY_OF_WIN_LOW,
			self::PROBABILITY_OF_WIN_MIDDLE,
			self::PROBABILITY_OF_WIN_HIGH,
		);
	}
	
	/**
	 * Gets all thresholds of probability of win as array.
	 * @return array Array of full names of thresholds of probability of win indexed by threshold value.
	 */
	public static function getProbabilityOfWinThresholdsFullNames()
	{
		return array(
			self::PROBABILITY_OF_WIN_LOW => self::FULL_NAME_PROBABILITY_OF_WIN_LOW,
			self::PROBABILITY_OF_WIN_MIDDLE => self::FULL_NAME_PROBABILITY_OF_WIN_MIDDLE,
			self::PROBABILITY_OF_WIN_HIGH => self::FULL_NAME_PROBABILITY_OF_WIN_HIGH,
		);
	}
	
	/**
	 * Converts threshold of probability of win to float.
	 * @param int $threshold Number in range of Pursuit::PROBABILITY_OF_WIN_*
	 * @return float Number divided by 100.
	 * @throws InvalidArgumentException
	 */
	public static function getProbabilityOfWinThresholdFloat($threshold)
	{
		if (!in_array($threshold, self::getProbabilityOfWinThresholds()))
			    throw new InvalidArgumentException('Threshold of probability of win of Pursuit must be in set of pre defined thresholds. Wrong param: ' . $threshold);
		
		return round(($threshold / 100), 2); // e.g.: 0.64
	}
	
	/**
     * Sets URL of document storage.
	 * @param string
     */
	public function setDocumentStorage($url)
	{
	    $this->documentStorage = $url;
	}
    
    /**
     * Gets URL of document storage.
     * @return string
     */
	public function getDocumentStorage()
	{
		return $this->documentStorage;
	}
}
