<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for accessing PursuitAction from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class PursuitActionDibiMapper extends DibiMapper implements IPursuitActionMapper
{
	/**
	 * Creates PursuitAction instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	PursuitAction 
	 */
    private function load(DibiRow $row)
    {
		$pursuitAction = new PursuitAction();
		$pursuitAction->ID = $row->Action_ID;
		$pursuitAction->PursuitID = $row->Pursuit_ID;
		$pursuitAction->Action = $row->Action;
		$pursuitAction->Owner = $row->Owner;
		$pursuitAction->Deadline = $this->dateOnly($row->Deadline);
		$pursuitAction->Status = $row->Status;
		$pursuitAction->Remarks = $row->Remarks;
		return $pursuitAction;
    }
	
	/**
	 * Saves new PursuitAction.
	 * 
	 * @param PursuitAction $pursuitAction
	 * @return boolean
	 */
	public function save(PursuitAction $pursuitAction)
	{
        try
        {
            $array = array();
			$array['Pursuit_ID'] = $pursuitAction->PursuitID;
			$array['Action'] = $pursuitAction->Action;
			$array['Owner'] = $pursuitAction->Owner;
			$array['Deadline'] = $pursuitAction->Deadline;
			$array['Status'] = $pursuitAction->Status;
			$array['Remarks'] = $pursuitAction->Remarks;
            $this->conn->insert('PursuitAction', $array)->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Insert new PursuitAction with action: " . $pursuitAction->Action . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Updates existing PursuitAction.
	 * 
	 * @param PursuitAction $pursuitAction
	 * @return boolean
	 */
	public function update(PursuitAction $pursuitAction)
	{
        try
        {
            $array = array();
			$array['Action_ID'] = $pursuitAction->ID;
			$array['Pursuit_ID'] = $pursuitAction->PursuitID;
			$array['Action'] = $pursuitAction->Action;
			$array['Owner'] = $pursuitAction->Owner;
			$array['Deadline'] = $pursuitAction->Deadline;
			$array['Status'] = $pursuitAction->Status;
			$array['Remarks'] = $pursuitAction->Remarks;
            $this->conn->update('PursuitAction', $array)->where('[Action_ID] = %i', $pursuitAction->ID)->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Update of PursuitAction with ID = " . $pursuitAction->ID . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Deletes existing PursuitAction.
	 * 
	 * @param int $id PursuitAction ID 
	 * @return boolean
	 */
	public function delete($id)
	{
		try
		{
		    $result = $this->conn->delete('PursuitAction')->where('[Action_ID] = %i', $id)->execute();
			if ($result)
				Debugger::log("PursuitAction with ID = " . $id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of PursuitAction with ID = " . $id . " failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of PursuitAction with ID = " . $id . " failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Gets PursuitAction.
	 * 
	 * @param int $id PursuitAction id
	 * @return PursuitAction or null if not found
	 */
	public function find($id)
	{
		$pursuitAction_db_result = $this->conn->select('*')->from('PursuitAction')->where('[Action_ID] = %i', $id)->fetch();
		if ($pursuitAction_db_result !== FALSE)
			return $this->load($pursuitAction_db_result);
		else
			return NULL;
	}
	
	/**
	 * Gets PursuitAction by purusit ID.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return array Of PursuitAction or empty array if not found
	 */
	public function findByPursuitID($pursuitID)
	{
		$pursuitAction_db_result = $this->conn->select('*')->from('PursuitAction')->where('[Pursuit_ID] = %i', $pursuitID)->fetchAll();
		
		$pursuitAction_array = array();
		if ($pursuitAction_db_result !== FALSE)
		{
			foreach ($pursuitAction_db_result as $row)
				$pursuitAction_array[] = $this->load($row);
		}
		return $pursuitAction_array;
	}
	
	/**
	 * Gets nearest PursuitAction by purusit ID.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return PursuitPlan or null if not found
	 */
	public function findNearestByPursuitID($pursuitID)
	{
		$now = new DateTime();
		$pursuitAction_db_result = $this->conn->query('SELECT * FROM [PursuitAction]
														WHERE [Deadline] > %d
															AND [Pursuit_ID] = %i', $now, $pursuitID)->fetch();
		if ($pursuitAction_db_result !== FALSE)
			return $this->load($pursuitAction_db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds out if exists missed milestone.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return True or false
	 */
	public function existsMissedAction($pursuitID)
	{
		$now = new DateTime();
		$pursuitPlan_db_result = $this->conn->query('SELECT * FROM [PursuitAction]
														WHERE [Deadline] < %d
															AND [Pursuit_ID] = %i
																AND [Status] <> %s', $now, $pursuitID, PursuitAction::PURSUIT_ACTION_STATUS_COMPLETED)->fetch();
		if ($pursuitPlan_db_result !== FALSE)
			return true;
		else
			return false;
	}
	
	/**
	 * Gets missed PursuitAction by purusit ID.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return array Of PursuitAction or empty array if not found
	 */
	public function findMissedByPursuitID($pursuitID)
	{
		$now = new DateTime();
		$pursuitRisk_db_result = $this->conn->query('SELECT * FROM [PursuitAction]
														WHERE [Deadline] < %d
															AND [Pursuit_ID] = %i
																AND [Status] <> %s', $now, $pursuitID, PursuitAction::PURSUIT_ACTION_STATUS_COMPLETED)->fetchAll();
		
		$pursuitRisk_array = array();
		if ($pursuitRisk_db_result !== FALSE)
		{
			foreach ($pursuitRisk_db_result as $row)
				$pursuitRisk_array[] = $this->load($row);
		}
		return $pursuitRisk_array;
	}
}
