<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing Pursuit from database using dibi.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class PursuitDibiMapper extends DibiMapper implements IPursuitMapper
{	
	/**
	 * Creates Pursuit instance from DibiRow.
	 * @param DibiRow $row
	 * @return Pursuit
	 */
    private function load(DibiRow $row)
    {
		$pursuit = new Pursuit();
		$pursuit->PursuitID = $row->Pursuit_ID;
		$pursuit->CustomerID = $row->Customer_ID;
		$pursuit->ProgramID = $row->Program_ID;
		$pursuit->CallID = $row->Call_ID;
		$pursuit->CostShare = $row->CostShare;
		$pursuit->Name = $row->Name;
		$pursuit->Status = $row->Status;
		$pursuit->CreatedDate = $this->dateOnly($row->CreatedDate);
		$pursuit->LastUpdatedDate = $this->dateOnly($row->LastUpdatedDate);
		$pursuit->UpdatedBy = $row->UpdatedBy;
		$pursuit->SFDCNumber = $row->SFDCNumber;
		$pursuit->PursuitManager = $row->PursuitManager;
		$pursuit->PursuitPI = $row->PursuitPI;
		$pursuit->Remarks = $row->Remarks;
		$pursuit->ClosureResult = $row->ClosureResult;
		$pursuit->ProbabilityOfWin = $row->ProbabilityOfWin;
		$pursuit->DocumentStorage = $row->DocumentStorage;
		return $pursuit;
    }
	
	/**
	 * Saves pursuit.
	 * @param Pursuit $pursuit
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(Pursuit $pursuit)
	{
        try
        {
            $array = array();
            $array['Customer_ID'] = $pursuit->CustomerID;
            $array['Program_ID'] = $pursuit->ProgramID;
            $array['Call_ID'] = $pursuit->CallID;
            $array['CostShare'] = $pursuit->CostShare;
            $array['Name'] = $pursuit->Name;
            $array['Status'] = $pursuit->Status;
            $array['CreatedDate'] = $pursuit->CreatedDate;
            $array['LastUpdatedDate'] = $pursuit->LastUpdatedDate;
            $array['UpdatedBy'] = $pursuit->UpdatedBy;
            $array['SFDCNumber'] = $pursuit->SFDCNumber;
            $array['PursuitManager'] = $pursuit->PursuitManager;
            $array['PursuitPI'] = $pursuit->PursuitPI;
            $array['Remarks'] = $pursuit->Remarks;
			// ClosureResult is updatable only
            $array['ProbabilityOfWin'] = $pursuit->ProbabilityOfWin;
            $array['DocumentStorage'] = $pursuit->DocumentStorage;
            $db_result = $this->conn->insert('Pursuit', $array)->execute();
                        $pursuit->PursuitID = $this->conn->insertId();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving Pursuit with Name = " . $pursuit->Name . " failed. " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Updates pursuit.
	 * @param Pursuit $pursuit
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(Pursuit $pursuit)
	{
        try
        {
            $array = array();
            $array['Customer_ID'] = $pursuit->CustomerID;
            $array['Program_ID'] = $pursuit->ProgramID;
            $array['Call_ID'] = $pursuit->CallID;
            $array['CostShare'] = $pursuit->CostShare;
            $array['Name'] = $pursuit->Name;
            $array['Status'] = $pursuit->Status;
			// CreatedDate is not updatable
            $array['LastUpdatedDate'] = $pursuit->LastUpdatedDate;
            $array['UpdatedBy'] = $pursuit->UpdatedBy;
            $array['SFDCNumber'] = $pursuit->SFDCNumber;
            $array['PursuitManager'] = $pursuit->PursuitManager;
            $array['PursuitPI'] = $pursuit->PursuitPI;
            $array['Remarks'] = $pursuit->Remarks;
            $array['ClosureResult'] = $pursuit->ClosureResult;
            $array['ProbabilityOfWin'] = $pursuit->ProbabilityOfWin;
            $array['DocumentStorage'] = $pursuit->DocumentStorage;
            $db_result = $this->conn->update('Pursuit', $array)->where('[Pursuit_ID] = %i', $pursuit->PursuitID)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Pursuit update with ID = " . $pursuit->PursuitID . " and Name = " . $pursuit->Name . " failed. " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Deletes existing pursuit.
	 * @param int $id pursuit ID.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($id)
	{
		try
		{
		    $db_result = $this->conn->delete('Pursuit')->where('[Pursuit_ID] = %i', $id)->execute();
			if ($db_result)
				Debugger::log("Pursuit with ID = " . $id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of Pursuit with ID = " . $id . " failed.", Debugger::ERROR);
			return $db_result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of Pursuit with ID = " . $id . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Finds pursuit by ID.
	 * @param int $id pursuit ID.
	 * @return Pursuit or NULL if not found.
	 */
	public function find($id)
	{
		$db_result = $this->conn->select('*')->from('Pursuit')->where('[Pursuit_ID] = %i', $id)->fetch();
		if ($db_result !== FALSE)
			return $this->load($db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds all pursuits.
	 * @return array Array of Pursuit or empty array.
	 */
	public function findAll()
	{
		$db_result = $this->conn->select('*')->from('Pursuit')->orderBy('Name', 'ASC')->fetchAll();     
		$pursuit_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $pursuit)
                $pursuit_array[$pursuit->Pursuit_ID] = $this->load($pursuit);
        }		
		return $pursuit_array;
	}
	
	/**
	 * Finds pursuits according to filter.
	 * @param PursuitFilter $filter
	 * @return array Array of Pursuit or empty array.
	 */
	public function findByFilter(PursuitFilter $filter)
	{
		$db_result = $this->conn->select('*')->from('[Pursuit]' . (($filter->ProposalSubmissionAfter || $filter->ProposalSubmissionBefore) ? ', [PursuitPlan]' : NULL));
		
		if ($filter->Name)
			$db_result->where("[Name] LIKE %~like~", $filter->Name);
		if ($filter->Status)
        {
            if (!in_array('', $filter->Status)) // all statuses, so do not filter according to status
            {
                $status_condition = array();
                foreach ($filter->Status as $status)
                    array_push($status_condition, array("[Status] = %i", $status));
                $db_result->where('%or', $status_condition);
            }
        }
		if ($filter->PursuitManager)
			$db_result->where("[PursuitManager] LIKE %~like~", $filter->PursuitManager);
		if ($filter->PursuitPI)
			$db_result->where("[PursuitPI] LIKE %~like~", $filter->PursuitPI);
		if ($filter->ProposalSubmissionAfter)
			$db_result->where("([Pursuit].[Pursuit_ID] = [PursuitPlan].[Pursuit_ID] AND [PursuitPlan].[Milestone] = %s AND [PlannedDate] > %t)", PursuitPlan::MILESTONE_PROPOSAL_SUBMISSION, $filter->ProposalSubmissionAfter);
		if ($filter->ProposalSubmissionBefore)
			$db_result->where("([Pursuit].[Pursuit_ID] = [PursuitPlan].[Pursuit_ID] AND [PursuitPlan].[Milestone] = %s AND [PlannedDate] < %t)", PursuitPlan::MILESTONE_PROPOSAL_SUBMISSION, $filter->ProposalSubmissionBefore);
		if ($filter->ProbabilityOfWin)
        {
            if (!in_array('', $filter->ProbabilityOfWin)) // all probabilities, so do not filter according to probability
            {
                $probability_condition = array();
                foreach ($filter->ProbabilityOfWin as $threshold)
				{
					switch ($threshold)
					{
						case Pursuit::PROBABILITY_OF_WIN_LOW:
							array_push($probability_condition, array("[ProbabilityOfWin] < ROUND(%f, 2)", Pursuit::getProbabilityOfWinThresholdFloat(Pursuit::PROBABILITY_OF_WIN_LOW)));
							break;
						case Pursuit::PROBABILITY_OF_WIN_MIDDLE:
							array_push($probability_condition, array("[ProbabilityOfWin] >= ROUND(%f, 2) AND [ProbabilityOfWin] < ROUND(%f, 2)", Pursuit::getProbabilityOfWinThresholdFloat(Pursuit::PROBABILITY_OF_WIN_LOW), Pursuit::getProbabilityOfWinThresholdFloat(Pursuit::PROBABILITY_OF_WIN_MIDDLE)));
							break;
						case Pursuit::PROBABILITY_OF_WIN_HIGH:
							array_push($probability_condition, array("[ProbabilityOfWin] >= ROUND(%f, 2)", Pursuit::getProbabilityOfWinThresholdFloat(Pursuit::PROBABILITY_OF_WIN_MIDDLE)));
							break;
					}
				}
                $db_result->where('%or', $probability_condition);
            }
        }
		if ($filter->Customer)
			$db_result->where('[Customer_ID] = %i', $filter->Customer);
		if ($filter->ProgramID)
			$db_result->where('[Program_ID] = %i', $filter->ProgramID);
		if ($filter->CallID)
			$db_result->where('[Call_ID] = %i', $filter->CallID);
		
		$db_result->orderBy('[Name], [Status]');		
		$db_result->fetchAll();
//		$db_result->test(); die;
		
		$pursuit_array = array();
		if ($db_result !== FALSE)
        {
            foreach ($db_result as $pursuit)
                $pursuit_array[$pursuit->Pursuit_ID] = $this->load($pursuit);
        }
		return $pursuit_array;
	}
	
	/**
	 * Gets all Pursuits for given STRAP item activity.
	 * 
	 * @param int $activityID
	 * @return array Array of Pursuits where Pursuit ID is key or null if not found.
	 */
	public function findAllPursuitsByActivityID($activityID)
	{
        $pursuits_array = array();
		$pursuits_db_result = $this->conn->query('SELECT [Pursuit].* FROM [Pursuit]
													JOIN [StrapItemActivity_Pursuit] ON [StrapItemActivity_Pursuit].[Pursuit_ID] = [Pursuit].[Pursuit_ID]
														WHERE [StrapItemActivity_Pursuit].[Activity_ID] = %i', $activityID)->fetchAll();
		if ($pursuits_db_result !== FALSE)
		{
				foreach ($pursuits_db_result as $pursuit)
				{
					$pursuits_array[$pursuit->Pursuit_ID] = $this->load($pursuit);
				}
				return $pursuits_array;
		}
		else
			return null;
	}
	
	/**
	 * Gets all Pursuits for given call.
	 * 
	 * @param int $callID
	 * @return array Array of Pursuits where Pursuit ID is key or null if not found.
	 */
	public function findAllPursuitsByCallID($callID)
	{
        $pursuits_array = array();
		$pursuits_db_result = $this->conn->query('SELECT * FROM [Pursuit] WHERE [Pursuit].[Call_ID] = %i', $callID)->fetchAll();
		if ($pursuits_db_result !== FALSE)
		{
				foreach ($pursuits_db_result as $pursuit)
				{
					$pursuits_array[$pursuit->Pursuit_ID] = $this->load($pursuit);
				}
				return $pursuits_array;
		}
		else
			return null;
	}
}
