<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository of tasks.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class TaskRepository extends Object
{
    /** @var ITaskMapper */
 	private $mapper;
	/** @var MultiLogger */
	private $multilog;
	/** @var boolean */
 	private $showFakeTasks;

    public function __construct(ITaskMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'Task-'.time().'-'.rand(1, 9);
		/*
		 * Temporary fix!
		 * Operation authenticated via token doesnt have available user stettings.
		 * Therefore SBE virtual leader set as current leader.
		 */
		$eid = Environment::getUser()->getId();
		if(is_null($eid))
			$eid = 'ENN0000';//ATE SBE leader eid hard coded. Needs to find better solution.
		$this->showFakeTasks = Environment::getContext()->userSettingRepository->find($eid)->showFakeTasks;
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Task"));
    }
	
	/**
	 * Function returns array of tasks without fake tasks.
	 * 
	 * @param array $tasksArray from find methods.
	 * @return array Array of Tasks where task ID is key or null if nothing left
	 */
	private function popFakeTasks($tasksArray)
	{
		foreach ($tasksArray as $id => $task)
		{
			if($task->Fake == Task::TASK_TYPE_PLACEHOLDER)
				unset($tasksArray[$id]);
		}
		if($tasksArray)
			return $tasksArray;
		else
			return array();
	}

	/**
	 * Saves task attributes that can be updated:
	 * Remark
	 * 
	 * @param Task $task Task to update in database.
	 * @return	Task 
	 */
	public function save($task)
	{
		if($this->mapper->save($task))
		{
			$this->multilog->log('Task with ID: ' . $task->ID . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Updates existing task.
	 * 
	 * Updates only attributes that are updatable:
	 * - remark/comment
	 * 
	 * @param Task $task.
	 */
	public function update(Task $task)
	{
		$oldObject = $this->find($task->ID);
		if($this->mapper->update($task))
		{
			$this->multilog->log('Task with ID: ' . $task->ID . ' updated. ' . $this->multilog->differences($oldObject, $task), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Updates value of real cost share of each task which belongs to project of given id
	 *
	 * @param float $cost_share New value
	 * @param int $project_id
	 * @return boolean True if update was successful, false otherwise
	 */
	public function updateRealCostShare($cost_share, $project_id)
	{
		if($this->mapper->updateRealCostShare($cost_share, $project_id))
		{
			$this->multilog->log('Real cost share of Tasks with Project_ID: ' . $project_id . ' updated to: ' . $cost_share . '. ', ILogger::INFO);
			return true;
		}
		
		return false;
	}
	
	/**
	 * Deletes existing task.
	 * 
	 * @param int $task_id
	 * @return boolean
	 */
	public function delete($task_id)
	{
		if($this->mapper->delete($task_id))
		{
			$this->multilog->log('Task with ID: ' . $task_id . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}		
	}
	
	/**
	 * Finds task by id.
	 * 
	 * @param int $id task ID.
	 * @return	Task 
	 */
	public function find($id)
	{
		$task = $this->mapper->find($id);
		if($task == null)
			return null;
		if($this->showFakeTasks == false && $task->Fake == Task::TASK_TYPE_PLACEHOLDER)
			return null;
		else
			return $task;
	}
    
	/**
	 * Finds task by SAP id.
	 * 
	 * @param string $id task SAP ID.
	 * @return Task 
	 */
	public function findBySAPID($sap_id)
	{
		$task = $this->mapper->findBySAPID($sap_id);
		if($task == null)
			return null;
		if($this->showFakeTasks == false && $task->Fake == Task::TASK_TYPE_PLACEHOLDER)
			return null;
		else
			return $task;
	}

	/**
	 * Returns all task that are part of a project ordered by name.
	 * 
	 * @param int $project_id ID of a project.
	 * @param int $year year.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findAllProjectTasks($project_id)
	{
		if($this->showFakeTasks == true)
			return $this->mapper->findAllProjectTasks($project_id);
		else
			return $this->popFakeTasks($this->mapper->findAllProjectTasks($project_id));
	}
	
	/**
	 * Returns all task that are part of a workpackage ordered by name.
	 * 
	 * @param int $workpackage_ID ID of a project.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findAllWorkPackageTasks($workpackage_ID)
	{
		if($this->showFakeTasks == true)
			return $this->mapper->findAllWorkPackageTasks($workpackage_ID);
		else
			return $this->popFakeTasks($this->mapper->findAllWorkPackageTasks($workpackage_ID));
	}
	
	/**
	 * Returns all task that are part of a project and are active
	 * in selected year which means task that have some persons
	 * assigned ordered by name.
	 * 
	 * @param int $project_id ID of a project.
	 * @param int $year year.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findAssignedProjectTasksInYear($project_id, $year)
	{
		if($this->showFakeTasks == true)
			return $this->mapper->findAssignedProjectTasksInYear($project_id, $year);
		else
			return $this->popFakeTasks($this->mapper->findAssignedProjectTasksInYear($project_id, $year));		
	}

	/**
	 * Returns all tasks that are part of a project and have
	 * plan for selected year ordered by name.
	 * 
	 * @param int $project_id ID of a project.
	 * @param int $year year.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findPlannedProjectTasksInYear($project_id, $year)
	{
		if($this->showFakeTasks == true)
			return $this->mapper->findPlannedProjectTasksInYear($project_id, $year);
		else
			return $this->popFakeTasks($this->mapper->findPlannedProjectTasksInYear($project_id, $year));	
	}
    
    /**
	 * Returns all tasks that are part of a workpackage and have
	 * plan for selected year.
	 * 
	 * @param int $wp_id ID of a workpackage.
	 * @param int $year year.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findPlannedWorkpackageTasksInYear($wp_id, $year)
    {
        if($this->showFakeTasks == true)
			return $this->mapper->findPlannedWorkpackageTasksInYear($wp_id, $year);
		else
			return $this->popFakeTasks($this->mapper->findPlannedWorkpackageTasksInYear($wp_id, $year));
    }

	/**
	 * Returns all tasks that are active in a project in given year even if they
	 * are not planned for the current year. 
	 * 
	 * Returns only tasks that exist in the system . Deleted are not returned
	 * even if there is some activity (e.g. invoicing or work).
	 * 
	 * @param int $project_id ID of a project.
	 * @param int $year year.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findActiveProjectTasksInYear($project_id, $year)
	{
		if($this->showFakeTasks == true)
			return $this->mapper->findActiveProjectTasksInYear($project_id, $year);
		else
			return $this->popFakeTasks($this->mapper->findActiveProjectTasksInYear($project_id, $year));	
	}
    
	/**
	 * Returns all Tasks' IDs and theirs SAP IDs.
	 * 
	 * @param bool $removeSpaces Remove all spaces from key strings.
	 * @return array Task_ID => SAP_ID
	 */
	public function findAllProjectTasksSAPIDs($removeSpaces = FALSE)
	{
			return $this->mapper->findAllProjectTasksSAPIDs($removeSpaces);
	}
	
    /**
     * Returns distinct task SAP IDs.
     * 
     * @return array Array of all task SAP IDs 
     */
    public function findDistinctSAPIDs()
    {
        return $this->mapper->findDistinctSAPIDs();
    }
	
    /**
     * Returns distinct labour task SAP IDs.
     * 
     * @return array Array of labour task SAP IDs 
     */
    public function findDistinctLaborTaskSAPIDs()
    {
        return $this->mapper->findDistinctLaborTaskSAPIDs();
    }
	
    /**
     * Returns distinct labour task activity.
     * 
     * @return array Array of labour task activities
     */
    public function findDistinctLaborTaskActivity()
    {
		return $this->mapper->findDistinctLaborTaskActivity();
    }
    
    /**
	 * Returns all task that are part of a project ordered by name.
	 * 
	 * @param int $project_id ID of a project.
	 * @param int $year year.
	 * @return	array associative array of Task where key is task id.
	 */
    public function findAllProjectTasksInYear($project_id, $year)
    {
		if($this->showFakeTasks == true)
			return $this->mapper->findAllProjectTasksInYear($project_id, $year);
		else
			return $this->popFakeTasks($this->mapper->findAllProjectTasksInYear($project_id, $year));	
    }
	
	/**
	 * Returns all task that are part of a project ordered by name.
	 * 
	 * @param int $project_id ID of a project.
	 * @param int $year year.
	 * @return	array associative array of Task where key is task id.
	 */
	public function findAllProjectTasksInYearAndFuture($project_id, $year)
	{
		if($this->showFakeTasks == true)
			return $this->mapper->findAllProjectTasksInYearAndFuture($project_id, $year);
		else
			return $this->popFakeTasks($this->mapper->findAllProjectTasksInYearAndFuture($project_id, $year));
	}
    
    /**
     * Gets all tasks of person in given project and year.
     * @param string $person_id person ID.
	 * @param int $project_id
     * @param int $year year.
     * @return array of tasks. Return array if find succesfull, otherwise returns null.
     */
    public function findPersonTasksInYear($person_id, $project_id, $year)
    {
		if($this->showFakeTasks == true)
			return $this->mapper->findPersonTasksInYear($person_id, $project_id, $year);
		else
			return $this->popFakeTasks($this->mapper->findPersonTasksInYear($person_id, $project_id, $year));
    }
	
    /**
     * Gets all tasks where is person assigned.
     * @param string $person_id person ID.
     * @return array of tasks. Return array if find succesfull, otherwise returns null.
     */
    public function findAllTasksWhereIsPersonAssigned($person_id)
    {
		if($this->showFakeTasks == true)
			return $this->mapper->findAllTasksWhereIsPersonAssigned($person_id);
		else
			return $this->popFakeTasks($this->mapper->findAllTasksWhereIsPersonAssigned($person_id));
    }
	
    /**
     * Gets tasks where person charged some hours in given project and year.
     * @param string $person_id person ID.
	 * @param int $project_id
     * @param int $year year.
     * @return array of tasks. Return array if find succesfull, otherwise returns null.
     */
    public function findPersonActiveTasksInYear($person_id, $project_id, $year)
    {
		if($this->showFakeTasks == true)
			return $this->mapper->findPersonActiveTasksInYear($person_id, $project_id, $year);
		else
			return $this->popFakeTasks($this->mapper->findPersonActiveTasksInYear($person_id, $project_id, $year));	
    }
	
	/**
	 * Gets next task id.
	 * 
	 * @return int Task ID;
	 */
	public function getNextTaskID()
	{
		return $this->mapper->getNextTaskID();
	}
	
	
	/**
     * Gets all placeholder tasks for iASAP import.
	 * @param string $importFrom iASAP or E-Charge(Task constants).
     * @return array of tasks. Return array if find succesfull, otherwise returns empty array.
     */
    public function findAllPlaceholderTasksForImport($importFrom)
    {
		return $this->mapper->findAllPlaceholderTasksForImport($importFrom);
    }
	
	/**
	 * Returns all labour tasks with task plan by given year.
	 * 
	 * @param int $year Year.
	 * @return	array Returns associative array of tasks (key is TaskID).
	 */
	public function findLabourTaskWithTaskPlanForYear($year)
	{
		return $this->mapper->findLabourTaskWithTaskPlanForYear($year);
	}
	
    /**
     * Gets all years of task.
     * 
     * @param int $taskID
     * @return array Array of available years
     */
    public function findTaskYears($taskID)
    {
		return $this->mapper->findTaskYears($taskID);
    }
}