<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and licence information, please view
 * the file licence.txt that was distributed with this source code.
 * @package models
 */

/**
 * Utility for work with EC access classes
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ECUtility extends Utility
{
	/** @var array Cash for ECLicenceName prefixes */
	private $cachedECLicenceNamePrefixes = NULL;
	/** @var array Cash for all possible licences */
	private $cachedECLicences = NULL;
	/** @var array Cash for max value of licence */
	private $cachedMaxECLicenceValue = NULL;
	/** @var ECLicence Cash for default licence */
	private $cachedDefaultLicence = NULL;
	
	public function __construct()
	{
		parent::__construct();
	}
	
	/**
	 * Assigns ECLicenceName to Project.
	 * @param string $name Should be name of ECLicenceName.
	 * @param int $project_id Should be project id.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function assignLicence($name, $project_id)
	{
		try
		{
            $project = $this->context->projectRepository->find($project_id);
			if (!$project)
				return FALSE;
            
            $name = trim($name);
            // Is it licence name and not note? Then save it
            // Be careful with loading of default licence before comparison
            if (($name != '') && ($this->guessLicenceFromLicenceName($name) != $this->cachedDefaultLicence->ECLicenceID))
            {            
                if (!$this->context->ecLicenceNameRepository->add($name))
                    return FALSE;

                $licence_name = $this->context->ecLicenceNameRepository->findByName($name);
                if (!$licence_name)
                    return FALSE;
                
                if ($this->context->projectECLicenceNameRepository->find($project_id, $licence_name->ECLicenceNameID) === NULL)
                {
                    $project_licence = new ProjectECLicenceName();
                    $project_licence->ProjectID = $project_id;
                    $project_licence->ECLicenceNameID = $licence_name->ECLicenceNameID;
                    $this->context->projectECLicenceNameRepository->save($project_licence);
                }
            }
            
            $licence_id = $this->guessProjectLicence($project_id);
			
			if ($project->ECLicenceID != $licence_id)
			{
			    $project->ECLicenceID = $licence_id;
				// There is need to add licence to every assigned worker
				// TODO: rewrite it according assignment from month not from whole year.
				$workers = $this->context->personRepository->findAllPersonsAssignedToProjectInYearToFuture($project_id, Date('Y'));
				if($workers)
				{
					foreach($workers as $worker)
					{
						if(is_null($this->context->personECLicenceRepository->find($worker->ID, $project->ECLicenceID)))
						{
							$projectTasks = $this->context->taskRepository->findPersonTasksInYear($worker->ID, $project->ID, Date('Y'));
							foreach($projectTasks as $task)
							{
								$assignment = 0;
								$assigns = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($task->ID, Date('Y'), $worker->ID);
								foreach($assigns as $assign)
								{
									if($assign->Month >= Date('m'))
										$assignment += $assign->Hours;
								}
								if($assignment > 0)
								{
									$personLicence = new PersonECLicence();
									$personLicence->EID = $worker->ID;
									$personLicence->ECLicenceID = $project->ECLicenceID;
									$this->context->personECLicenceRepository->add($personLicence);
								}
							}
						}
					}
				}
			    return $this->context->projectRepository->update($project);
			}
		}
		catch (DibiException $e)
		{
			Debugger::log("Cannot assign licence to project. " . $e->getMessage(), Debugger::ERROR);
            return FALSE;
		}
		
		return TRUE;
	}
    
    /**
     * Guesses licence of project with highest priority from all assigned licences of project
     * @param int $project_id
     * @return int ECLicence id
     */
    public function guessProjectLicence($project_id)
    {
        $licence_id = $this->guessLicenceFromLicenceName('');
                
        // Let's cache possible licences
        if ($this->cachedECLicences === NULL)
            $this->cachedECLicences = $this->context->ecLicenceRepository->findAll();
        
        // Let's cache max value of licence
        if ($this->cachedMaxECLicenceValue === NULL)
            $this->cachedMaxECLicenceValue = $this->context->ecLicenceRepository->findMaxValue();
        
        
        $min_licence_value = $this->cachedMaxECLicenceValue;
        $all_project_licences = $this->context->projectECLicenceNameRepository->findByProjectID($project_id);
        
        // Find licence with highest priority = Find minimal value
        foreach ($all_project_licences as $project_licence)
        {
            $licence_name = $this->context->ecLicenceNameRepository->find($project_licence->ECLicenceNameID);
            // Skip unreadable name
            if (!$licence_name)
                continue;
            
            $temp_licence_id = $this->guessLicenceFromLicenceName($licence_name->Name);
            if (isset($this->cachedECLicences[$temp_licence_id]))
            {
                $licence_value = $this->cachedECLicences[$temp_licence_id]->Value;

                // Current licence has higher priority
                if ($min_licence_value > $licence_value)
                {
                    $licence_id = $this->cachedECLicences[$temp_licence_id]->ECLicenceID;
                    $min_licence_value = $licence_value;
                }
            }
        }
        
        return $licence_id;
    }
	
	/**
	 * Guesses licence of project by name from ECLicenceName and returns ECLicence ID.
	 * 
	 * ECLicence ID is decided by prefix in name from ECLicenceName.
	 * Testing is case insensitive (so prefix "MIL" and "mil" matches the same).
     * In case no licence was found, id of default one is returned.
	 * 
	 * @param string $licence_name Name of ECLicenceName.
	 * @return int ECLicence id or NULL if error occured.
	 */
	public function guessLicenceFromLicenceName($licence_name)
	{        
		// Let's cache default licence
        if ($this->cachedDefaultLicence === NULL)
        {
            $this->cachedDefaultLicence = $this->context->ecLicenceRepository->findByECLicenceNamePrefix(ECLicence::PREFIX_NON_LICENCE_REQUIRED);
            if (!$this->cachedDefaultLicence)
                return NULL;
        }
        
		// Let's cache ECLicenceName prefixes
		if ($this->cachedECLicenceNamePrefixes === NULL)
		{
            // Let's cache possible licences
            if ($this->cachedECLicences === NULL)
                $this->cachedECLicences = $this->context->ecLicenceRepository->findAll();
            
			$this->cachedECLicenceNamePrefixes = array();
			foreach ($this->cachedECLicences as $licence)
			    if (trim($licence->ECLicenceNamePrefix) != '') // Throw out licences without prefix
					$this->cachedECLicenceNamePrefixes[$licence->ECLicenceID] = $licence->ECLicenceNamePrefix;
			// Need to sort them by prefix length descending	
			uasort($this->cachedECLicenceNamePrefixes, 'strcmp');
			$this->cachedECLicenceNamePrefixes = array_reverse($this->cachedECLicenceNamePrefixes, TRUE);
		}
        
        $licence_name = trim($licence_name);
		if ($licence_name != '')
        {
            // Find licence by prefix
            foreach ($this->cachedECLicenceNamePrefixes as $licence_id => $prefix)
            {
                if (strpos(strtolower($licence_name), strtolower($prefix)) === 0) // prefix is divided from the rest of task name by a space
                    return $licence_id;
            }
        }
        
		return $this->cachedDefaultLicence->ECLicenceID;
	}
	
	/**
	 * Checks access rights of given nationality on object licensed by given licence.
	 * @param int $nationality_id Should be Nationality id.
	 * @param int $licence_id Should be ECLicence id.
	 * @return boolean TRUE if given nationality has access to object licensed by given licence, FALSE otherwise.
	 */
	public function hasAccess($nationality_id, $licence_id)
	{
		$ec_licence = $this->context->ecLicenceRepository->find($licence_id);
		$nationality = $this->context->nationalityRepository->find($nationality_id);
		
		if (!$ec_licence || !$nationality)
		    return FALSE;
		
		if ($ec_licence->Value !== 0)
		{
			if ($ec_licence->Value < $nationality->Value)
				return FALSE;
		}
		// TODO else branch
		// Place where to check relation Person_ECLicence
		
		return TRUE;
	}
	
	/**
	 * Finds all ECLicenceName by project id
	 * @param int $project_id Project id.
	 * @return array Array of ECLicenceName or empty array.
	 */
	public function findECLicenceNameByProjectID($project_id)
	{
		$all_project_licences = $this->context->projectECLicenceNameRepository->findByProjectID($project_id);
		$all_licence_names = array();
		foreach ($all_project_licences as $project_licence)
		{
			$licence_name = $this->context->ecLicenceNameRepository->find($project_licence->ECLicenceNameID);
			if ($licence_name)
			    $all_licence_names[$licence_name->ECLicenceNameID] = $licence_name;
		}
		
		return $all_licence_names;
	}
}
