<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository for accessing work packages.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class WorkPackageRepository
{
	
    /** @var IWorkPackageMapper */
 	private $mapper;
    /** @var ITaskMapper */
 	private $task_mapper;
	/** @var MultiLogger */
	private $multilog;

    public function __construct(IWorkPackageMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'WorkPackage-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Work Package"));
    }	

	/**
	 * Sets task mapper (method suporting DI).
	 * 
	 * @param ITaskMapper $mapper
	 */
	public function setTaskMapper(ITaskMapper $mapper)
	{
		return $this->task_mapper = $mapper;
	}
	
	/**
	 * Saves new WorkPackage.
	 * 
	 * @param WorkPackage $wp New Workpackage to add.
	 * @return	boolean 
	 */
	public function save(WorkPackage $wp)
	{
		if($this->mapper->save($wp))
		{
			$this->multilog->log('Work Package with ID: ' . $wp->ID . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Updates existing WorkPackage.
	 * 
	 * @param WorkPackage $wp Existing WorkPackage.
	 * @return	boolean 
	 */
	public function update(WorkPackage $wp)
	{
		$oldObject = $this->find($wp->ID);
		if($this->mapper->update($wp))
		{
			$this->multilog->log('Work Package with ID: ' . $wp->ID . ' updated. ' . $this->multilog->differences($oldObject, $wp), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Finds work package by id.
	 * 
	 * @param int $id work package ID.
	 * @return	WorkPackage 
	 */
	public function find($id)
	{
		return $this->mapper->find($id);
	}
	
	/**
	 * Finds work package by work package parent id.
	 * 
	 * @param int $workpackageParent_id work package parent ID.
	 * @return	WorkPackage or empty array if not found.
	 */
	public function findByParentID($workpackageParent_id) 
	{
		return $this->mapper->findByParentID($workpackageParent_id);
	}
	
	/**
	 * Finds work package by SAP id.
	 * 
	 * @param string $id workpackage SAP ID.
	 * @return	WorkPackage 
	 */
	public function findBySAPID($id)
	{
		return $this->mapper->findBySAPID($id);
	}
	
	/**
	 * Returns iterator over project's work package structure.
	 * 
	 * WBS structure is a tree of WBSTreeElementItem with WorkPackages 
	 * and Tasks as data.
	 * 
	 * @param int $project_id Project ID.
	 * @return	WBSTree 
	 * @todo Create defensively and catch exceptions throwable by WBSTree::addItem method
	 */
	public function getProjectWBSTree($project_id)
	{
		$work_package_list = $this->mapper->getProjectWorkPackages($project_id);
		$tree = new WBSTree();
		$wp_uuid_map = array();
		/* work packages are expected to be sorted be ID meaning creation
			so it shouldn't happen that sub-workpackage is
			added before its parent work package */
		foreach ($work_package_list as $work_package)
		{
			$item = new WBSTreeElementItem($work_package);
			if (($work_package->getParentID() != 0) && (!isset($wp_uuid_map[$work_package->getParentID()])))
				Debugger::log ('Workpackage\'s (ID='.$work_package->getID().') parent work package (PARENT WP ID='.$work_package->getParentID().') not found in the WP list upon WBS tree creation.', Debugger::ERROR);
			else
			{
				// only first item should have parent id NULL
				$parent_item_id = ($work_package->getParentID() != 0) ? $wp_uuid_map[$work_package->getParentID()] : NULL; 
				$tree->addItem($item, $parent_item_id);
				$wp_uuid_map[$work_package->getID()] = $item->getUUID();
			}
		}                                        
        
        $task_list = Environment::getContext()->taskRepository->findAllProjectTasks($project_id);
		foreach ($task_list as $task)
		{
			$item = new WBSTreeElementItem($task);
			if (!isset($wp_uuid_map[$task->getWorkPackageID()]))
				Debugger::log ('Task\'s (ID='.$task->getID().') work package (WP ID='.$task->getWorkPackageID().') not found in the WP list upon WBS tree creation.', Debugger::ERROR);
			else
				$tree->addItem($item, $wp_uuid_map[$task->getWorkPackageID()]);
		}
		return $tree;
	}	
        
    /**
	 * Returns iterator over project's work package structure.
	 * 
	 * WBS structure is a tree of WBSTreeElementItem with WorkPackages 
	 * and Tasks as data.
	 * 
	 * @param int $project_id Project ID.
	 * @return	WBSTree 
	 * @todo Create defensively and catch exceptions throwable by WBSTree::addItem method
	 */
	public function getProjectWBSTreeInYear($project_id, $year)
	{
		$work_package_list = $this->mapper->getProjectWorkPackages($project_id);
		$tree = new WBSTree();
		$wp_uuid_map = array();
		/* work packages are expected to be sorted be ID meaning creation
			so it shouldn't happen that sub-workpackage is
			added before its parent work package */
		foreach ($work_package_list as $work_package)
		{
			$item = new WBSTreeElementItem($work_package);
			if (($work_package->getParentID() != 0) && (!isset($wp_uuid_map[$work_package->getParentID()])))
				Debugger::log ('Workpackage\'s (ID='.$work_package->getID().') parent work package (PARENT WP ID='.$work_package->getParentID().') not found in the WP list upon WBS tree creation.', Debugger::ERROR);
			else
			{
				// only first item should have parent id NULL
				$parent_item_id = ($work_package->getParentID() != 0) ? $wp_uuid_map[$work_package->getParentID()] : NULL; 
				$tree->addItem($item, $parent_item_id);
				$wp_uuid_map[$work_package->getID()] = $item->getUUID();
			}
		}                            
        $task_list = $this->task_mapper->findAllProjectTasksInYear($project_id, $year);
		foreach ($task_list as $task)
		{
			$item = new WBSTreeElementItem($task);
			if (!isset($wp_uuid_map[$task->getWorkPackageID()]))
				Debugger::log ('Task\'s (ID='.$task->getID().') work package (WP ID='.$task->getWorkPackageID().') not found in the WP list upon WBS tree creation.', Debugger::ERROR);
			else
				$tree->addItem($item, $wp_uuid_map[$task->getWorkPackageID()]);
		}
		return $tree;
	}
    
	/**
	 * Returns array of WorkPackages on the path
	 * in the WorkPackage tree from root to the task
	 * specified by $task_id.
	 * 
	 * @param int $task_id Task ID.
	 * @return	array of WorkPackage 
	 * @throws InvalidArgumentException if task ID is invalid.
	 */
	public function getWPTreePathByTaskID($task_id)
	{
		return $this->mapper->getWPTreePathByTaskID($task_id);
	}	
	
	/** 
	 * Returns array of all WorkPackages that are in Project WBS. 
	 *  
	 * @param int $project_id Project ID. 
	 * @return      array of WorkPackage  
	 */ 
	public function getProjectWorkPackages($project_id) 
	{ 
		return $this->mapper->getProjectWorkPackages($project_id); 
	}
    
    /**
     * Gets project id
     * @param int $wp_id Should be workpackage id
     * @return int Return project id or NULL
     */
    public function getProjectID($wp_id)
    {
        return $this->mapper->getProjectID($wp_id);
    }
	
    /**
     * Gets all years of work package.
     * 
     * @param int $workpackage_ID
     * @return array Array of available years
     */
    public function findWorkPackageYears($workpackage_ID)
    {
        return $this->mapper->findWorkPackageYears($workpackage_ID);
    }
	
	/**
	 * Deletes existing WorkPackage.
	 * 
	 * @param int $workpackage_ID
	 * @return boolean
	 */
	public function delete($workpackage_ID)
	{
		if($this->mapper->delete($workpackage_ID))
		{
			$this->multilog->log('Work Package with ID: ' . $workpackage_ID . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}	
	}
}