<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package utils/Files
 */

/**
 * Class representing writer to delimiter-separated-file format.
 * It is a text file with lines of values delimited by delimiter.
 * 
 * Typical DSV file is CSV (comma separated values file), which
 * is also default setup.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class DSVFileWriter
{
	/** UNIX-like end of line character */
	const UNIX_EOL = "\n";
	/** Windows-like end of line characters */
	const WIN_EOL = "\r\n";
	
	/** var string Name and path of the file */
	private $filename = '';

	/** var boolean Whether to overwrite target file. */
	private $overwrite = FALSE;
	
	/** var resource Handle to the file after it is open. */
	private $handle = NULL;

	/** var resource Character for delimiting values. */
	private $delimiter = '';

	/** var resource Character to quote values (encapsulation of string value) */
	private $quote = '';

	/** var string End of line characters */
	private $eol = self::WIN_EOL;

	/** var int Count of added lines */
	private $addedLines = 0;
	
	/**
	 * Creates new DSV file.
	 * 
	 * Default is CSV file output: ',' as delimiter and '"' as quote (de facto standard).
	 * 
	 * @param	$filename	string	Name of the output file.
	 * @param	$overwrite	boolean	If TRUE and file exists, it will be overwritten. Otherwise it will be appended.
	 * @param	$delimiter	string Delimiter character.
	 * @param	$quote		string Quote character.
	 * @param	$eol		string Ends of line format. Default: WIN_EOL.
	 * @throws	InvalidArgumetException If delimiter of quote is invalid.
	 */
	public function __construct($filename, $overwrite = FALSE, $delimiter = ',', $quote = '"', $eol = self::WIN_EOL)
	{
		$this->filename = $filename;
		$this->overwrite = ($overwrite) ? TRUE : FALSE;
		$this->eol = ($eol == self::UNIX_EOL) ? self::UNIX_EOL : self::WIN_EOL;
		$this->setDelimiter($delimiter);
		$this->setQuote($quote);
	}
	
	/**
	 * Opens DSV file.
	 * 
	 * @throws IOException If opening of file fails.
	 */
	public function open()
	{
		if (!$this->isOpened())
		{
			if (file_exists($this->filename) && $this->overwrite)
				$this->handle = fopen($this->filename, "a");
			else
				$this->handle = fopen($this->filename, "w");
			if ($this->handle === FALSE)
				throw new IOException('Failed to open DSV file '.$this->filename.' for writing.');
		}
	}

    /**
	 * Returns TRUE if file is opened, otherwise false.
	 * 
     * @return boolean
	 */
	public function isOpened()
	{
		return ($this->handle === NULL) ? FALSE : TRUE;
	}

    /**
	 * Writes passed arguments into the output DSV file.
	 * 
	 * Only scalar values and arrays of scalars are written as single
	 * delimited values.
	 * 
	 * Example of usage:
	 * <pre>
	 * $writer->write('First', 'Second', 3, array(4, '5', 'Six'), 'Seven')->writeln('Eight');
	 * </pre>
	 * 
     * @param mixed $values...	variable count of arguments to write
	 * @throws IOException when file is not opened.
	 * @return DSVFileWriter Returns itself for fluent interface.
	 */
	public function write()
	{
		if ($this->isOpened())
		{
			for ($i = 0; $i < func_num_args(); $i++)
			{
				$arg = func_get_arg($i);
				if (is_scalar($arg))
				{
					fwrite($this->handle, $this->quote . $arg . $this->quote . $this->delimiter);
				}
				else
					if (is_array($arg))
					{
						foreach ($arg as $value)
						{
							if (is_scalar($value))
								fwrite($this->handle, $this->quote . $value . $this->quote . $this->delimiter);
							else
								throw new InvalidArgumentException('Passed array on position '.$i.' has invalid non-scalar value on key ' . key($arg). ' :' . $value);
						}
					}
					else
						throw new InvalidArgumentException('Passed argument on position '.$i.' is not scalar nor array of scalars : ' . $arg);
			}
			return $this;
		}
			
		else
			throw new IOException('File is not opened.');
	}


    /**
	 * Writes all passed parameters into the output DSV file and ends the line.
	 * 
	 * Only scalar values and arrays of scalars are written as single
	 * delimited values.
	 * If no argument is passed, only end of line is written.
	 * 
	 * Example of usage:
	 * <pre>
	 * $writer->write('First', 'Second', 3, array(4, '5', 'Six'), 'Seven')->writeln('Eight');
	 * </pre>
	 * 
     * @param mixed $values...	variable count of arguments to write
	 * @throws IOException when file is not opened.
	 * @return DSVFileWriter Returns itself for fluent interface.
	 */
	public function writeln()
	{
		if ($this->isOpened())
		{
			for ($i = 0; $i < func_num_args(); $i++)
			{
				$arg = func_get_arg($i);
				if (is_scalar($arg))
				{
					fwrite($this->handle, $this->quote . $arg . $this->quote . $this->delimiter);
				}
				else
					if (is_array($arg))
					{
						foreach ($arg as $value)
						{
							if (is_scalar($value))
								fwrite($this->handle, $this->quote . $value . $this->quote . $this->delimiter);
							else
								throw new InvalidArgumentException('Passed array on position '.$i.' has invalid non-scalar value on key ' . key($arg). ' :' . $value);
						}
					}
					else
						throw new InvalidArgumentException('Passed argument on position '.$i.' is not scalar nor array of scalars : ' . $arg);
			}
			fwrite($this->handle, $this->eol);
			$this->addedLines++;
			return $this;
		}
		else
			throw new IOException('File is not opened.');
	}
	
    /**
	 * Closes output file.
	 * 
     * @param array $values
	 * @throws IOException when closing fails.
	 */
	public function close()
	{
		if ($this->handle !== NULL)
		{
			if (fclose($this->handle) == FALSE)
				throw new IOException('Closing of file failed.');
			else
				$this->handle = NULL;
		}
	}

    /**
     * Character for delimiting values.
	 * 
	 * It should be one character so only first character of string is used.
	 * 
     * @param string $delimiter Character that will be used as value delimiter.
	 */
	public function setDelimiter($delimiter)
	{
		if (is_string($delimiter)) 
			$this->delimiter = substr($delimiter, 0, 1);
		else
			throw new InvalidArgumentException('Delimiter must be string.');
	}

	
    /**
     * Character for quoting values. 
	 * 
	 * It should be one character so only first character of string is used.
	 * 
     * @param string $quote Character that will be used as value quote.
	 */
	public function setQuote($quote)
	{
		if (is_string($quote))
			$this->quote = substr($quote, 0, 1);
		else
			throw new InvalidArgumentException('Quote must be string.');
	}

    /**
     * Returns count of added lines.
	 * 
     * @return int 
	 */
	public function getAddedLines()
	{
		return $this->addedLines;
	}
	
}

