<?php

class TrackingTableWrapper
{
    /** @var DIContainer DI context with application repositories access */
    private $context;

    /** @var PHPExcel PHP Excel library */
    private $objPHPExcel;
    
    /** @var integer Year */
    private $year;
    
    /** @var string/path Path to file template */
    private $fileName  = 'data/templates/excel/tracking-table.xlsx';
    
    /** @var string Sheet name with raw data */
    private $rawDataSheetName = 'RawData'; // use alphanumeric chars only (VLOOKUP PHPExcel function)
    
    /** @var string Sheet name with cost centers */
    private $costCentersSheetName = 'CostCenters'; // use alphanumeric chars only (VLOOKUP PHPExcel function)
    
    /** @var IFileLogger Logger */
    private $logger;
    
    /**
     * Creates wrapper for creating Tracking Table
     * 
     * @param IFileLogger $logger Logger
     */
    public function __construct(DIContainer $context, IFileLogger $logger = NULL)
    {
        
        $this->context = $context;
        $this->logger = ($logger === NULL) ? (new NullLogger('')) : $logger;
        $this->year = date('Y'); // if changed to something else than current year, mind 'Actuals until' column in fillTemplate() method
        
    }
    
    public function loadTemplate($fileName = NULL)
    {
        
        if ($fileName === NULL)
            $fileName = $this->fileName;
        
        $this->objPHPExcel = PHPExcel_IOFactory::load($fileName);
        
    }
    
    public function fillTemplate($rawDataSheetName = NULL, $costCentersSheetName = NULL)
    {
        
        // init sheets
        if ($rawDataSheetName === NULL)
            $rawDataSheetName = $this->rawDataSheetName;
        
        if ($costCentersSheetName === NULL)
            $costCentersSheetName = $this->costCentersSheetName;

        $rawDataList = $this->objPHPExcel->getSheetByName($rawDataSheetName);
        if (!$rawDataList)
        {
            throw new TrackingTableWrapperException("Sheet '$rawDataSheetName' not found");
        }
        
        $costCentersList = $this->objPHPExcel->getSheetByName($costCentersSheetName);
        if (!$costCentersList)
        {
            throw new TrackingTableWrapperException("Sheet '$costCentersSheetName' not found");
        }
        
        // cost centers sheet
        $costCenters = $this->context->costCenterRepository->findAllCostCenters();
        $rowIterator = $costCentersList->getRowIterator();
        $rowIterator->next(); //skip first row (heading)
        foreach ($costCenters as $costCenterId => $costCenter)
        {
            $cellIterator = $rowIterator->current()->getCellIterator();
            $costCenterLabourRate = $costCenter->getYearLabourRate($this->year);
            
            $cellIterator->current()->setValue($costCenterId);
            $cellIterator->next();
            
            $cellIterator->current()->setValue($costCenterLabourRate ? $costCenterLabourRate->getYear() : $this->year);
            $cellIterator->next();
            
            for ($month = 1; $month <= 12; $month++)
            {
                $cellIterator->current()->setValue($costCenterLabourRate ? $costCenterLabourRate->getMonthlyHourRate($month) : 0);
                $cellIterator->next();
            }
            
            $rowIterator->next();
        }
        
        /**
         * Fill Raw Data sheet
         * 
         * FOR EACH Project                         Record Type
         *   FOR EACH Task
         *     IF TaskYearPlan (2012)               P[L|N]
         *       IF labor Task                      
         *         THEN
         *           assigned Persons               AL
         *           unassigned Persons             AL
         *         ELSE
         *           FOR EACH ActualsCost           AN
         *           ???                            EN
         */
        $rowIterator = $rawDataList->getRowIterator();
        $rowIterator->next(); // skip first row (heading)
//        $project = $this->context->projectRepository->findProject($projectID);
        foreach ($this->context->projectRepository->findAllProjects() as $project)
        {
            $tasks = $this->context->taskRepository->findAllProjectTasks($project->ID);
            $customers = $this->context->customerRepository->findAll();
            $persons = $this->context->personRepository->findAll();
            $fiscalMonth = FiscalCalendar::getFiscalMonthByTimestamp(time())->index;
            $actualsUntil = date("F", mktime(0, 0, 0, ($fiscalMonth)));
            $responsibilities = $this->context->responsibilityRepository->findAll();
			$project_TM = (isset($persons[$project->TM])) ? $persons[$project->TM]->getReversedFullNameWithEID() : '' . $project->TM;
			$project_PIPM = (isset($persons[$project->PIPM])) ? $persons[$project->PIPM]->getReversedFullNameWithEID() : '' . $project->PIPM;
			$project_PMOE = (isset($persons[$project->PMOE])) ? $persons[$project->PMOE]->getReversedFullNameWithEID() : '' . $project->PMOE;
			$project_PPaC = (isset($persons[$project->PPaC])) ? $persons[$project->PPaC]->getReversedFullNameWithEID() : '' . $project->PPaC;
			foreach ($tasks as $task)
            {
                $cellIterator = $rowIterator->current()->getCellIterator();

                $taskYearPlan = $this->context->taskPlanRepository->findTaskYearPlan($task->ID, $this->year);
                if ($taskYearPlan)
                {
                    $cellIterator->current()->setValue($project->KeyCode);
                    $cellIterator->next();

                    $cellIterator->current()->setValue($project->SAP_ID);
                    $cellIterator->next();

                    $cellIterator->current()->setValue($project->Name);
                    $cellIterator->next();

                    $cellIterator->current()->setValue($task->getSAPIDWithDash());
                    $cellIterator->next();

                    $cellIterator->current()->setValue($task->Name);
                    $cellIterator->next();

                    $cellIterator->current()->setValue($task->Category); // MATERIAL ,TRAVEL, LABOR, SUBCONTRACTING
                    $cellIterator->next();

                    $cellIterator->current()->setValue('P' . ($task->LabourTask ? 'L' : 'N')); // PN (Plan Nonlabor), PL (Plan Labor), AN (Actuals Nonlabor), AL (Actuals Labor), EL (???)
                    $cellIterator->next();

                    $cellIterator->current()->setValue($task->CostShare * 100);
                    $cellIterator->next();

                    $cellIterator->current()->setValue($task->CostCenterID);
                    $cellIterator->next();

                    $cellIterator->current()->setValue($project->PO);
                    $cellIterator->next();

                    $cellIterator->current()->setValue($project->TotalBudget);
                    $cellIterator->next();

                    $cellIterator->current()->setValue($project->State);
                    $cellIterator->next();

                    $cellIterator->current()->setValue($project_TM);
                    $cellIterator->next();

                    $cellIterator->current()->setValue($project_PIPM);
                    $cellIterator->next();

                    $cellIterator->current()->setValue($project_PMOE);
                    $cellIterator->next();

                    $cellIterator->current()->setValue($project_PPaC);
                    $cellIterator->next();

                    $cellIterator->current()->setValue(isset($customers[$project->Customer]) ? $customers[$project->Customer]->Name : $project->Customer);
                    $cellIterator->next();

                    $cellIterator->current()->setValue($project->Type);
                    $cellIterator->next();

                    $cellIterator->current()->setValue($project->Phase);
                    $cellIterator->next();

                    $cellIterator->current()->setValue(isset($responsibilities[$project->Responsibility]) ? $responsibilities[$project->Responsibility]->Name : $project->Responsibility);
                    $cellIterator->next();

                    $cellIterator->current()->setValue('');
                    $cellIterator->next();

                    $cellIterator->current()->setValue($taskYearPlan->Year);
                    $cellIterator->next();

                    $cellIterator->current()->setValue($actualsUntil);
                    $cellIterator->next();

                    foreach ($taskYearPlan->MonthPlans as $monthPlan)
                    {
                        if ($task->LabourTask)
                        {
                            // VLOOKUP Cost Center rate from CostCenter sheet, eg. =AJ2*VLOOKUP($I2;'Cost Centers'!$A$2:$N$11;3)
                            $cellIterator->current()->setValue('=' . $cellIterator->current()->stringFromColumnIndex($cellIterator->key() + 12) . $rowIterator->key() . '*VLOOKUP($I' . $rowIterator->key() . ',' . $this->costCentersSheetName . '!$A$2:$N$' . (1 + count($costCenters)) . ',3)');
                        }
                        else
                        {
                            $cellIterator->current()->setValue($monthPlan->Budget);
                        }
                        $cellIterator->next();
                    }

                    foreach ($taskYearPlan->MonthPlans as $monthPlan)
                    {
                        if ($task->LabourTask)
                        {
                            $cellIterator->current()->setValue($monthPlan->Hours);
                        }
                        $cellIterator->next();
                    }

                    if ($task->LabourTask)
                    {
                        $taskPlanner = new TaskResourceYearPlanner($this->context, $taskYearPlan);
                        foreach ($taskPlanner->getAssignedWorkerEACArray() as $eid => $assignedWorkerEAC)
                        {
                            $rowIterator->next();
                            $cellIterator = $rowIterator->current()->getCellIterator();

                            $cellIterator->current()->setValue($project->KeyCode);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project->SAP_ID);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project->Name);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($task->getSAPIDWithDash());
                            $cellIterator->next();

                            $cellIterator->current()->setValue($task->Name);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($task->Category); // MATERIAL ,TRAVEL, LABOR, SUBCONTRACTING
                            $cellIterator->next();

                            $cellIterator->current()->setValue('AL'); // Actuals Labor
                            $cellIterator->next();

                            $cellIterator->current()->setValue($task->CostShare * 100);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($task->CostCenterID);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project->PO);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project->TotalBudget);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project->State);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project_TM);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project_PIPM);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project_PMOE);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project_PPaC);
                            $cellIterator->next();

                            $cellIterator->current()->setValue(isset($customers[$project->Customer]) ? $customers[$project->Customer]->Name : $project->Customer);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project->Type);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project->Phase);
                            $cellIterator->next();

                            $cellIterator->current()->setValue(isset($responsibilities[$project->Responsibility]) ? $responsibilities[$project->Responsibility]->Name : $project->Responsibility);
                            $cellIterator->next();

                            $cellIterator->current()->setValue(isset($persons[$eid]) ? $persons[$eid]->getReversedFullNameWithEID() : $eid);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($taskYearPlan->Year);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($actualsUntil);
                            $cellIterator->next();

                            for ($month = 1; $month <= 12; $month++)
                            {
                                // VLOOKUP Cost Center rate from CostCenter sheet, eg. =AJ2*VLOOKUP($I2;'Cost Centers'!$A$2:$N$11;3)
                                $cellIterator->current()->setValue('=' . $cellIterator->current()->stringFromColumnIndex($cellIterator->key() + 12) . $rowIterator->key() . '*VLOOKUP($I' . $rowIterator->key() . ',' . $this->costCentersSheetName . '!$A$2:$N$' . (1 + count($costCenters)) . ',3)');
                                $cellIterator->next();
                            }

                            for ($month = 1; $month <= 12; $month++)
                            {
                                $cellIterator->current()->setValue($assignedWorkerEAC->offsetGet($month));
                                $cellIterator->next();
                            }
                        }
                        foreach ($taskPlanner->getUnassignedWorkersEACArray() as $eid => $unassignedWorkerEAC)
                        {
                            $rowIterator->next();
                            $cellIterator = $rowIterator->current()->getCellIterator();

                            $cellIterator->current()->setValue($project->KeyCode);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project->SAP_ID);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project->Name);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($task->getSAPIDWithDash());
                            $cellIterator->next();

                            $cellIterator->current()->setValue($task->Name);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($task->Category); // MATERIAL ,TRAVEL, LABOR, SUBCONTRACTING
                            $cellIterator->next();

                            $cellIterator->current()->setValue('AL'); // Actuals Labor
                            $cellIterator->next();

                            $cellIterator->current()->setValue($task->CostShare * 100);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($task->CostCenterID);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project->PO);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project->TotalBudget);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project->State);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project_TM);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project_PIPM);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project_PMOE);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project_PPaC);
                            $cellIterator->next();

                            $cellIterator->current()->setValue(isset($customers[$project->Customer]) ? $customers[$project->Customer]->Name : $project->Customer);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project->Type);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($project->Phase);
                            $cellIterator->next();

                            $cellIterator->current()->setValue(isset($responsibilities[$project->Responsibility]) ? $responsibilities[$project->Responsibility]->Name : $project->Responsibility);
                            $cellIterator->next();

                            $cellIterator->current()->setValue(isset($persons[$eid]) ? $persons[$eid]->getReversedFullNameWithEID() : $eid);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($taskYearPlan->Year);
                            $cellIterator->next();

                            $cellIterator->current()->setValue($actualsUntil);
                            $cellIterator->next();

                            for ($month = 1; $month <= 12; $month++)
                            {
                                // VLOOKUP Cost Center rate from CostCenter sheet, eg. =AJ2*VLOOKUP($I2;'Cost Centers'!$A$2:$N$11;3)
                                $cellIterator->current()->setValue('=' . $cellIterator->current()->stringFromColumnIndex($cellIterator->key() + 12) . $rowIterator->key() . '*VLOOKUP($I' . $rowIterator->key() . ',' . $this->costCentersSheetName . '!$A$2:$N$' . (1 + count($costCenters)) . ',3)');
                                $cellIterator->next();
                            }

                            for ($month = 1; $month <= 12; $month++)
                            {
                                $cellIterator->current()->setValue($unassignedWorkerEAC->offsetGet($month));
                                $cellIterator->next();
                            }
                        }
                    }
                    else
                    {
                        $actualsCosts = $this->context->actualsCostRepository->findAllForTask($task->ID);
                        if (isset($actualsCosts[$this->year]))
                        {
                            for ($month = 1; $month < 12; $month++)
                            {
                                if (isset($actualsCosts[$this->year][$month]))
                                {
                                    foreach ($actualsCosts[$this->year][$month] as $actualsCost)
                                    {
                                        $rowIterator->next();
                                        $cellIterator = $rowIterator->current()->getCellIterator();

                                        $cellIterator->current()->setValue($project->KeyCode);
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue($project->SAP_ID);
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue($project->Name);
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue($task->getSAPIDWithDash());
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue($task->Name);
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue($task->Category); // MATERIAL ,TRAVEL, LABOR, SUBCONTRACTING
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue('AN'); // Actuals Nonlabor
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue($task->CostShare * 100);
                                        $cellIterator->next();

                                        // no cost center
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue($project->PO);
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue($project->TotalBudget);
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue($project->State);
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue($project_TM);
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue($project_PIPM);
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue($project_PMOE);
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue($project_PPaC);
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue(isset($customers[$project->Customer]) ? $customers[$project->Customer]->Name : $project->Customer);
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue($project->Type);
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue($project->Phase);
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue(isset($responsibilities[$project->Responsibility]) ? $responsibilities[$project->Responsibility]->Name : $project->Responsibility);
                                        $cellIterator->next();

                                        // no person name (only for actuals)
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue($taskYearPlan->Year);
                                        $cellIterator->next();

                                        $cellIterator->current()->setValue($actualsUntil);
                                        $cellIterator->next();

                                        $monthsToSkip = $month - 1;
                                        while ($monthsToSkip--)
                                        {
                                            $cellIterator->next();
                                        }
                                        $cellIterator->current()->setValue($actualsCost->Value_usd);
                                        $cellIterator->next();

                                        $monthsToSkip = 12 + 12 - $month;
                                        while ($monthsToSkip--)
                                        {
                                            $cellIterator->next();
                                        }
                                        $cellIterator->current()->setValue($actualsCost->Description);
                                    }
                                }
                            }
                        }
                        $rowIterator->next();
                        $cellIterator = $rowIterator->current()->getCellIterator();

                        $cellIterator->current()->setValue($project->KeyCode);
                        $cellIterator->next();

                        $cellIterator->current()->setValue($project->SAP_ID);
                        $cellIterator->next();

                        $cellIterator->current()->setValue($project->Name);
                        $cellIterator->next();

                        $cellIterator->current()->setValue($task->getSAPIDWithDash());
                        $cellIterator->next();

                        $cellIterator->current()->setValue($task->Name);
                        $cellIterator->next();

                        $cellIterator->current()->setValue($task->Category); // MATERIAL ,TRAVEL, LABOR, SUBCONTRACTING
                        $cellIterator->next();

                        $cellIterator->current()->setValue('EN'); // ???
                        $cellIterator->next();

                        $cellIterator->current()->setValue($task->CostShare * 100);
                        $cellIterator->next();

                        // no cost center
                        $cellIterator->next();

                        $cellIterator->current()->setValue($project->PO);
                        $cellIterator->next();

                        $cellIterator->current()->setValue($project->TotalBudget);
                        $cellIterator->next();

                        $cellIterator->current()->setValue($project->State);
                        $cellIterator->next();

                        $cellIterator->current()->setValue($project_TM);
                        $cellIterator->next();

                        $cellIterator->current()->setValue($project_PIPM);
                        $cellIterator->next();

                        $cellIterator->current()->setValue($project_PMOE);
                        $cellIterator->next();

                        $cellIterator->current()->setValue($project_PPaC);
                        $cellIterator->next();

                        $cellIterator->current()->setValue(isset($customers[$project->Customer]) ? $customers[$project->Customer]->Name : $project->Customer);
                        $cellIterator->next();

                        $cellIterator->current()->setValue($project->Type);
                        $cellIterator->next();

                        $cellIterator->current()->setValue($project->Phase);
                        $cellIterator->next();

                        $cellIterator->current()->setValue(isset($responsibilities[$project->Responsibility]) ? $responsibilities[$project->Responsibility]->Name : $project->Responsibility);
                        $cellIterator->next();

                        // no person name (only for actuals)
                        $cellIterator->next();

                        $cellIterator->current()->setValue($taskYearPlan->Year);
                        $cellIterator->next();

                        $cellIterator->current()->setValue($actualsUntil);
                        $cellIterator->next();

                        $monthsToSkip = $fiscalMonth;
                        foreach ($taskYearPlan->MonthPlans as $monthPlan)
                        {
                            if ($monthsToSkip-- <= 0)
                            {
                                $cellIterator->current()->setValue($monthPlan->Budget);
                            }
                            $cellIterator->next();
                        }

                    }

                    $rowIterator->next();

                }

            }
            
        }
        
    }
    
    public function saveTemplate($fileType = 'Excel5')
    {
        
        header('Content-Type: application/vnd.ms-excel');
        header('Content-Disposition: attachment;filename="tracking-table.xls"');
        header('Cache-Control: max-age=0');
        
        $objWriter = PHPExcel_IOFactory::createWriter($this->objPHPExcel, $fileType);
        $objWriter->save('php://output');
        
    }
    
}

class TrackingTableWrapperException extends Exception {}
